####################################################################################
#
# File name:		Call_MR_coupled
# Version:			0.1
# Software:		    
# Authors:			Zhifei Wang, Jiang Xu
# Website:			http://www.ece.ust.hk/~eexu/
# The copyright information of this program can be found in the file COPYRIGHT.
#
####################################################################################
import numpy as np
import Cal_Matrix as cm
import os

def MR_serial_coupled(lam, kappa, t, L, alpha0, delta_Ne, delta_Nh, NumMR, Power_flag = 1, dB_flag = 1,
                      debug_flag = 0, T_in = np.array([1.+0j]), T_add = np.array([0j]), FileName = 'Output'):
    Num = len(lam)
    if len(T_in) == 1:
        T_in = T_in*np.ones(Num)
    if len(T_add) == 1:
        T_add = T_add*np.ones(Num)
    ''' Initialization '''
    T_through = np.zeros(Num, dtype=complex)
    T_drop = np.zeros(Num, dtype=complex)
    T_through_on = np.zeros(Num, dtype=complex)
    T_drop_on = np.zeros(Num, dtype=complex)
    # FileName = "SpectrumData"
    ''' Calculation for each wavelength '''
    for iter_wl in range(0, Num):
        ''' When MR is off-state, no carrier '''
        # Note that the first element of p is useless !!!
        p = np.append(np.ones(1), cm.Calp(lam[iter_wl], L, alpha0))
        # p = Calp(lam[iter_wl], L, alpha0)
        # p[1] = Calp_scalar(lam[iter_wl], 1.25*L, alpha0)      # evaluate vernier effect !! OK !
        T = cm.CalTransferMatrix(kappa, t, p, NumMR)*np.matrix([[T_in[iter_wl]], [T_add[iter_wl]]])
        T_through[iter_wl] = T[0, 0]
        T_drop[iter_wl] = T[1, 0]

        ''' When MR is on-state, carrier injected '''
        p_on = np.append(np.ones(1), cm.Calp(lam[iter_wl], L, alpha0, delta_Ne, delta_Nh))
        # p_on = Calp(lam[iter_wl], L, alpha0, delta_Ne, delta_Nh)
        T_on = cm.CalTransferMatrix(kappa, t, p_on, NumMR) * np.matrix([[T_in[iter_wl]], [T_add[iter_wl]]])
        T_through_on[iter_wl] = T_on[0, 0]
        T_drop_on[iter_wl] = T_on[1, 0]
    Power_through = np.power(np.absolute(T_through), 2)
    Power_drop = np.power(np.absolute(T_drop), 2)
    Power_through_on = np.power(np.absolute(T_through_on), 2)
    Power_drop_on = np.power(np.absolute(T_drop_on), 2)

    ''' For debugging, verify the result when there is only one MR '''
    if debug_flag == 1:
        T_through_test = np.zeros(Num, dtype=complex)   # for debugging
        T_drop_test = np.zeros(Num, dtype=complex)      # for debugging
        for iter_wl in range(0, Num):
            p_scalar = cm.Calp(lam[iter_wl], L[0], alpha0)
            (T_through_test[iter_wl], T_drop_test[iter_wl]) = cm.CalOneMR(0.28j, t[0], p_scalar)  # for debugging
        Power_through_test = np.power(np.absolute(T_through_test), 2)   # for debugging
        Power_drop_test = np.power(np.absolute(T_drop_test), 2)         # for debugging
        return (Power_through, Power_drop, Power_through_test, Power_drop_test)

    ''' Return calculation result '''
    if Power_flag == 1:
        if dB_flag == 0:
            if FileName != '':
                data = (lam, Power_through, Power_drop, Power_through_on, Power_drop_on)
                # np.savetxt(FileName, data)
                np.save(FileName, data)
            return (Power_through, Power_drop, Power_through_on, Power_drop_on)
        if dB_flag == 1:
            Power_through_dB = 10 * np.log10(Power_through)
            Power_drop_dB = 10 * np.log10(Power_drop)
            Power_through_on_dB = 10 * np.log10(Power_through_on)
            Power_drop_on_dB = 10 * np.log10(Power_drop_on)
            if FileName != '':
                PathName = 'Data/'
                if not os.path.exists(PathName):
                    os.makedirs(PathName)
                data = (lam, Power_through_dB, Power_drop_dB, Power_through_on_dB, Power_drop_on_dB)
                # np.savetxt(FileName, data)
                np.save(PathName+FileName, data)
            return (Power_through_dB, Power_drop_dB, Power_through_on_dB, Power_drop_on_dB)
    else:
        if FileName != '':
            PathName = 'Data/'
            if not os.path.exists(PathName):
                os.makedirs(PathName)
            data = (lam, T_through, T_drop, T_through_on, T_drop_on)
            # np.savetxt(FileName, data)
            np.save(PathName+FileName, data)
        return (T_through, T_drop, T_through_on, T_drop_on)


def MR_serial_parallel_coupled(lam, kappa, t, alpha0, delta_Ne, delta_Nh,
                            L, delta_perimeter, MR_spacing, NumMR_s, NumMR_p,
                            Power_flag = True, dB_flag = True,
                            T_in = np.array([1.+0j]), T_add = np.array([0j]), FileName = 'Output'):
    Num = len(lam)
    ''' make input easier to use, convert scalar to vector '''
    if len(T_in) == 1:
        T_in = T_in*np.ones(Num)
    if len(T_add) == 1:
        T_add = T_add*np.ones(Num)
    ''' Initialization '''
    T_through = np.zeros(Num, dtype=complex)
    T_drop = np.zeros(Num, dtype=complex)
    T_through_on = np.zeros(Num, dtype=complex)
    T_drop_on = np.zeros(Num, dtype=complex)
    # FileName = "SpectrumData"
    ''' Calculation for each wavelength '''
    for iter_wl in range(0, Num):
        ''' When MR is off-state, no carrier '''
        # Note that the first element of p is useless !!!
        Parallel_Matrix = cm.CalParallelTransferMatrix(lam[iter_wl], kappa, t, alpha0, 0, 0, L,
                                   delta_perimeter, MR_spacing, NumMR_s, NumMR_p)
        T = Parallel_Matrix*np.matrix([[T_in[iter_wl]], [T_add[iter_wl]]])
        T_through[iter_wl] = T[0, 0]
        T_drop[iter_wl] = T[1, 0]

        ''' When MR is on-state, carrier injected '''
        Parallel_Matrix_on = cm.CalParallelTransferMatrix(lam[iter_wl], kappa, t, alpha0,
                              delta_Ne, delta_Nh, L, delta_perimeter, MR_spacing, NumMR_s, NumMR_p)
        T_on = Parallel_Matrix_on * np.matrix([[T_in[iter_wl]], [T_add[iter_wl]]])
        T_through_on[iter_wl] = T_on[0, 0]
        T_drop_on[iter_wl] = T_on[1, 0]
    Power_through = np.power(np.absolute(T_through), 2)
    Power_drop = np.power(np.absolute(T_drop), 2)
    Power_through_on = np.power(np.absolute(T_through_on), 2)
    Power_drop_on = np.power(np.absolute(T_drop_on), 2)

    ''' Return calculation result '''
    if Power_flag:
        if not dB_flag:
            data = (lam, Power_through, Power_drop, Power_through_on, Power_drop_on)
            # np.savetxt(FileName, data)
            np.save(FileName, data)
            return (Power_through, Power_drop, Power_through_on, Power_drop_on)
        if dB_flag:
            Power_through_dB = 10 * np.log10(Power_through)
            Power_drop_dB = 10 * np.log10(Power_drop)
            Power_through_on_dB = 10 * np.log10(Power_through_on)
            Power_drop_on_dB = 10 * np.log10(Power_drop_on)
            PathName = 'Data/'
            if not os.path.exists(PathName):
                os.makedirs(PathName)
            data = (lam, Power_through_dB, Power_drop_dB, Power_through_on_dB, Power_drop_on_dB)
            # np.savetxt(FileName, data)
            np.save(PathName+FileName, data)
            return (Power_through_dB, Power_drop_dB, Power_through_on_dB, Power_drop_on_dB)
    else:
        PathName = 'Data/'
        if not os.path.exists(PathName):
            os.makedirs(PathName)
        data = (lam, T_through, T_drop, T_through_on, T_drop_on)
        # np.savetxt(FileName, data)
        np.save(PathName+FileName, data)
        return (T_through, T_drop, T_through_on, T_drop_on)