####################################################################################
#
# File name:		FuncInt
# Version:			0.1
# Software:		    
# Authors:			Zhifei Wang, Jiang Xu
# Website:			http://www.ece.ust.hk/~eexu/
# The copyright information of this program can be found in the file COPYRIGHT.
#
####################################################################################
import numpy as np
import math
from joblib import Parallel, delayed

def Multiply(x, y):
    return x*y

def dB2Ratio(x):
    return 10**(x/10)

def Ratio2dB(x):
    return 10 * math.log(x,10)

def MostClose(x, x0):
    (value, index) = min((v, i) for i, v in enumerate(abs(x-x0)))
    return (value, index)

def CalLossXtalk(DataFileName, JobNum, lam0, BR):
    c0 = 3e8       # m/s, nm/ns
    f0 = c0/(lam0*1e-6)     #Hz
    lam, Power_through_dB, Power_drop_dB, Power_through_on_dB, Power_drop_on_dB = np.load(DataFileName)
    Power_through    = Parallel(n_jobs=JobNum)(delayed(dB2Ratio)(x) for x in Power_through_dB)
    Power_drop       = Parallel(n_jobs=JobNum)(delayed(dB2Ratio)(x) for x in Power_drop_dB)
    Power_through_on = Parallel(n_jobs=JobNum)(delayed(dB2Ratio)(x) for x in Power_through_on_dB)
    Power_drop_on    = Parallel(n_jobs=JobNum)(delayed(dB2Ratio)(x) for x in Power_drop_on_dB)
    Num = len(lam)
    step_f = c0 / (lam0 ** 2) * (lam[1] - lam[0]) / 1e-6  # Hz
    T0 = 1 / (BR * 1e9)  # s
    f = c0 / (lam * 1e-6)  # Hz
    ''' The power spectrum density of the NRZ is
    S(f) = T/4*sinc(fT)**2 + 1/4*delta(f), where f = c/lam '''
    # LaserPSD0 = T0/4*np.sinc((c0/(lam*1e-6)-c0/(lam0*1e-6)) * T0)**2
    LaserPSD = T0 / 4 * np.sinc((f - f0) * T0) ** 2
    Power_drop0 = Power_drop[np.argmin(abs(lam-lam0))]
    Power_drop_on0 = Power_drop_on[np.argmin(abs(lam - lam0))]
    Power_through0 = Power_through[np.argmin(abs(lam - lam0))]
    Power_through_on0 = Power_through_on[np.argmin(abs(lam - lam0))]

    Crosstalk_drop0 = Parallel(n_jobs=JobNum)(delayed(Multiply)(Power_drop[i], LaserPSD[i])
                                      for i in range(Num))
    Crosstalk_drop = sum(Crosstalk_drop0) * step_f + 1/4*Power_drop0
    Crosstalk_through0 = Parallel(n_jobs=JobNum)(delayed(Multiply)(Power_through_on[i], LaserPSD[i])
                                              for i in range(Num))
    Crosstalk_through = sum(Crosstalk_through0) * step_f + 1 / 4 * Power_through_on0

    InsertionLoss0 = Parallel(n_jobs=JobNum)(delayed(Multiply)(Power_drop_on[i], LaserPSD[i])
                                      for i in range(Num))
    InsertionLoss = sum(InsertionLoss0) * step_f + 1 / 4 * Power_drop_on0
    PassingLoss0 = Parallel(n_jobs=JobNum)(delayed(Multiply)(Power_through[i], LaserPSD[i])
                                      for i in range(Num))
    PassingLoss = sum(PassingLoss0) * step_f + 1 / 4 * Power_through0

    ''' we calculate the peak value!!! '''
    InsertionLoss = InsertionLoss * 2
    PassingLoss = PassingLoss * 2


    return InsertionLoss, PassingLoss, Crosstalk_drop, Crosstalk_through


# def CalCrosstalk_test(test_lam, test_power, JobNum, lam0, BR):
#     c0 = 3e8       # m/s, nm/ns
#     f0 = c0/(lam0*1e-6)     # Hz
#     Num = len(test_lam)
#     step_f = c0/(lam0**2)*(test_lam[1]-test_lam[0])/1e-6    # Hz
#     T0 = 1/(BR * 1e9)   # s
#     f = c0/(test_lam * 1e-6)    # Hz
#     ''' The power spectrum density of the NRZ is
#     S(f) = T/4*sinc(fT)**2+1/4*delta(f), where f = c/lam '''
#     LaserPSD0 = T0/4*np.sinc((f-f0) * T0)**2
#     # Results = Parallel(n_jobs=JobNum)(delayed(lambda x, y: x*y)(Power_drop_on_dB[i], LaserPSD0[i])
#     #                           for i in range(Num))
#     Result0 = Parallel(n_jobs=JobNum)(delayed(Multiply)(test_power[i], LaserPSD0[i])
#                                       for i in range(Num))
#     Result = sum(Result0) * step_f + 1/4
#     return Result




if __name__ == "__main__":
    PathName = 'Data/'
    DataFileName = '1-parallel-coupled_1-single-MRs.npy'
    # DataFileName = 'test.npy'
    # BR = 10    # Gbps
    BR = 0.1  # 500 Mbps
    lam0 = 1.55069
    JobNum = 1
    InsertionLoss, PassingLoss, Crosstalk_drop, Crosstalk_through = \
        CalLossXtalk(PathName+DataFileName, JobNum, lam0, BR)

    # test_lam = np.arange(1.53, 1.57, 0.01)
    # test_power = np.array([0.2, 0.5, 1, 0.5, 0.2])
    # Crosstalk_drop = CalCrosstalk_test(test_lam, test_power, JobNum, lam0, BR)

    InsertionLoss_dB = Ratio2dB(InsertionLoss)
    PassingLoss_dB = Ratio2dB(PassingLoss)
    Crosstalk_drop_dB = Ratio2dB(Crosstalk_drop)
    Crosstalk_through_dB = Ratio2dB(Crosstalk_through)
    print(InsertionLoss_dB, PassingLoss_dB, Crosstalk_drop_dB, Crosstalk_through_dB)
    print(InsertionLoss_dB-Crosstalk_drop_dB)
    
def CalXtalk_forC(JobNum, lam0, BR):
    PathName = 'Data/'
    DataFileName = '1-parallel-coupled_1-single-MRs.npy'
    InsertionLoss, PassingLoss, Crosstalk_drop, Crosstalk_through = \
        CalLossXtalk(PathName+DataFileName, JobNum, lam0, BR)

    # test_lam = np.arange(1.53, 1.57, 0.01)
    # test_power = np.array([0.2, 0.5, 1, 0.5, 0.2])
    # Crosstalk_drop = CalCrosstalk_test(test_lam, test_power, JobNum, lam0, BR)

    InsertionLoss_dB = Ratio2dB(InsertionLoss)
    PassingLoss_dB = Ratio2dB(PassingLoss)
    Crosstalk_drop_dB = Ratio2dB(Crosstalk_drop)
    Crosstalk_through_dB = Ratio2dB(Crosstalk_through)
    print(Crosstalk_drop_dB, Crosstalk_through_dB)
    return (Crosstalk_drop_dB, Crosstalk_through_dB)
 

