####################################################################################
#
# File name:		Top_serial_coupled_MR_group
# Version:			0.1
# Software:		    
# Authors:			Zhifei Wang, Jiang Xu
# Website:			http://www.ece.ust.hk/~eexu/
# The copyright information of this program can be found in the file COPYRIGHT.
#
####################################################################################
import numpy as np
import os
import time
import Cal_MR_coupled as cmc
import SpectrumAnalysis as sa
import matplotlib.pyplot as plt

if __name__ == "__main__":
    start_time = time.time()
    ''' Basic Parameters '''
    Num = int(8e3)
    # Num = int(8e2)
    # lam = np.linspace(1.53, 1.568, Num)
    # lam = np.linspace(1.5525, 1.556, Num)
    lam = np.linspace(1.547, 1.553, Num)
    ''' MR Parameters '''
    # for NumMR_p in range(4,5):
    #     for NumMR_s in range(3,4):
    for NumMR_p in range(1,2):
        for NumMR_s in range(1,2):
            R = np.ones(NumMR_s)*20
            L = 2*np.pi*R
            delta_perimeter = 165e-3        # um\
            # delta_perimeter = 0        # um
            MR_spacing = 0.58    # um
            propagation_loss = 2 # dB/cm
            (delta_Ne, delta_Nh) = (3e17, 3e17)
            # (kappa1, kappa2, kappa3) = (0.28j, 0.28j, 0.06j)
            (kappa1, kappa2, kappa3) = (0.3j, 0.3j, 0.06j)

            # alpha = 0.5*propagation_loss*math.log(10)*1e-5
            # t2 = (1-abs(kappa2)**2)**0.5
            # t1 = t2*math.exp(alpha*L)
            # kappa1 = 1j*(1-abs(t1)**2)**0.5
            ''' kappa1 and kappa2 are outer coupling coefficient,
            and kappa3 is the inner coupling coefficient '''

            ''' Basic Assignment'''
            alpha0 = 0.5*propagation_loss*np.log(10)*1e-5   # Unit: 1/um
            kappa = np.ones(NumMR_s+1)*kappa3
            kappa[0] = kappa1
            kappa[-1]= kappa2
            t = np.power(1-np.power(np.absolute(kappa),2), 0.5)
            if NumMR_s == 1:
                file_name = str(NumMR_p) + '-parallel-coupled_' + str(NumMR_s) + '-single-MRs' \
                    # + '_heavydoping'
            elif NumMR_p == 1:
                file_name = str(NumMR_s) + '-single-MRs' \
                    # + '_heavydoping'
            else:
                file_name = str(NumMR_p) + '-parallel-coupled_' + str(NumMR_s) + '-serial-coupled-MRs' \
                        # + '_heavydoping'
            # file_name = 'Vernier_Effect_1.5'

            # file_name = 'test'
            ''' Calculation '''
            (Power_through, Power_drop, Power_through_on, Power_drop_on) = \
                cmc.MR_serial_parallel_coupled(lam, kappa, t, alpha0, delta_Ne, delta_Nh,
                               L, delta_perimeter, MR_spacing, NumMR_s, NumMR_p, FileName=file_name)



        sa.SpectrumAnalysis(lam*1e3, Power_through, Power_drop, Power_through_on, Power_drop_on, SpacingEstimation=4,
                         delta_dB=10, FileName=file_name)
        # SpectrumAnalysis(lam*1e3, Power_through_on, Power_drop_on, Power_through, Power_drop, FSR_estimate=10,
        #                delta_dB=10, FileName=file_name+'_depletion')

        ''' Plot '''
        plt.figure(facecolor="w")
        plt.plot(lam*1e3, Power_through, color="b", linewidth=1)
        plt.plot(lam*1e3, Power_drop, color="c", linestyle=":", linewidth=2)
        plt.plot(lam*1e3, Power_through_on, color="g", linestyle=":", linewidth=2)
        plt.plot(lam*1e3, Power_drop_on, color="r", linewidth=1)
        plt.xlabel("Wavelength (nm)")
        plt.ylabel("Power (dB)")
        plt.title("Spectrum")
        # plt.xlim(1.5, 1.6)
        # plt.ylim(-80,0)
        # plt.legend()

        ''' Save the figure '''
        PathName = 'Graphics/'
        if not os.path.exists(PathName):
            os.makedirs(PathName)
        plt.savefig(PathName+file_name+'.tif', dpi=800, bbox_inches='tight')
        plt.savefig(PathName+file_name+'.pdf', dpi=800, bbox_inches='tight')

    print("--- Running time is {0:.3f} seconds ---".format(time.time() - start_time))
    plt.show()
    # return 0



def CalData_forC():
    # print("Now is CalData_forC")
    Num = int(8e3)
    ''' Basic Parameters '''
    # Num = int(8e3)
    # Num = int(8e2)
    # lam = np.linspace(1.53, 1.568, Num)
    # lam = np.linspace(1.5525, 1.556, Num)
    lam = np.linspace(1.547, 1.553, Num)
    ''' MR Parameters '''
    # for NumMR_p in range(4,5):
    #     for NumMR_s in range(3,4):
    for NumMR_p in range(1,2):
        for NumMR_s in range(1,2):
            R = np.ones(NumMR_s)*20
            L = 2*np.pi*R
            delta_perimeter = 165e-3        # um\
            # delta_perimeter = 0        # um
            MR_spacing = 0.58    # um
            propagation_loss = 2 # dB/cm
            (delta_Ne, delta_Nh) = (3e17, 3e17)
            # (kappa1, kappa2, kappa3) = (0.28j, 0.28j, 0.06j)
            (kappa1, kappa2, kappa3) = (0.3j, 0.3j, 0.06j)

            # alpha = 0.5*propagation_loss*math.log(10)*1e-5
            # t2 = (1-abs(kappa2)**2)**0.5
            # t1 = t2*math.exp(alpha*L)
            # kappa1 = 1j*(1-abs(t1)**2)**0.5
            ''' kappa1 and kappa2 are outer coupling coefficient,
            and kappa3 is the inner coupling coefficient '''

            ''' Basic Assignment'''
            alpha0 = 0.5*propagation_loss*np.log(10)*1e-5   # Unit: 1/um
            kappa = np.ones(NumMR_s+1)*kappa3
            kappa[0] = kappa1
            kappa[-1]= kappa2
            t = np.power(1-np.power(np.absolute(kappa),2), 0.5)
            if NumMR_s == 1:
                file_name = str(NumMR_p) + '-parallel-coupled_' + str(NumMR_s) + '-single-MRs' \
                    # + '_heavydoping'
            elif NumMR_p == 1:
                file_name = str(NumMR_s) + '-single-MRs' \
                    # + '_heavydoping'
            else:
                file_name = str(NumMR_p) + '-parallel-coupled_' + str(NumMR_s) + '-serial-coupled-MRs' \
                        # + '_heavydoping'
            # file_name = 'Vernier_Effect_1.5'

            # file_name = 'test'
            ''' Calculation '''
            (Power_through, Power_drop, Power_through_on, Power_drop_on) = \
                cmc.MR_serial_parallel_coupled(lam, kappa, t, alpha0, delta_Ne, delta_Nh,
                               L, delta_perimeter, MR_spacing, NumMR_s, NumMR_p, FileName=file_name)
    # return 0