/*********************************************************************************
 *
 * File name:		call_python
 * Version:			0.1
 * Software:		This function is used to call python script to calculate crosstalk
 * Authors:			Zhifei Wang, Jiang Xu
 * Website:			http://www.ece.ust.hk/~eexu/
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include </home/luan/anaconda3/include/python3.6m/Python.h>
#include <stdlib.h>  

int call_pyfunc(double *Xtalk, int jobnum, int lam, int BR, bool HasData=true)
/* Xtalk is the crosstalk, which should be initiated as a array including two double variable. The first one is the crosstalk at drop port and the second one is the crosstalk at through port.
jobnum is the number of the parallel jobs you want to excute, you can always set as 1.
lam is the target wavelength.
BR is the target bit rate.
HasData is the flag showing whether there is MR spectrum data. True represents that there is such data, and false represents no. By the way, it should be noted that the data should be under the path of "./Data" and named as "1-parallel-coupled_1-single-MRs.np". If you want to customize the data path and name, you should change related setting in "Top_serial_coupled_MR_group.py". */
{	
    PyObject *pName, *pModule, *pDict, *pFunc;
    PyObject *pArgs, *pValue, *p_value1, *p_value2;
    // int i=10;
	
	// setenv("PYTHONHOME", "/home/zhifei/App/anaconda3", 1);
	// setenv("PYTHONPATH", ".", 1);
	
    Py_Initialize();
	
	// char* PHOME[20] = "/home/zhifei/App/anaconda3"
	// char* PPATH[20] = "."
	// PyRun_SimpleString("import sys");
	// PyRun_SimpleString("import os");
	// PyRun_SimpleString("PHOME = \"/home/zhifei/App/anaconda3\"");
	// PyRun_SimpleString("PPATH = \".\"");
	// PyRun_SimpleString("os.environ['PYTHONHOME'] = PHOME");
	// PyRun_SimpleString("os.environ['PYTHONPATH'] = PPATH");
	
	
	// printf("%d\t%d\n", i, HasData);
	if (!HasData){
		pName = PyUnicode_DecodeFSDefault("Top_serial_coupled_MR_group");
		/* Error checking of pName left out */
		pModule = PyImport_Import(pName);
		// pModule = PyImport_ReloadModule(pName);
		
		Py_DECREF(pName);
		// printf("%d\n", pModule != NULL);
		if (pModule != NULL) {
			// printf("1\n");
			pFunc = PyObject_GetAttrString(pModule, "CalData_forC");
			/* pFunc is a new reference */

			if (pFunc && PyCallable_Check(pFunc)) {
				// printf("2\n");
				pArgs = PyTuple_New(0);
				pValue = PyObject_CallObject(pFunc, pArgs);
				Py_DECREF(pArgs);
				if (pValue != NULL) {
					// printf("success!!");
					Py_DECREF(pValue);
				}
				else {
					Py_DECREF(pFunc);
					Py_DECREF(pModule);
					PyErr_Print();
					fprintf(stderr,"Call failed\n");
					// return -1;
				}
			}
			else {
				// if (PyErr_Occurred())
					// PyErr_Print();
				fprintf(stderr, "Cannot find function CalData_forC!! \n");
				// return -2;
			}
			Py_XDECREF(pFunc);
			Py_DECREF(pModule);
		}
		else {
			// PyErr_Print();
			fprintf(stderr, "Failed to load Top_serial_coupled_MR_group \n");
			return -3;
		}
	}
	
	
	
	
    pName = PyUnicode_DecodeFSDefault("FuncInt");
    /* Error checking of pName left out */
    pModule = PyImport_Import(pName);
    Py_DECREF(pName);
    if (pModule != NULL) {
        pFunc = PyObject_GetAttrString(pModule, "CalXtalk_forC");
        /* pFunc is a new reference */

        if (pFunc && PyCallable_Check(pFunc)) {
            pArgs = PyTuple_New(3);
			PyTuple_SetItem(pArgs, 0, PyLong_FromLong(jobnum));
			PyTuple_SetItem(pArgs, 1, PyLong_FromLong(lam));
			PyTuple_SetItem(pArgs, 2, PyLong_FromLong(BR));
            pValue = PyObject_CallObject(pFunc, pArgs);
			p_value1 = PyTuple_GetItem(pValue, 0);
			p_value2 = PyTuple_GetItem(pValue, 1);
            Py_DECREF(pArgs);
            if (pValue != NULL) {
                printf("Result 1 of call: %f\n", PyFloat_AsDouble(p_value1));
				printf("Result 2 of call: %f\n", PyFloat_AsDouble(p_value2));
				Xtalk[0] = PyFloat_AsDouble(p_value1);
				Xtalk[1] = PyFloat_AsDouble(p_value2);
                Py_DECREF(pValue);
            }
            else {
                Py_DECREF(pFunc);
                Py_DECREF(pModule);
                PyErr_Print();
                fprintf(stderr,"Call failed\n");
                return -1;
            }
        }
        else {
            // if (PyErr_Occurred())
                // PyErr_Print();
            fprintf(stderr, "Cannot find function CalXtalk_forC!! \n");
			return -2;
        }
        Py_XDECREF(pFunc);
        Py_DECREF(pModule);
    }
    else {
        // PyErr_Print();
        // fprintf(stderr, "Failed to load \"%s\"\n", argv[1]);
        return -3;
    }
    Py_Finalize();
    return 0;
}
