/*********************************************************************************
 *
 * File name:       basic_elements.h
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Mahdi Nikdast (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#ifndef basic_elements
#define basic_elements

//Basic optical elements are microresonators, ports, waveguide crossings, waveguide bendings, optical terminators, and waveguide.

class MR{
	int MR_status; //1 means ON and 0 means OFF
	int MR_num;
public:
	MR(){MR_status=0;}
	void set_MR_status(int x){MR_status=x;}
	void set_MR_num(int x){MR_num=x;}
	int get_MR_status(){return MR_status;}
	int get_MR_num(){return MR_num;}
};

class Port{
	static const int type=1; 
	int next;
	int prev;
	double pwr;
public:
	Port(){next=prev=-1;}
	void set_prev(int x){prev=x;}
	void set_next(int x){next=x;}
	void set_pwr(double x){pwr=x;}
	int get_next(){return next;}
	int get_prev(){return prev;}
	double get_pwr(){return pwr;}
};

class Crossing{
	static const int type=2;
	int west,north,east,south;
	double w_pwr,n_pwr,e_pwr,s_pwr;
public:
	Crossing(){west=north=east=south=-1;}
	void set_west(int x){west=x;}
	void set_south(int x){south=x;}
	void set_north(int x){north=x;}
	void set_east(int x){east=x;}
	void set_w_pwr(double x){w_pwr=x;}
	void set_n_pwr(double x){n_pwr=x;}
	void set_e_pwr(double x){e_pwr=x;}
	void set_s_pwr(double x){s_pwr=x;}
	int get_west(){return west;}
	int get_north(){return north;}
	int get_east(){return east;}
	int get_south(){return south;}
	double get_w_pwr(){return w_pwr;}
	double get_n_pwr(){return n_pwr;}
	double get_e_pwr(){return e_pwr;}
	double get_s_pwr(){return s_pwr;}
};

class Bending{
	static const int type=3;
	int in,out;
	double in_pwr,out_pwr;
public:
	Bending(){in=out=-1;}
	void set_in(int x){in=x;}
	void set_out(int x){out=x;}
	void set_in_pwr(double x){in_pwr=x;}
	void set_out_pwr(double x){out_pwr=x;}
	int get_in(){return in;}
	int get_out(){return out;}
	double get_in_pwr(){return in_pwr;}
	double get_out_pwr(){return out_pwr;}
};

class Terminator{
	static const int type=4;
	int in;
	double in_pwr,out_pwr;
public:
	Terminator(){in=-1;}
	void set_in(int x){in=x;}
	void set_in_pwr(double x){in_pwr=x;}
	void set_out_pwr(double x){out_pwr=x;}
	int get_in(){return in;}
	double get_in_pwr(){return in_pwr;}
	double get_out_pwr(){return out_pwr;}
};

class OPN{
	static const int type=11;
	int in;
	double in_pwr,out_pwr;
public:
	OPN(){in=-1;}
	void set_in(int x){in=x;}
	void set_in_pwr(double x){in_pwr=x;}
	void set_out_pwr(double x){out_pwr=x;}
	int get_in(){return in;}
	double get_in_pwr(){return in_pwr;}
	double get_out_pwr(){return out_pwr;}
};

class CSE{
	static const int type=5;
	int west,north,east,south;
	double w_pwr,n_pwr,e_pwr,s_pwr;
	int CSE_MR_Loc;
	MR CSE_MR;
public:
	CSE(){west=north=east=south=-1;}
	void set_west(int x){west=x;}
	void set_north(int x){north=x;}
	void set_east(int x){east=x;}
	void set_south(int x){south=x;}
	void set_CSE_MR_Loc(int x){CSE_MR_Loc=x;}
	void set_CSE_MR(MR m){CSE_MR.set_MR_status(m.get_MR_status());}
	void set_w_pwr(double x){w_pwr=x;}
	void set_n_pwr(double x){n_pwr=x;}
	void set_e_pwr(double x){e_pwr=x;}
	void set_s_pwr(double x){s_pwr=x;}
	void set_CSE_MR_num(int x){CSE_MR.set_MR_num(x);}
	int get_west(){return west;}
	int get_north(){return north;}
	int get_east(){return east;}
	int get_south(){return south;}
	int get_CSE_MR_Loc(){return CSE_MR_Loc;}
	int get_CSE_MR(){return CSE_MR.get_MR_status();}
	double get_w_pwr(){return w_pwr;}
	double get_n_pwr(){return n_pwr;}
	double get_e_pwr(){return e_pwr;}
	double get_s_pwr(){return s_pwr;}
	int get_CSE_MR_num(){return CSE_MR.get_MR_num();}
};

class PSE{
	static const int type=6;
	int input,drop,through,add;
	double in_pwr,d_pwr,t_pwr,ad_pwr;
	MR PSE_MR;
public:
	PSE(){input=drop=through=add=-1;}
	void set_input(int x){input=x;}
	void set_drop(int x){drop=x;}
	void set_thorugh(int x){through=x;}
	void set_add(int x){add=x;}
	void set_PSE_MR(MR m){PSE_MR.set_MR_status(m.get_MR_status());}
	void set_in_pwr(double x){in_pwr=x;}
	void set_d_pwr(double x){d_pwr=x;}
	void set_t_pwr(double x){t_pwr=x;}
	void set_ad_pwr(double x){ad_pwr=x;}
	void set_PSE_MR_num(int x){PSE_MR.set_MR_num(x);}
	int get_input(){return input;}
	int get_drop(){return drop;}
	int get_through(){return through;}
	int get_add(){return add;}
	int get_PSE_MR(){return PSE_MR.get_MR_status();}
	double get_in_pwr(){return in_pwr;}
	double get_d_pwr(){return d_pwr;}
	double get_t_pwr(){return t_pwr;}
	double get_ad_pwr(){return ad_pwr;}
	int get_PSE_MR_num(){return PSE_MR.get_MR_num();}
};

class Waveguide{
	static const int type=7;
	double length;
	int in,out;
	double in_pwr,out_pwr;
public:
	Waveguide(){length=0;}
	void set_length(double x){length=x;}
	void set_in(int x){in=x;}
	void set_out(int x){out=x;}
	void set_in_pwr(double x){in_pwr=x;}
	void set_out_pwr(double x){out_pwr=x;}
	double get_length(){return length;}
	int get_in(){return in;}
	int get_out(){return out;}
	double get_in_pwr(){return in_pwr;}
	double get_out_pwr(){return out_pwr;}
};
//the Modulator is at the start of the communication line, for details please see the document
class Modulator{
	static const int type=9;
	double in_pwr;//from the laser source
	int num_mod_Mrs;//number of modulating MRs
public:
	void set_in_pwr(double x){in_pwr=x;}
	void set_num_mod_Mrs(int x){num_mod_Mrs=x;}
	double get_in_pwr(){return in_pwr;}
	int get_num_mod_Mrs(){return num_mod_Mrs;}
};
//the detector is at the end of the communication line, for details please see the document
class Detector{
	static const int type=10;
	double in_pwr;
	double photodetector_sens;//the sensitivity of the photodetector
	int num_detecting_Mrs;//number of detectign MRs
public:
	void set_int_pwr(double x){in_pwr=x;}
	void set_photodetector_sens(double x){photodetector_sens=x;}
	void set_num_detecting_Mrs(int x){num_detecting_Mrs=x;}
	double get_int_pwr(){return in_pwr;}
	double get_photodetector_sens(){return photodetector_sens;}
	int get_num_detecting_Mrs(int x){return num_detecting_Mrs;}
};

#endif