/*********************************************************************************
/*********************************************************************************
 *
 * File name:       main.cpp
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include "network_structure.h"
#include "router_function.h"
#include <cstdlib>
#include <string>
using namespace std;
void ftree_snr_function();
void find_src_des(string str,int& src_x,int& src_y,int& des_x,int& des_y);

int main(int argc, char* argv[])
{	
	//system("clear");
	if(argc!=3)//there is a problem in entering arguments
	{
		cout<<"\nERROR: We expect three arguments! Please see the manual for further information.\n";
		exit(1);
	}
	string arg1,arg2,arg3;
	arg1=argv[1];
	arg2=argv[2];

	//Check the input arguments	
	if(arg1.compare(0,7,"network")!=0 && arg1.compare(0,6,"router")!=0)
	{
		cout<<"\nERROR: The first argument should be either 'network' or 'router'!\n";
		exit(1);
	}
	if(arg2.compare(0,1,"f")!=0 && arg2.compare(0,1,"s")!=0)
	{
		cout<<"\nERROR: The second argument should be either 'f', for outputing results in a file, or 's', for showing results on the screen!\n";
		exit(1);
	}
	//find the number of wavelengths
	ifstream inputs("inputs.txt",ios::in);
	if(!inputs.is_open())
	{
		cout<<"\nERROR: The input file can not be opened!\n";
		exit(1);
	}
	//the input file is open
	int loc,wdm_number;
	string wdm_number_txt,dummy;
	while(!inputs.eof())
	{
		getline(inputs,dummy);
		if((dummy.length()!=0 && dummy[0]=='/' && dummy[1]=='/') || dummy.length()==0) //Skip the commments and empty lines
			continue;
		else if(dummy.compare(0,3,"end")==0)
			break;
		else if(dummy.compare(0,10,"arch_type=")==0)
			continue;
		else if(dummy.compare(0,7,"set_wdm")==0)
		{
			loc=0;
			while(dummy[dummy.find("set_wdm")+7+loc]!=';')
				loc++;
			wdm_number_txt.assign(dummy,dummy.find("set_wdm")+7,loc);
			wdm_number=atoi(wdm_number_txt.c_str());
			dummy.erase();
			loc=0;
			break;
		}
		else if(dummy.compare(0,9,"unset_wdm")==0)
		{
			wdm_number=1;
			break;
		}
	}
	inputs.close();
	loc=0;
	dummy.erase();
	//
	if(arg1.compare(0,7,"network")==0)//Network analyzer is called
	{
		//The input is from the file defined by the user as the third argument
		ifstream inputs("inputs.txt",ios::in);
		if(!inputs.is_open())
		{
			cout<<"\nERROR: The input file can not be opened!\n";
			exit(1);
		}
		//the input file is open
		//int loc;
		string onoc_type;
		while(!inputs.eof())
		{
			getline(inputs,dummy);
			if((dummy.length()!=0 && dummy[0]=='/' && dummy[1]=='/') || dummy.length()==0) //Skip the commments and empty lines
				continue;
			else if(dummy.compare(0,3,"end")==0)
				break;
			else if(dummy.compare(0,10,"arch_type=")==0)
			{
				loc=0;
				while(dummy[dummy.find("arch_type=")+10+loc]!=';')
					loc++;
				onoc_type.assign(dummy,dummy.find("arch_type=")+10,loc);
				dummy.erase();
				loc=0;
				break;
			}
		}
		inputs.close();
		//The architecture type is known 	
		inputs.open("inputs.txt",ios::in);
		if(!inputs.is_open())
		{
			cout<<"\nERROR: The input file can not be opened!\n";
			exit(1);
		}
		if(onoc_type.compare(0,4,"mesh")==0 || onoc_type.compare(0,6,"ftorus")==0)//if the considered onoc is a mesh or folded-torus onoc
		{
			int src_x,src_y,des_x,des_y;
			string str;
			while(!inputs.eof())
			{
				getline(inputs,str);
				if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0 || (str.compare(0,10,"arch_type=")==0)) //Skip the commments and empty lines
					continue;
				else if(str.compare(0,3,"end")==0)
					break;
				else if(str.compare(0,4,"from")==0)
				{
					find_src_des(str,src_x,src_y,des_x,des_y);//The source and destination from the file
					Network onoc;
					cout << "Running network analyzer " << src_x << " " << src_y << " " << des_x << " " << des_y << " " << arg2[0] << " " << wdm_number << endl; 
					if(onoc_type.compare(0,4,"mesh")==0)//if the onoc type is a mesh
					{
						//onoc.SNR_Mesh(src_y,src_x,des_y,des_x,arg2[0],wdm_number);
						onoc.SNR_Mesh(src_x,src_y,des_x,des_y,arg2[0],wdm_number);
					}
					else if(onoc_type.compare(0,6,"ftorus")==0)
					{
						onoc.SNR_FTorus(src_x,src_y,des_x,des_y,arg2[0],wdm_number);
					}
				}
			}
		}
		else if(onoc_type.compare(0,5,"ftree")==0)//if the considered onoc is a fat-tree-based onoc
		{
			int src,des,ben,crs;
			bool drop;
			string str;
			while(!inputs.eof())
			{
				getline(inputs,str);
				if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0 || (str.compare(0,10,"arch_type=")==0)) //skip the commments and empty lines
					continue;
				else if(str.compare(0,3,"end")==0)
					break;
				else if(str.compare(0,4,"from")==0)
				{
					//find the source
					loc=0;
					while(str[4+loc]!='t')
						loc++;
					dummy.assign(str,4,loc);
					src=atoi(dummy.c_str());
					dummy.erase();

					//find the destination
					loc=0;
					while(str[str.find('t')+2+loc]!=';')
						loc++;
					dummy.assign(str,str.find('t')+2,loc);
					des=atoi(dummy.c_str());
					dummy.erase();
					FTree_Network ftree;
					ftree.SNR_FTree(src,des,arg2[0],wdm_number);
				}
			}
		}
		inputs.close();
	}
	else if(arg1.compare(0,6,"router")==0)//Router Analyzer is called
	{
            cout << 'Running router analyzer...' << endl;
            Router_func router;
            router.optical_router_analyzer(arg2[0]);
	}
	//system("pause");
	return 0;
}


void find_src_des(string str,int& src_x,int& src_y,int& des_x,int& des_y)
{
	string dummy;
	int loc;
	if(str.compare(0,4,"from")==0) //Start of a communication pair
	{
		//x of the sender
		loc=0;
		while(str[4+loc]!=',')
			loc++;
		dummy.assign(str,4,loc);
		src_x=atoi(dummy.c_str());
		dummy.erase();

		//y of the sender
		loc=0;
		while(str[str.find(',')+1+loc]!='t')
			loc++;
		dummy.assign(str,str.find(',')+1,loc);
		src_y=atoi(dummy.c_str());
		dummy.erase();

		//x of the receiver
		loc=0;
		while(str[str.find('t')+2+loc]!=',')
			loc++;
		dummy.assign(str,str.find('t')+2,loc);
		des_x=atoi(dummy.c_str());
		dummy.erase();

		//y of the Receiver
		int loc2=0;
		while(str[str.find('t')+2+loc+1+loc2]!=';')
			loc2++;
		dummy.assign(str,str.find('t')+2+loc+1,loc2);
		des_y=atoi(dummy.c_str());
		dummy.erase();
	}
}
