/*********************************************************************************
 *
 * File name:       network_structure.h
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include "router_analyzer.h"
#include <math.h>

using namespace std;

#ifndef network_structure
#define network_structure

struct router_node{
	int x;
	int y;
	int status;
	Router_Analyzer* net_router;
};

struct node{
	int x;
	int y;
	int status;
};

struct result{
	string* Link_Power_loss;
	int* Link_path;
};

struct ft_rtr_node{
	bool prt_taken[4][2];//We have 4 ports and each port is a bidirectional one. prt_taken[i][0] is for the downward link while prt_taken[i][1] is for the upward link
	int rtr_num;
	int child[2];
	ft_rtr_node* nxt_rtr[4];
	int prt[4];
	int level;
	int loc;//Loc defines where the router is located, in the upper levels (loc=1) or in the down levels (loc=0). loc=-1 for first level
};

struct ft_router{
	int rtr_num;//router number
	int level_num;//level number
	int in;//input
	int out;//output
	Router_Analyzer* net_router;
	int crs;
	int rtr_net_side;
};

struct path_node{ //used for the power loss function in fat-tree
	int rtr_num;//router number
	int level_num;//level number
	int in;//input
	int out;//output
};

void copy_res(string[],string[],int[],int[],int&,int&);

//Mesh and Folded-Torus networks
class Network{
	//Dimension
	int size_M; 
	int size_N;
	ifstream net_config;
	ofstream results_power_loss,results_crosstalk;
	node src,des; //Source and Destination

	int routers_on_the_link; //Number of routers on the link
	result PL; //Power Loss of the link
	string* crosstalk_str;
	router_node* nodes_on_the_path;

	double chip_size;
	double hop_length;

	//int wdm_no;
	

public:
	Waveguide wg_net; //To consider the propagation loss at the network level
	Network();
	~Network();
	void set_M(int x){size_M=x;}
	void set_N(int x){size_N=x;}
	//void set_wdm_no(int x){wdm_no=x;}
	int get_M(){return size_M;}
	int get_N(){return size_N;}
	//int get_wdm_no{return wdm_no;}
	string* get_crosstalk_str(){return crosstalk_str;}
	result get_result(){return PL;}
	//Calculate the power loss of the link and crosstalk noise at the destination
	void Power_loss_Mesh(int start_x,int start_y,int end_x,int end_y,int wvlgth_no,int wdm_no);
	void crosstalk_Mesh(long double&,long double&,long double&,int,int);
	void SNR_Mesh(int src_x,int src_y,int rcv_x,int rcv_y,char ch,int wdm_no);
	void Power_loss_FTorus(int start_x,int start_y,int end_x,int end_y,int wvlgth_no,int wdm_no);
	void crosstalk_FTorus(long double&,long double&,long double&,int,int);
	void SNR_FTorus(int src_x,int src_y,int rcv_x,int rcv_y,char ch,int);
	void find_sender_receiver(string,node& sender,node& receiver);
	void cal_ben_crs(int,int,int,int,int&,int&);
};

//Fat-Tree network
class FTree_Network{
	
	//Dimension
	int num_cores; //Number of processor cores
	//The floorplan is supposed to be a MxN square in which M is the number of processing
	//cores in a column and N is the number of cores in a row
	int fp_M;
	int fp_N;
	int num_level;
	int direction;//Defines the direction of the optical signal (Upward=1 Downward=0 Deafult=-1)
	int routers_on_the_link; //Number of routers on the link

	ifstream net_config;
	ofstream results_power_loss,results_crosstalk;

	ft_rtr_node *rtr_1st_level;
	//Up level routers
	ft_rtr_node **up_rtr;
	//Down level routers
	ft_rtr_node **dwn_rtr;
	ft_router *nodes_on_the_path;

	result PL; //Power Loss of the link
	string* crosstalk_str;

	double chip_size;
	double hop_length;
	

public:
	Waveguide wg_net; //To cinsider the propagation loss at the network level
	FTree_Network();
	~FTree_Network();
	void set_num_cores(int x){num_cores=x;}
	int get_num_cores(){return num_cores;}
	void set_fp_M(int x){fp_M=x;}
	int get_fp_M(){return fp_M;}
	void set_fp_N(int x){fp_N=x;}
	int get_fp_N(){return fp_N;}
	void set_direction(int x){direction=x;}
	int get_direction(){return direction;}
	void build_FTnet(); //This function helps to build the fat-tree network in the memory to simplify the calculations
	void copy_two_rtrs(ft_rtr_node &rtr1,ft_rtr_node rtr2);//Copy rtr2 to rtr1
	bool find_child_path(ft_rtr_node cur_rtr,int dest,int prt_num);//This function helps to find the child path
	int find_ancestor_level(int src,int dest,bool &other_side);//Return the level number of the common ancestor
	string* get_crosstalk_str(){return crosstalk_str;}
	result get_result(){return PL;}
	int FT_num_crossings(ft_rtr_node rtr1,ft_rtr_node rtr2,int prt);//To calculate number of crossings between two levels and routers, mode defines the location of the current router: Upper Network->mode=1 and Down Network->mode=0
	int FT_num_bendings(ft_rtr_node rtr1,ft_rtr_node rtr2);//To calculate number of bendings between two optical routers
	int match_prt_ftree(int prt_num,int direc);//Math the ports with the optical router structure input file
	void find_src_dest_ftree(int &src,int &des,string str);
	//Calculate the power loss of the link and crosstalk noise at the destination
	void Power_loss_FTree(int src,int dest,int &crss_num, int &bend_num, bool &Drop,int wvlgth_no,int wdm_no);
	void crosstalk_FTree(long double&,long double&,long double&,int,int);
	void SNR_FTree(int src,int des,char ch,int);
	bool find_path_lower(int,int,int,int,int,int,int,int,int,int,path_node*,int,int,bool,int,int,bool&);
	bool find_path_upper(int,int,int,int,int,int,int,int,int,int,path_node*,int,int,bool,int,int,bool&);
};

//Auxiliary function to calculate crosstalk noise coefficients in WDM ONoCs
double xtalk_coeff_cse(int wvlgth_no,int wdm_no,int func_no,double K1,double K2,double K3,double Lp1,double Lc,double MR_FSR,double MR_3dB_BW,double MR_range);
//This function helps to calculate the crosstalk noise on the drop port of the PSE in ONoCs using WDM. Reference: Modeling and measurement of losses in silicon-on-insulator resonators and bends
double MR_Drop_Noise(double wvlngth,double MR_wvlngth,double MR_3dB_BW);

#endif
