/*********************************************************************************
 *
 * File name:       router_analyzer.cpp
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include "router_analyzer.h"

#include <cmath>
#include <cstdlib>
#include <string>

using namespace std;

Router_Analyzer::Router_Analyzer(int input, int output)
{
	//set up the router structure
	router.set_up_structure(input,output);

	res.cur_port=new int[router.total_number];
	res.path=new int[router.total_number];
	res.PL=new string[router.total_number];
	
	for(int i=0;i<router.total_number;i++)
	{
		res.path[i]=-1;
		res.PL[i]="$";
		res.cur_port[i]=-1;
	}
        
	load_parameters();
	coh_noise=0;
}
Router_Analyzer::~Router_Analyzer()
{ 
	delete [] res.cur_port;
	delete [] res.path;
	delete [] res.PL;
}

double convert_to_W(double val)
{
	return pow(10,(-val*0.1));
}
void Router_Analyzer::load_parameters() 
// Here is to load the parameters for the ONoC. In 4.0, we implemented a technology profile number. This number will open the appropriate technology profile text for the device parameters 
{
	string parameters_file,str,dummy,str1;
	int loc;
	ifstream par_file;
	
	std::stringstream ss; 
	ss << router.tech_profile_number;
	
	//int tech_profile_num = router.tech_profile_number;
	//ifstream par_file;
	//stringstream ss;
	//ss << tech_profile_num;
	
	parameters_file = "Technology_Profiles/Technology_Profile_"+ss.str()+".txt"; // load technology profile number to the parameter file name
	//cout << parameters_file << endl;
	
	par_file.open(parameters_file.c_str(),ios::in);
	//par_file.open("Technology_Profiles/Device_Parameters.txt",ios::in);
	if(!par_file.is_open())
	{
		cout<<"\nThe Technology_Profile number " << router.tech_profile_number << " is not yet defined!\n";
		exit(1);
	}
	else
	{
		while(!par_file.eof())
		{
			loc=0;
			getline(par_file,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
                        else if(str.compare(0,5,"#WDM=")==0) //Lc
			{
				while(str[str.find("#WDM=")+5+loc]!=';')
							loc++;
				dummy.assign(str,str.find("#WDM=")+5,loc);
				wvg_no=atoi(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,3,"Lc=")==0) //Lc
			{
				while(str[str.find("Lc=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Lc=")+3,loc);
				Lc=atof(dummy.c_str());
				Lc=convert_to_W(Lc);
				dummy.erase();
			}
			else if(str.compare(0,3,"Kc=")==0) //K1
			{
				while(str[str.find("Kc=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Kc=")+3,loc);
				K1=atof(dummy.c_str());
				K1=convert_to_W(K1);
				dummy.erase();
			}
			else if(str.compare(0,3,"Kr=")==0) //Kr
			{
				while(str[str.find("Kr=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Kr=")+3,loc);
				Kr=atof(dummy.c_str());
				Kr=convert_to_W(Kr);
				dummy.erase();
			}
			else if(str.compare(0,10,"L_pse_off=")==0) //Lp1
			{
				while(str[str.find("L_pse_off=")+10+loc]!=';')
							loc++;
				dummy.assign(str,str.find("L_pse_off=")+10,loc);
				Lp1=atof(dummy.c_str());
				Lp1=convert_to_W(Lp1);
				dummy.erase();
			}
			else if(str.compare(0,9,"L_pse_on=")==0) //Lp2
			{
				while(str[str.find("L_pse_on=")+9+loc]!=';')
							loc++;
				dummy.assign(str,str.find("L_pse_on=")+9,loc);
				Lp2=atof(dummy.c_str());
				Lp2=convert_to_W(Lp2);
				dummy.erase();
			}
			else if(str.compare(0,10,"K_pse_off=")==0) //K2
			{
				while(str[str.find("K_pse_off=")+10+loc]!=';')
							loc++;
				dummy.assign(str,str.find("K_pse_off=")+10,loc);
				K2=atof(dummy.c_str());
				K2=convert_to_W(K2);
				dummy.erase();
			}
			else if(str.compare(0,9,"K_pse_on=")==0) //K3
			{
				while(str[str.find("K_pse_on=")+9+loc]!=';')
							loc++;
				dummy.assign(str,str.find("K_pse_on=")+9,loc);
				K3=atof(dummy.c_str());
				K3=convert_to_W(K3);
				dummy.erase();
			}
                        else if(str.compare(0,10,"L_det_off=")==0) //Passing loss for detectors
                        {
                            for (int i=0; i<wvg_no; i++) // wgn_no defined by #WDM=
                            {
                                loc=0;
                                getline(par_file,str1);
                                while(str1[loc]!=';')
                                    loc++;
				dummy.assign(str1,0,loc-1);
				Ld0[i]=atof(dummy.c_str());
				dummy.erase();
                            }
                        }
                        else if(str.compare(0,9,"L_det_on=")==0) //Insertion loss for detectors
                        {
                            for (int i=0; i<wvg_no; i++) // wgv_no defined by #WDM=
                            {
                                loc=0;
                                getline(par_file,str1);
                                while(str1[loc]!=';')
                                    loc++;
				dummy.assign(str1,0,loc-1);
				Ld1[i]=atof(dummy.c_str());
				dummy.erase();
                            }
                        }
                        else if(str.compare(0,9,"K_det_on=")==0) //Crosstalk for detectors
                        {
                            for (int i=0; i<wvg_no; i++) // wvg_no defined by #WDM=
                            {
                                loc=0;
                                getline(par_file,str1);
                                while(str1[loc]!=';')
                                    loc++;
				dummy.assign(str1,0,loc-1);
				Kd1[i]=atof(dummy.c_str());
				dummy.erase();
                            }
                        }                        
			else if(str.compare(0,3,"Kt=")==0) //Kt
			{
				while(str[str.find("Kt=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Kt=")+3,loc);
				Kt=atof(dummy.c_str());
				Kt=convert_to_W(Kt);
				dummy.erase();
			}
			else if(str.compare(0,3,"Lb=")==0) //Lb
			{
				while(str[str.find("Lb=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Lb=")+3,loc);
				Lb=atof(dummy.c_str());
				Lb=convert_to_W(Lb);
				dummy.erase();
			}
			else if(str.compare(0,3,"Lp=")==0) //Lp
			{
				while(str[str.find("Lp=")+3+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Lp=")+3,loc);
				Lp=atof(dummy.c_str());
				Lp=convert_to_W(Lp);
				dummy.erase();
			}
			else if(str.compare(0,5,"Lpol=")==0) //Lpol --- Polarization loss
			{
				while(str[str.find("Lpol=")+5+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Lpol=")+5,loc);
				Lpol=atof(dummy.c_str());
				Lpol=convert_to_W(Lpol);
				dummy.erase();
			}
			else if(str.compare(0,5,"Lcpl=")==0)
			{
				while(str[str.find("Lcpl=")+5+loc]!=';')
				loc++;
				dummy.assign(str,str.find("Lcpl=")+5,loc);
				Lcpl=atof(dummy.c_str());
				Lcpl=convert_to_W(Lpol);
				dummy.erase();
			}
			else if(str.compare(0,13,"MR_Dimension=")==0) //MR_Dimension
			{
				while(str[str.find("MR_Dimension=")+13+loc]!=';')
							loc++;
				dummy.assign(str,str.find("MR_Dimension=")+13,loc);
				MR_Dimension=atof(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,10,"WGD_width=")==0) //Waveguide width
			{
				while(str[str.find("WGD_width=")+10+loc]!=';')
							loc++;
				dummy.assign(str,str.find("WGD_width=")+10,loc);
				WGD_width=atof(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,4,"Pin=")==0) //Input Power
			{
				while(str[str.find("Pin=")+4+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Pin=")+4,loc);
				input_power=atof(dummy.c_str());
				input_power=convert_to_W(-input_power);//Remember to use "-" whenever the input argument to the convert_to_W function is power
				dummy.erase();
			}
			else if(str.compare(0,5,"MR_Q=")==0) //Q-factor of the MR
			{
				while(str[str.find("MR_Q=")+5+loc]!=';')
							loc++;
				dummy.assign(str,str.find("MR_Q=")+5,loc);
				MR_Q=atof(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,16,"MR_wvlgth_range=")==0) //The wavelength range of the MR
			{
				while(str[str.find("MR_wvlgth_range=")+16+loc]!=';')
							loc++;
				dummy.assign(str,str.find("MR_wvlgth_range=")+16,loc);
				MR_wvlgth_range=atof(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,4,"FSR=")==0) //Free Spectral Range (FSR)
			{
				while(str[str.find("FSR=")+4+loc]!=';')
							loc++;
				dummy.assign(str,str.find("FSR=")+4,loc);
				FSR=atof(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,14,"Bending_angle=")==0) //Bending angle
			{
				while(str[str.find("Bending_angle=")+14+loc]!=';')
							loc++;
				dummy.assign(str,str.find("Bending_angle=")+14,loc);
				bend_angle=atof(dummy.c_str());
				dummy.erase();
			}
		}
		Lc1=Lc*Lp1;
		Lc2=Lp2*(1 + pow(K3,2)*pow(K1,2))+pow(K3,2)*K1;
		//Lc2=Lp2;//Valid approximation (see the paper)
		full_Lc2=Lc*Lc*Lp2;
		//Crosstalk Coefficients, used in the network level analyses
		cr_coef_1=(Lp1*K1);//cr_coef_1=(Lp1*K1 + Lp1*K1*K2*K1);
		cr_coef_2=(K1 + Lc*Lc*K2);
		cr_coef_3=(Lp1*Lp1*K1 + K2);//cr_coef_3=(Lp1*Lp1*K1 + Lp1*K1*K2*K1*Lp1 + K2);
		cr_coef_4=(K1*K3 + Lc*Lp2*Kr*K3);
		cr_coef_5=(Lc*K3 + Lc*Lp2*K1*K3);
		//3dB bandwitdh of MR
		MR_threedb_bwd=MR_wvlgth_range/MR_Q;
	}
	par_file.close();
	/*cout<<"\nLc="<<Lc<<endl<<" Kc="<<K1<<" Kr="<<Kr<<" Lp1="<<Lp1<<" Lp2="<<Lp2<<" K2="<<K2<<" K3="<<K3<<" Kt="<<Kt<<" Lb="<<Lb<<" Lp="<<Lp<<" MR="<<MR_Dimension<<" Pin="<<input_power<<" Lc1="<<Lc1<<" Lc2="<<Lc2
		<<"WGD= "<<WGD_width<<" MR_Q="<<MR_Q<<" MR_3dB="<<MR_threedb_bwd<<" FSR="<<FSR<<" Bending_Angle="<<bend_angle<<endl;*/
}

void Router_Analyzer::find_path(int in,int port_current,int count,int wvlnth_no,int wdm_no)
{
	int next; //The next element in router's structure
	int next_port; //The next port of the elemetn

	if(in!=-1 && port_current!=-1)
	{	
		if(router.elm[in].type==1) //Port
		{
			next=router.port[in].get_next();
			res.path[count]=next;
			if(router.port_type[0]==in) //Injection and modulation (see the document)
				res.PL[count]="P0_0";
			else if(router.port_type[2]==in) //Input_North
				res.PL[count]="P0_1";
			else if(router.port_type[4]==in) //Input_East
				res.PL[count]="P0_2";
			else if(router.port_type[6]==in) //Input_South
				res.PL[count]="P0_3";
			else if(router.port_type[8]==in) //Input_West
				res.PL[count]="P0_4";
			++count;
			find_path(next,router.elm[in].terminals[0],count,wvlnth_no,wdm_no);
		}
		else if(router.elm[in].type==2) //Waveguide Crossing
		{
			if(port_current==1) //Current port is West
			{
				next=router.wc[in].get_east();
				next_port=3;
			}
			else if(port_current==2) //Current port is North
			{
				next=router.wc[in].get_south();
				next_port=4;
			}
			else if(port_current==3) //Current port is East
			{
				next=router.wc[in].get_west();
				next_port=1;
			}
			else //Current port is South
			{
				next=router.wc[in].get_north();
				next_port=2;
			}

			res.path[count]=next;
			res.PL[count]="Lc";
			res.cur_port[count]=port_current;
			++count;

			//Length of a waveguide crossing is equal to 2*MR_D
			w.set_length(w.get_length()+2*MR_Dimension+WGD_width);

			find_path(next,router.elm[in].terminals[next_port],count,wvlnth_no,wdm_no);
		}
		else if(router.elm[in].type==3) //Bending
		{
			next=router.wb[in].get_out();
			res.path[count]=next;
			res.PL[count]="Lb";
			res.cur_port[count]=port_current;
			count++;

			//Length of a bending is equal to MR_D
			w.set_length(w.get_length()+MR_Dimension+WGD_width);

			find_path(next,router.elm[in].terminals[2],count,wvlnth_no,wdm_no);
		}
		else if(router.elm[in].type==4) //Terminator
		{
			next=-1; //Light is absorbed
			res.path[count]=next;
			res.PL[count]="Terminated";
			res.cur_port[count]=port_current;
			count++;

			//Length of a input port of a terminator is 0.5*MR_D
			w.set_length(w.get_length()+0.5*MR_Dimension);

			find_path(next,router.elm[in].terminals[1],count,wvlnth_no,wdm_no);
		}
		else if(router.elm[in].type==5) //CSE 
		{
			//Decide the turn
			if(router.cse[in].get_CSE_MR()==1) //If the CSE is in ON state make turn
			{
				if(router.cse[in].get_CSE_MR_Loc()==1) //If the MR is in the first locaton the turn should be from a->b or b->a, etc
				{
					if(port_current==1)
					{
						next=router.cse[in].get_north();
						next_port=2;
					}
					else if(port_current==2)
					{
						next=router.cse[in].get_west();
						next_port=1;
					}
					else if(port_current==3)
					{
						next=router.cse[in].get_south();
						next_port=4;
					}
					else if(port_current==4)
					{
						next=router.cse[in].get_east();
						next_port=3;
					}
				}
				else if(router.cse[in].get_CSE_MR_Loc()==2) //If the MR is in the second locaton the turn should be from b->c or c->b, etc
				{
					if(port_current==1)
					{
						next=router.cse[in].get_south();
						next_port=4;
					}
					else if(port_current==2)
					{
						next=router.cse[in].get_east();
						next_port=3;
					}
					else if(port_current==3)
					{
						next=router.cse[in].get_north();
						next_port=2;
					}
					else if(port_current==4)
					{
						next=router.cse[in].get_west();
						next_port=1;
					}
				}
				else if(router.cse[in].get_CSE_MR_Loc()==3) //If the MR is in the third locaton the turn should be from a->d or d->a, etc
				{
					if(port_current==1)
					{
						next=router.cse[in].get_south();
						next_port=4;
					}
					else if(port_current==2)
					{
						next=router.cse[in].get_east();
						next_port=3;
					}
					else if(port_current==3)
					{
						next=router.cse[in].get_north();
						next_port=2;
					}
					else if(port_current==4)
					{
						next=router.cse[in].get_west();
						next_port=1;
					}
				}
				else if(router.cse[in].get_CSE_MR_Loc()==4) //If the MR is in the fourth locaton the turn should be from d->c or c->d, etc
				{
					if(port_current==1)
					{
						next=router.cse[in].get_north();
						next_port=2;
					}
					else if(port_current==2)
					{
						next=router.cse[in].get_west();
						next_port=1;
					}
					else if(port_current==3)
					{
						next=router.cse[in].get_south();
						next_port=4;
					}
					else if(port_current==4)
					{
						next=router.cse[in].get_east();
						next_port=3;
					}
				}

				if((router.cse[in].get_CSE_MR_Loc()==1 && (port_current==3 || port_current==4)) ||
					(router.cse[in].get_CSE_MR_Loc()==2 && (port_current==1 || port_current==4)) || 
					(router.cse[in].get_CSE_MR_Loc()==3 && (port_current==2 || port_current==3)) ||
					(router.cse[in].get_CSE_MR_Loc()==4 && (port_current==1 || port_current==2)))
				{
					if(router.cse[in].get_CSE_MR_Loc()==1 || router.cse[in].get_CSE_MR_Loc()==3)
					{
						//Turning length in CSE is 2*MR_D
						w.set_length(w.get_length()+2*(wdm_no-wvlnth_no)*MR_Dimension+6*MR_Dimension+3*WGD_width);
						res.PL[count]="(Lc Lp2 Lc) Lp1^";
						res.PL[count]+=convertInt(2*(wdm_no-wvlnth_no));
						res.PL[count]+="@";//@ is used to help find the exponent
					}
					else if(router.cse[in].get_CSE_MR_Loc()==2 || router.cse[in].get_CSE_MR_Loc()==4)
					{
						//Turning length in CSE is 2*MR_D
						w.set_length(w.get_length()+2*(wvlnth_no-1)*MR_Dimension+6*MR_Dimension+3*WGD_width);
						res.PL[count]="(Lc Lp2 Lc) Lp1^";
						res.PL[count]+=convertInt(2*(wvlnth_no-1));
						res.PL[count]+="@";//@ is used to help find the exponent
					}
				}
				else
				{
					if(wdm_no==1)//single wavelength
					{
						w.set_length(w.get_length()+2*MR_Dimension+WGD_width);
						res.PL[count]="Lp2";
					}
					else if(wdm_no>1)//WDM
					{
						if(router.cse[in].get_CSE_MR_Loc()==1 || router.cse[in].get_CSE_MR_Loc()==3)
						{
							w.set_length(w.get_length()+2*(wvlnth_no-1)*MR_Dimension+3*MR_Dimension+WGD_width);
							res.PL[count]="Lp2 Lp1^";
							res.PL[count]+=convertInt(2*(wvlnth_no-1));
							res.PL[count]+="@";//@ is used to help find the exponent
						}
						else if(router.cse[in].get_CSE_MR_Loc()==2 || router.cse[in].get_CSE_MR_Loc()==4)
						{
							w.set_length(w.get_length()+2*(wdm_no-wvlnth_no)*MR_Dimension+3*MR_Dimension+WGD_width);
							res.PL[count]="Lp2 Lp1^";
							res.PL[count]+=convertInt(2*(wdm_no-wvlnth_no));
							res.PL[count]+="@";//@ is used to help find the exponent
						}
					}
				}

				//coherent crosstalk
				if(wdm_no>1)//first order coherent crosstalk is applicable to WDM networks when CSE is ON
				{
					if((router.cse[in].get_CSE_MR_Loc()==1 && (port_current==1 || port_current==2)) ||
						(router.cse[in].get_CSE_MR_Loc()==3 && (port_current==1 || port_current==4))||
						(router.cse[in].get_CSE_MR_Loc()==2 && (port_current==1 || port_current==4)) ||
						(router.cse[in].get_CSE_MR_Loc()==4 && (port_current==1 || port_current==2)))
						for(int i=1;i<wvlnth_no;i++)
							//coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)/(Lp2*pow(Lp1,2*(wvlnth_no-i))));
                                                    coh_noise*=Lp1; //Matrix method replaced
					else if((router.cse[in].get_CSE_MR_Loc()==1 && (port_current==3 || port_current==4)) ||
						(router.cse[in].get_CSE_MR_Loc()==3 && (port_current==2 || port_current==3)) ||
						(router.cse[in].get_CSE_MR_Loc()==2 && (port_current==3 || port_current==2)) ||
						(router.cse[in].get_CSE_MR_Loc()==4 && (port_current==3 || port_current==4)))
						for(int i=wdm_no;i>wvlnth_no;i--)
							//coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)/(Lp2*pow(Lp1,2*(i-wvlnth_no))));
                                                    coh_noise*=Lp1; //Matrix method replaced
				}
			}
			else //If MR is OFF (Act like a crossing)
			{
				if(port_current==1) //Current port is West
				{
					next=router.cse[in].get_east();
					next_port=3;
				}
				else if(port_current==2) //Current port is North
				{
					next=router.cse[in].get_south();
					next_port=4;
				}
				else if(port_current==3) //Current port is East
				{
					next=router.cse[in].get_west();
					next_port=1;
				}
				else //Current port is South
				{
					next=router.cse[in].get_north();
					next_port=2;
				}
				
				if(wdm_no==1)//single wavelength
				{
					//The length is 2*MR_D (The same as Crossing)
					w.set_length(w.get_length()+2*MR_Dimension+WGD_width);
					res.PL[count]="Lc1";
				}
				else if(wdm_no>1)//WDM
				{
					w.set_length(w.get_length()+(wdm_no+1)*MR_Dimension+WGD_width);
					res.PL[count]="Lc Lp1^";
					res.PL[count]+=convertInt(wdm_no);
					res.PL[count]+="@";//@ is used to help find the exponent

					//if(port_current==1 || port_current==3)
					//{
					//	w.set_length(w.get_length()+(wdm_no+1)*MR_Dimension+WGD_width);
					//	res.PL[count]="Lc Lp1^";
					//	res.PL[count]+=convertInt(wdm_no);
					//	res.PL[count]+="@";//@ is used to help find the exponent
					//}
					//if(port_current==2 || port_current==4)
					//{
					//	w.set_length(w.get_length()+(wdm_no+2)*MR_Dimension+WGD_width);
					//	res.PL[count]="Lc Lp1^";
					//	res.PL[count]+=convertInt(wdm_no+1);
					//	res.PL[count]+="@";//@ is used to help find the exponent
					//}
				}
				//Coherent crosstalk analyses for CSE in OFF state
				if(wdm_no>1)
				{
					for(int i=1;i<=wdm_no;i++)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));
					/*if((router.cse[in].get_CSE_MR_Loc()==1 && (port_current==1 || port_current==2)) ||
						(router.cse[in].get_CSE_MR_Loc()==2 && (port_current==1 || port_current==4))||
						(router.cse[in].get_CSE_MR_Loc()==3 && (port_current==1 || port_current==4)) ||
						(router.cse[in].get_CSE_MR_Loc()==4 && (port_current==1 || port_current==2)))
						for(int i=1;i<=wdm_no;i++)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));
					else if((router.cse[in].get_CSE_MR_Loc()==1 && (port_current==3 || port_current==4)) ||
						(router.cse[in].get_CSE_MR_Loc()==2 && (port_current==2 || port_current==3))||
						(router.cse[in].get_CSE_MR_Loc()==3 && (port_current==2 || port_current==3)) ||
						(router.cse[in].get_CSE_MR_Loc()==4 && (port_current==3 || port_current==4)))
						for(int i=wdm_no;i>=1;i--)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));*/
				}
				else if(wdm_no==1)
							coh_noise+=pow(K2,2)/Lp1;
			}

			//The turning decision is already made
			res.path[count]=next;
			res.cur_port[count]=port_current;
			++count;

			find_path(next,router.elm[in].terminals[next_port],count,wvlnth_no,wdm_no);

		}
		else if(router.elm[in].type==6) // PSE
		{
			if(router.pse[in].get_PSE_MR()==1) //If the PSE is in ON state make turn
			{
				if(port_current==1) //Input port
				{
					next=router.pse[in].get_drop();
					next_port=2; 
				}
				else if(port_current==2) //Drop port
				{
					next=router.pse[in].get_input();
					next_port=1; 
				}
				else if(port_current==3) //Through port
				{
					next=router.pse[in].get_add();
					next_port=4;
				}
				else if(port_current==4) //Add port
				{
					next=router.pse[in].get_through();
					next_port=3;
				}
				if(wdm_no==1)//Single wavelength
				{
					//Turning length in PSE is 4*MR_D
					w.set_length(w.get_length()+4*MR_Dimension+WGD_width);
					res.PL[count]="Lp2";
				}
				else if(wdm_no>1)//WDM 
				{
					if(port_current==1 || port_current==2)
					{
						w.set_length(w.get_length()+2*(wvlnth_no-1)*MR_Dimension+4*MR_Dimension+WGD_width);
						res.PL[count]="Lp2 Lp1^";
						res.PL[count]+=convertInt(2*(wvlnth_no-1));
						res.PL[count]+="@";//@ is used to help find the exponent
					}
					else if(port_current==3 || port_current==4)
					{
						w.set_length(w.get_length()+2*(wdm_no-wvlnth_no)*MR_Dimension+4*MR_Dimension+WGD_width);
						res.PL[count]="Lp2 Lp1^";
						res.PL[count]+=convertInt(2*(wdm_no-wvlnth_no));
						res.PL[count]+="@";//@ is used to help find the exponent
					}
				}
				//coherent crosstalk
				if(wdm_no>1)//firt order coherent crosstalk is applicable to WDM ONoCs when PSE is ON
				{
					if(port_current==1 || port_current==2)
						for(int i=1;i<wvlnth_no;i++)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)/(Lp2*pow(Lp1,2*(wvlnth_no-i))));
					else if(port_current==3 || port_current==4)
						for(int i=wdm_no;i>wvlnth_no;i--)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)/(Lp2*pow(Lp1,2*(i-wvlnth_no))));
				}
			}
			else //If MR is OFF
			{
				if(port_current==1) //a->c
				{
					next=router.pse[in].get_through();
					next_port=3;
				}
				else if(port_current==2) //b->d
				{
					next=router.pse[in].get_add();
					next_port=4;
				}
				else if(port_current==3) //c->1
				{
					next=router.pse[in].get_input();
					next_port=1;
				}
				else if(port_current==4) //d->b
				{
					next=router.pse[in].get_drop();
					next_port=2;
				}

				//Coherent crosstalk
				if(wdm_no>1)
				{
					for(int i=1;i<=wdm_no;i++)
							//coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));
                                            coh_noise+=pow(K2,2)/Lp1;
					/*if(port_current==1 || port_current==2)
						for(int i=1;i<=wdm_no;i++)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));
					else if(port_current==3 || port_current==4)
						for(int i=wdm_no;i>=1;i--)
							coh_noise+=(MR_lor_func(wvlnth_no,i,wdm_no)*K2/(Lp1));*/
				}
				else if(wdm_no==1)
							coh_noise+=pow(K2,2)/Lp1;

				//Passing length of PSE is 2*MR_D
				//Both Single wavelength and WDM
				w.set_length(w.get_length()+(wdm_no+1)*MR_Dimension);
				res.PL[count]="Lp1^";
				res.PL[count]+=convertInt(wdm_no);
				res.PL[count]+="@";//@ is used to help find the exponent
			}

			//Turning decision is already made
			res.path[count]=next;
			res.cur_port[count]=port_current;
			count++;
			
			find_path(next,router.elm[in].terminals[next_port],count,wvlnth_no,wdm_no);
		}
		else if(router.elm[in].type==7) //Extra waveguide
		{
			next=router.wgd[in].get_out();
			res.path[count]=next;
			res.PL[count]="Wgd";
			res.cur_port[count]=port_current;
			count++;

			//Length of the extra waveguide is specified by the user
			w.set_length(w.get_length()+router.wgd[in].get_length());

			find_path(next,router.elm[in].terminals[2],count,wvlnth_no,wdm_no);
		}
	}
	////Add the detector at the end of the communication line
	//next=-1; //Light is absorbed
	//res.path[count]=next;
	//res.PL[count]="Lp2 Lp1^";
	//res.PL[count]+=convertInt(wvlnth_no-1);
	//res.PL[count]+="@";//@ is used to help find the exponent
	//count++;
	//res.PL[count++]="Detected";
	////Length of the detector at the end of the line
	//w.set_length(w.get_length()+4*MR_Dimension+(wvlnth_no-1)*MR_Dimension+WGD_width);
	//for(int i=0;i<router.total_number;i++)
	//	if(res.path[i]!=-1)
	//	cout<<res.path[i]<<" ";
	//cout<<"\nCoherent noise is:\n"<<coh_noise;
}

void Router_Analyzer::refresh()
{
	for(int i=0;i<router.total_number;i++)
	{
		res.path[i]=-1;
		res.PL[i]="$";
		res.cur_port[i]=-1;
	}
	w.set_length(0);
}

void Router_Analyzer::change_MRs(int inprt,int outprt)
{
	ifstream rtr_strc_def("Router_Structure_Definition.txt",ios::in);
	//int PRT_count=0,WC_count=0,WB_count=0,OT_count=0,CSE_count=0,PSE_count=0,WGD_count=0;
	if(rtr_strc_def.is_open())
	{
		string str,dummy;
		int loc,id;
		//Start reading the number of elements from the input text file
		while(!rtr_strc_def.eof())
		{
			getline(rtr_strc_def,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str=="MR_config")
				break;
		}
		//Now at the MR Configuration definitions
		while(!rtr_strc_def.eof())
		{
			loc=0;
			getline(rtr_strc_def,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str.compare(0,5,"start")==0)
				break;
			else if(str.compare(0,2,"if")==0)	
			{
				//Read the related input port
				while(str[str.find("input=")+6+loc]!=';')
					loc++;
				dummy.assign(str,str.find("input=")+6,loc);
				if(inprt==atoi(dummy.c_str()))
				{
					loc=0;
					dummy.erase();
					while(str[str.find("output=")+7+loc]!=';')
						loc++;
					dummy.assign(str,str.find("output=")+7,loc);
					if(outprt==atoi(dummy.c_str()))
					{
						loc=0;
						dummy.erase();
						while(str[str.find("MR=")+3+loc]!=';')
							loc++;
						dummy.assign(str,str.find("MR=")+3,loc);
						router.mr[atoi(dummy.c_str())].set_MR_status(1);
						//We need to also change the related MRs in PSEs or CSEs
						for(int i=0;i<router.total_number;i++)
						{
							if(router.pse[i].get_PSE_MR_num()==atoi(dummy.c_str()))
								router.pse[i].set_PSE_MR(router.mr[atoi(dummy.c_str())]);
							else if(router.cse[i].get_CSE_MR_num()==atoi(dummy.c_str()))
								router.cse[i].set_CSE_MR(router.mr[atoi(dummy.c_str())]);
						}
					}
				}
			}
		}
	}
	else
	{
		cout<<"ERROR: Router Structure Definition file could not be opened!\n";
		exit(1);
	}
	rtr_strc_def.clear();
	rtr_strc_def.close();

//	for(int i=0;i<router.get_number_of_MRs();i++)
	//	cout<<"\nrouter id= "<<i<<"\nRouter status = "<<router.mr[i].get_MR_status()<<endl;
}
string Router_Analyzer::convertInt(int number)
{
   stringstream ss;//create a stringstream
   ss << number;//add number to the stream
   return ss.str();//return a string with the contents of the stream
}
double Router_Analyzer::MR_lor_func(int wvlngth_no,int MR_wvlngth_no,int wdm_no)
{
	double shift_to_off=0;
	double MR_3dB_half=((MR_wvlngth_no-1)*(FSR/wdm_no*1.0)+MR_wvlgth_range+shift_to_off)/(2*MR_Q);
	return pow(MR_3dB_half,2)/(pow(((wvlngth_no-1)*(FSR/wdm_no*1.0)+MR_wvlgth_range)-((MR_wvlngth_no-1)*(FSR/wdm_no)+MR_wvlgth_range+shift_to_off),2)+pow(MR_3dB_half,2));
}
