/*********************************************************************************
 *
 * File name:       router_analyzer.h
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
//#include "tech_profile.h"
#include "router_structure.h"
#include <iostream>
#include <string>
#include <sstream>
using namespace std;

#ifndef router_analyzer
#define router_analyzer

struct Result{
	int* path;
	string* PL;
	int* cur_port;
};

double convert_to_W(double val); //To convert the input device parameteres from dB to Percentage

class Router_Analyzer{

public:
	Router_Struct router;
	Result res;
	Waveguide w; //The waveguide between the input output pair

        int wvg_no;
        
	double input_power; //The input power at the injection port of different optical routers

	//Define the Device parameters used in ONoCs
	//Waveguide Crossing
	double Lc; //Waveguide Crossing Insertion Loss
	double K1; //Waveguide Crossing Crosstalk Coefficient
	double Kr; //Waveguide Crossing Reflectance Coefficient
	
	//Parallel Switching Element (PSE)
	double Lp1; //PSE Passing Loss
	double Lp2; //PSE Drop Loss
	double K2; //Crosstalk Coefficient per PSE in OFF State 
	double K3; //Crosstalk Coefficient per PSE in ON State
        
        //Optical Detectors
        double Ld0[64];
        double Ld1[64];
        double Kd1[64];

	//Optical Terminator
	double Kt;

	//Waveguide
	double Lb; //Waveguide Bending
	double bend_angle; //Bending angle
	double Lp; //Waveguide Propagation Loss in Cm2
	double Lpol; // Waveguide Polarization Loss
	double Lcpl; // Coupler Loss (for Optical pins)
	double MR_Dimension; //Dimension of the Microresonator in use
	double WGD_width;

	//CSE
	double Lc1;
	double Lc2;
	double full_Lc2; //When the optical signal make a complete trun (360)

	//MR
	double MR_threedb_bwd;
	double MR_Q;
	double FSR;
	double MR_wvlgth_range;
        

	//Crosstalk Coefficients
	long double cr_coef_1,cr_coef_2,cr_coef_3,cr_coef_4,cr_coef_5;

	//Coherent crosstalk 
	long double coh_noise;

	Router_Analyzer(int input, int output);
	~Router_Analyzer();
	Result get_res(){return res;}
	void set_coh_noise(long double x){coh_noise=x;}
	long double get_coh_noise(){return coh_noise;}
	void load_parameters();
	void refresh();
	void find_path(int in,int port,int count,int wvlngth_no,int wdm_no); //Used in calculations at the network level
	void change_MRs(int inprt,int outprt);
	string convertInt(int number);
	double MR_lor_func(int,int,int);
};

#endif
