/*********************************************************************************
 *
 * File name:       router_function.cpp
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include "router_function.h"
#include <cmath>
#include <cstdlib>
#include <string>
using namespace std;

Router_func::Router_func()
{
	router=new Router_Analyzer(-1,-1);
	router_config.open("Router_Configuration.txt",ios::in);
	if(!router_config.is_open())
	{
		cout<<"\nERROR: Router Configuration file could not be opened!\n";
		exit(1);
	}
	string str,dummy;
	int loc=0,id;
	wdm_num=1;
	while(!router_config.eof())
	{
		getline(router_config,str);
		if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
			continue;
		if(str.compare(0,12,"xtalk_order=")==0)
		{
			while(str[str.find("xtalk_order=")+12+loc]!=';')
					loc++;
				dummy.assign(str,str.find("xtalk_order=")+12,loc);
				xtalk_order=atoi(dummy.c_str());
				loc=0;
				dummy.erase();
		}
		else if(str.compare(0,7,"set_wdm")==0)
		{
			while(str[str.find("set_wdm")+7+loc]!=';')
					loc++;
				dummy.assign(str,str.find("set_wdm")+7,loc);
				wdm_num=atoi(dummy.c_str());
				loc=0;
				dummy.erase();
		}
		else if(str.compare(0,9,"unset_wdm")==0)
		{
			wdm_num=1;
		}
		else if(str.compare(0,7,"prt_id=")==0)
		{
			while(str[str.find("prt_id=")+7+loc]!='s')
					loc++;
				dummy.assign(str,str.find("prt_id=")+7,loc);
				id=atoi(dummy.c_str());
				loc=0;
				dummy.erase();
				while(str[str.find("set_pwr=")+8+loc]!=';')
					loc++;
				dummy.assign(str,str.find("set_pwr=")+8,loc);
				router->router.port[id].set_pwr(convert_to_W(-atof(dummy.c_str())));//convert to mw
				loc=0;
				dummy.erase();
		}
	}
	router_config.close();
	//cout<<"\nXtalk order="<<xtalk_order<<endl;
	output_pwrs=new double [router->router.get_number_of_ports()]; //Output powers at each port, both input and output ports

	for(int i=0;i<router->router.get_number_of_ports();i++)
		output_pwrs[i]=0;
	//cout<<"number of ports="<<router->router.get_number_of_ports()<<endl;
	//for(int i=0;i<router->router.total_number;i++)
		//cout<<router->router.port[i].get_pwr()<<endl;
	//cout<<wdm_no;
}

Router_func::~Router_func()
{
	delete router;//remember to delete
}

void Router_func::set_mrs()
{
	int inprt=-1,outprt=-1;
	router_config.open("Router_Configuration.txt",ios::in);
	if(!router_config.is_open())
	{
		cout<<"ERROR: Router Configuration file could not be opened!\n";
		exit(1);
	}
	string str,dummy;
	int loc;
	//Start reading input text file
	while(!router_config.eof())
	{
		getline(router_config,str);
		if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
			continue;
		else if(str=="config_start")
			break;
	}

	while(!router_config.eof())
	{
		loc=0;
		getline(router_config,str);
		if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
			continue;
		else if(str.compare(0,5,"config_end")==0)
			break;
		else if(str.compare(0,4,"from")==0)	
		{
			//Read the input port
			while(str[str.find("from")+4+loc]!='t')
				loc++;
			dummy.assign(str,str.find("from")+4,loc);
			inprt=atoi(dummy.c_str());
			loc=0;
			dummy.erase();
			//cout<<"\ninput="<<inprt<<endl;
			while(str[str.find("to")+2+loc]!=';')
				loc++;
			dummy.assign(str,str.find("to")+2,loc);
			outprt=atoi(dummy.c_str());
			//cout<<"\noutput="<<outprt<<endl;
			//Set the MRs based on the User's Configuration
			router->change_MRs(inprt,outprt);
		}
	}
	router_config.clear();
	router_config.close();
	//for(int i=0;i<router->router.get_number_of_MRs();i++)
	//cout<<"\nMR id= "<<i<<"\nMR status = "<<router->router.mr[i].get_MR_status()<<endl;
	//cout<<"\nPSE 29="<<router->router.pse[29].get_PSE_MR();
}
double MR_Drop_Noise_rtr(double wvlngth,double MR_wvlngth,double MR_3dB_BW)
{
	return pow(MR_3dB_BW,2)/(pow(wvlngth-MR_wvlngth,2)+pow(MR_3dB_BW,2));
}
void Router_func::find_path_rtrfunc(int in,int port_current,int order,double power,int prev_loc,int wvlth_no,int wdm_no)
{
	int next; //The next element in router's structure
	int next_port; //The next port of the elemetn
	double prp_loss;
	//cout<<"\ncurrent element = "<<in;
	if(in!=-1 && port_current!=-1)
	{	
		if(router->router.elm[in].type==1) //Port
		{
			if(router->router.port[in].get_next()!=prev_loc)
				next=router->router.port[in].get_next();
			else
				next=router->router.port[in].get_prev();
			//	router->res.path[count]=next;
			if(router->router.port_type[0]==in) //Injection
			{
				//router->res.PL[count]="P0_0";
				output_pwrs[router->router.port_type[0]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[0]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[0]].get_pwr();
			}
			else if(router->router.port_type[2]==in) //Input_North
			{
				//router->res.PL[count]="P0_1";
				output_pwrs[router->router.port_type[2]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[2]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[2]].get_pwr();
			}
			else if(router->router.port_type[4]==in) //Input_East
			{
				//router->res.PL[count]="P0_2";
				output_pwrs[router->router.port_type[4]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[4]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[4]].get_pwr();
			}
			else if(router->router.port_type[6]==in) //Input_South
			{
				//router->res.PL[count]="P0_3";
				output_pwrs[router->router.port_type[6]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[6]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[6]].get_pwr();
			}
			else if(router->router.port_type[8]==in) //Input_West
			{
				//router->res.PL[count]="P0_4";
				output_pwrs[router->router.port_type[8]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[8]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[8]].get_pwr();
			}
			else if(router->router.port_type[1]==in) //Ejection
			{
				//router->res.PL[count]="Ej";
				output_pwrs[router->router.port_type[1]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[1]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[1]].get_pwr();
			}
			else if(router->router.port_type[3]==in) //Output North
			{
				//router->res.PL[count]="OtNrth";
				output_pwrs[router->router.port_type[3]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[3]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[3]].get_pwr();
			}
			else if(router->router.port_type[5]==in) //Output East
			{
				//router->res.PL[count]="OtEast";
				output_pwrs[router->router.port_type[5]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[5]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[5]].get_pwr();
			}
			else if(router->router.port_type[7]==in) //Output South
			{
				//router->res.PL[count]="OtSth";
				output_pwrs[router->router.port_type[7]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[7]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[7]].get_pwr();
			}
			else if(router->router.port_type[9]==in) //Output West
			{
				//router->res.PL[count]="OtWest";
				output_pwrs[router->router.port_type[9]]+=power; //Add the power to the seen power at the port
				power=router->router.port[router->router.port_type[9]].get_pwr();
				//cout<<endl<<router->router.port[router->router.port_type[9]].get_pwr();
			}

			//++count;
			prev_loc=in;
			find_path_rtrfunc(next,router->router.elm[in].terminals[0],order,power,prev_loc,wvlth_no,wdm_no);
		}
		else if(router->router.elm[in].type==2) //Waveguide Crossing
		{
			if(port_current==1) //Current port is West
			{
				if(order>0)
				{
					//North port
					prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.wc[in].set_n_pwr(router->router.wc[in].get_n_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_north();
					next_port=2;

					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//	++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					//prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
					prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);

					//South port
					router->router.wc[in].set_s_pwr(router->router.wc[in].get_s_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_south();
					next_port=4;

					//	router->res.path[count]=next;
					//	router->res.PL[count]="K1";
					//	router->res.cur_port[count]=port_current;
					//	++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);
					//Decrease the crosstalk order
					//--order;

					//West port, Reflection
					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
					router->router.wc[in].set_w_pwr(router->router.wc[in].get_w_pwr()+(power*router->Kr*prp_loss));
					next=router->router.wc[in].get_west();
					next_port=1;
					prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc,wvlth_no,wdm_no);
				}
			}
			else if(port_current==2) //Current port is North
			{
				if(order>0)
				{
					//West port
					prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.wc[in].set_w_pwr(router->router.wc[in].get_w_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_west();
					next_port=1;
					prev_loc=in;
					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);

					//East port
					router->router.wc[in].set_e_pwr(router->router.wc[in].get_e_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_east();
					next_port=3;

					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//	++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);
					//Decrease the crosstalk order
					//--order;

					//North port, Reflection
					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
					router->router.wc[in].set_n_pwr(router->router.wc[in].get_n_pwr()+(power*router->Kr*prp_loss));
					next=router->router.wc[in].get_north();
					next_port=2;
					prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc,wvlth_no,wdm_no);
				}
			}
			else if(port_current==3) //Current port is East
			{
				if(order>0)
				{
					//North port
					prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.wc[in].set_n_pwr(router->router.wc[in].get_n_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_north();
					next_port=2;
					prev_loc=in;
					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);

					//South port
					router->router.wc[in].set_s_pwr(router->router.wc[in].get_s_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_south();
					next_port=4;

					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);
					//Decrease the crosstalk order
					//--order;

					//East port, Reflection
					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
					router->router.wc[in].set_e_pwr(router->router.wc[in].get_e_pwr()+(power*router->Kr*prp_loss));
					next=router->router.wc[in].get_east();
					next_port=3;
					prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc,wvlth_no,wdm_no);
				}
			}
			else //Current port is South
			{
				if(order>0)
				{
					//West port
					prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.wc[in].set_w_pwr(router->router.wc[in].get_w_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_west();
					next_port=1;
					prev_loc=in;
					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//	++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);

					//East port
					router->router.wc[in].set_e_pwr(router->router.wc[in].get_e_pwr()+(power*router->K1*prp_loss));
					next=router->router.wc[in].get_east();
					next_port=3;

					//router->res.path[count]=next;
					//router->res.PL[count]="K1";
					//router->res.cur_port[count]=port_current;
					//++count;
					//Length of a waveguide crossing is equal to 2*MR_D
					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*prp_loss,prev_loc,wvlth_no,wdm_no);
					//Decrease the crosstalk order
					//--order;

					//South port, Reflection
					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
					router->router.wc[in].set_s_pwr(router->router.wc[in].get_s_pwr()+(power*router->Kr*prp_loss));
					next=router->router.wc[in].get_south();
					next_port=4;
					prev_loc=in;
					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc,wvlth_no,wdm_no);
				}
			}

			//router->res.path[count]=next;
			//router->res.PL[count]="Lc";
			//	router->res.cur_port[count]=port_current;
			//++count;
			//Length of a waveguide crossing is equal to 2*MR_D
			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			if(port_current==1)//West
			{
				next=router->router.wc[in].get_east();
				next_port=3;
				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
				router->router.wc[in].set_w_pwr(router->router.wc[in].get_w_pwr()+power);
				router->router.wc[in].set_e_pwr(router->router.wc[in].get_e_pwr()+(power*router->Lc*prp_loss));
			}
			else if(port_current==2)//North
			{
				next=router->router.wc[in].get_south();
				next_port=4;
				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
				router->router.wc[in].set_n_pwr(router->router.wc[in].get_n_pwr()+power);
				router->router.wc[in].set_s_pwr(router->router.wc[in].get_s_pwr()+(power*router->Lc*prp_loss));
			}
			else if(port_current==3)//East
			{
				next=router->router.wc[in].get_west();
				next_port=1;
				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
				router->router.wc[in].set_e_pwr(router->router.wc[in].get_e_pwr()+power);
				router->router.wc[in].set_w_pwr(router->router.wc[in].get_w_pwr()+(power*router->Lc*prp_loss));
			}
			else if(port_current==4)//South
			{
				next=router->router.wc[in].get_north();
				next_port=2;
				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
				router->router.wc[in].set_s_pwr(router->router.wc[in].get_s_pwr()+power);
				router->router.wc[in].set_n_pwr(router->router.wc[in].get_n_pwr()+(power*router->Lc*prp_loss));
			}
			prev_loc=in;
			find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->Lc*prp_loss,prev_loc,wvlth_no,wdm_no);
		}
		else if(router->router.elm[in].type==3) //Bending
		{
			if(router->router.wb[in].get_out()!=prev_loc)
			{
				next=router->router.wb[in].get_out();
				next_port=2;
			}
			else
			{
				next=router->router.wb[in].get_in();
				next_port=1;
			}
			router->router.wb[in].set_in_pwr(router->router.wb[in].get_in_pwr()+power);
			prp_loss=pow(router->Lp,(router->MR_Dimension+router->WGD_width)*0.0001);
			router->router.wb[in].set_out_pwr(router->router.wb[in].get_out_pwr()+(power*router->Lb*prp_loss));
			//router->res.path[count]=next;
			//router->res.PL[count]="Lb";
			//router->res.cur_port[count]=port_current;
			//count++;

			//Length of a bending is equal to MR_D
			//router->w.set_length(router->w.get_length()+router->MR_Dimension+router->WGD_width);
			prev_loc=in;
			find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->Lb*prp_loss,prev_loc,wvlth_no,wdm_no);
		}
		else if(router->router.elm[in].type==4) //Terminator
		{
			next=-1; //Light is absorbed
			router->router.ot[in].set_in_pwr(router->router.ot[in].get_in_pwr()+power);
			//router->res.path[count]=next;
			//router->res.PL[count]="Terminated";
			//router->res.cur_port[count]=port_current;
			//count++;

			if(order>0)
			{
				//Light reflects back on the transmission line
				next=router->router.ot[in].get_in(); //Light is reflected back
				prp_loss=pow(router->Lp,(router->MR_Dimension)*0.0001);
				router->router.ot[in].set_in_pwr(router->router.ot[in].get_in_pwr()+(power*router->Kt*prp_loss));
				//	router->res.path[count]=next;
				//router->res.PL[count]="Reflected";
				//router->res.cur_port[count]=port_current;
				//count++;
				//Length of a input port of a terminator is 0.5*MR_D
				//router->w.set_length(router->w.get_length()+0.5*router->MR_Dimension);
				prev_loc=in;
				find_path_rtrfunc(next,router->router.elm[in].terminals[1],order-1,power*router->Kt*prp_loss,prev_loc,wvlth_no,wdm_no);
				//--order;
			}

			//Length of a input port of a terminator is 0.5*MR_D
			//router->w.set_length(router->w.get_length()+0.5*router->MR_Dimension);
			prev_loc=in;
			find_path_rtrfunc(next,router->router.elm[in].terminals[1],order,0,prev_loc,wvlth_no,wdm_no);
		}
		else if(router->router.elm[in].type==5) //Parallel BOSE
		{
			//for the router analyzer you need to define the CSE as a PSE connected to a waveguide crossing. Please see the document for more details.

			////Decide the turn
			//if(router->router.cse[in].get_CSE_MR()==1) //If the CSE is in ON state make turn
			//{
			//	if(router->router.cse[in].get_CSE_MR_Loc()==1) //If the MR is in the first locaton the turn should be from a->b or b->a, etc
			//	{
			//		if(port_current==1)//West port
			//		{
			//			if(order>0) //East port
			//			{
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)
			//				{
			//					//South port
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//West port, Reflection
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//			}
			//		}
			//		else if(port_current==2)//North port
			//		{
			//			if(order>0) //South port
			//			{
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//East port
			//				{
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//North port, Reflection
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//			}
			//		}
			//		else if(port_current==3)//East port
			//		{
			//			if(order>0) //West port
			//			{
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//East port, Reflection
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1)
			//				{
			//					//North port
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==4)//South port
			//		{
			//			if(order>0)//North port
			//			{
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//South port, Reflection
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //West port
			//				{
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==2) //If the MR is in the second locaton the turn should be from b->c or c->b, etc
			//	{
			//		if(port_current==1)//West port
			//		{
			//			if(order>0) //East port
			//			{
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//West port, Reflection
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //North port
			//				{
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//	router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//	router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//	router->res.PL[count]="Lc Lp2 Kr K3";
			//					//	++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==2)//North port
			//		{
			//			if(order>0) //South port
			//			{
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//	router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//	++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//West port
			//				{
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//	++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//North port, Reflection
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==3)//East port
			//		{
			//			if(order>0) //West port
			//			{
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//South port
			//				{
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;
			//					prev_loc=in;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//East port, Reflection
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==4)//South port
			//		{
			//			if(order>0) //North port
			//			{
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//South port, Reflection
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //East port
			//				{
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					prev_loc=in;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==3) //If the MR is in the third locaton the turn should be from a->d or d->a, etc
			//	{
			//		if(port_current==1)//West port
			//		{
			//			if(order>0) //East port
			//			{
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//North port
			//				{
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//West port, Reflection
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);

			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==2)//North port
			//		{
			//			if(order>0) //South port
			//			{
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//North port, Reflection
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //West port
			//				{
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==3)//East port
			//		{
			//			if(order>0) //West port
			//			{
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);
			//				
			//				//East port, Reflection
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //South port
			//				{
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==4)//South port
			//		{
			//			if(order>0) //North port
			//			{
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//East port
			//				{
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//South port, Reflection
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==4) //If the MR is in the fourth locaton the turn should be from d->c or c->d, etc
			//	{
			//		if(port_current==1)//West port
			//		{
			//			if(order>0) //East port
			//			{
			//				next=router->router.cse[in].get_east();
			//				next_port=3;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//West port, Reflection
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //South port
			//				{
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==2)//North port
			//		{
			//			if(order>0) //South port
			//			{
			//				next=router->router.cse[in].get_south();
			//				next_port=4;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				//North port, Reflection
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);

			//				if(order>1) //East port
			//				{
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//6*MR_D
			//					//router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="Lc Lp2 Kr K3";
			//					//++count;
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->Lp2*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==3)//East port
			//		{
			//			if(order>0) //West port
			//			{
			//				next=router->router.cse[in].get_west();
			//				next_port=1;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//North port
			//				{
			//					next=router->router.cse[in].get_north();
			//					next_port=2;
			//					router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//East port, Reflection
			//					next=router->router.cse[in].get_east();
			//					next_port=3;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//		else if(port_current==4)//South port
			//		{
			//			if(order>0) //North port
			//			{
			//				next=router->router.cse[in].get_north();
			//				next_port=2;
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc*router->K3*prp_loss));
			//				//router->res.path[count]=next;
			//				//router->res.cur_port[count]=port_current;
			//				//2*MR_D
			//				//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//				//router->res.PL[count]="Lc K3";
			//				//++count;
			//				prev_loc=in;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Lc*router->K3*prp_loss,prev_loc);

			//				if(order>1)//West port
			//				{
			//					next=router->router.cse[in].get_west();
			//					next_port=1;
			//					router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->K1*router->K3*prp_loss));
			//					//router->res.path[count]=next;
			//					//router->res.cur_port[count]=port_current;
			//					//2*MR_D
			//					//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//					//router->res.PL[count]="K1 K3";
			//					//++count;

			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->K3*prp_loss,prev_loc);

			//					//South port, Reflection
			//					next=router->router.cse[in].get_south();
			//					next_port=4;
			//					prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//					router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*router->K3*prp_loss));
			//					find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*router->K3*prp_loss,prev_loc);
			//				}
			//				//--order;
			//			}
			//		}
			//	}

			//	if(router->router.cse[in].get_CSE_MR_Loc()==1 && port_current==1)
			//	{
			//		next=router->router.cse[in].get_north();//Turn to North
			//		next_port=2;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc2*prp_loss));
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==1 && port_current==2)
			//	{
			//		next=router->router.cse[in].get_west();//Turn to West
			//		next_port=1;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+power);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==1 && port_current==3)
			//	{
			//		next=router->router.cse[in].get_south();//Turn to South
			//		next_port=4;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+power);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==1 && port_current==4)
			//	{
			//		next=router->router.cse[in].get_east();//Turn to East
			//		next_port=3;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+power);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==2 && port_current==1)
			//	{
			//		next=router->router.cse[in].get_south();//Turn to South
			//		next_port=4;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==2 && port_current==2)
			//	{
			//		next=router->router.cse[in].get_east(); //Turn to East
			//		next_port=3;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+power);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==2 && port_current==3)
			//	{
			//		next=router->router.cse[in].get_north();//Turn to North
			//		next_port=2;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+power);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==2 && port_current==4)
			//	{
			//		next=router->router.cse[in].get_west();//Turn to west
			//		next_port=1;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+power);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==3 && port_current==1)
			//	{
			//		next=router->router.cse[in].get_south();//Turn to South
			//		next_port=4;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc2*prp_loss));
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==3 && port_current==2)
			//	{
			//		next=router->router.cse[in].get_east();//Turn to East
			//		next_port=3;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+power);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==3 && port_current==3)
			//	{
			//		next=router->router.cse[in].get_north();//Turn to North
			//		next_port=2;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->full_Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==3 && port_current==4)
			//	{
			//		next=router->router.cse[in].get_west();//Turn to West
			//		next_port=1;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+power);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc2*prp_loss));
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==4 && port_current==1)
			//	{
			//		next=router->router.cse[in].get_north();//Turn to North
			//		next_port=2;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->full_Lc2*prp_loss));
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==4 && port_current==2)
			//	{
			//		next=router->router.cse[in].get_west();//Turn to West
			//		next_port=1;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+power);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->full_Lc2*prp_loss));
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==4 && port_current==3)
			//	{
			//		next=router->router.cse[in].get_south();//Turn to South
			//		next_port=4;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+power);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc2*prp_loss));	
			//	}
			//	else if(router->router.cse[in].get_CSE_MR_Loc()==4 && port_current==4)
			//	{
			//		next=router->router.cse[in].get_east();//Turn to East
			//		next_port=3;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+power);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lc2*prp_loss));	
			//	}

			//	if((router->router.cse[in].get_CSE_MR_Loc()==1 && (port_current==1 || port_current==2)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==2 && (port_current==3 || port_current==2)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==3 && (port_current==1 || port_current==4)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==4 && (port_current==3 || port_current==4)))
			//	{
			//		//Turning length in CSE is 2*MR_D
			//	//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//		//router->res.PL[count]="Lc2";
			//		//The turning decision is already made
			//		//router->res.path[count]=next;
			//		//router->res.cur_port[count]=port_current;
			//		//++count;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		prev_loc=in;
			//		find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->Lc2*prp_loss,prev_loc);
			//	}
			//	else if((router->router.cse[in].get_CSE_MR_Loc()==1 && (port_current==3 || port_current==4)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==2 && (port_current==1 || port_current==4)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==3 && (port_current==2 || port_current==3)) ||
			//		(router->router.cse[in].get_CSE_MR_Loc()==4 && (port_current==2 || port_current==1)))
			//	{
			//		//Full Turning length in CSE is 6*MR_D
			//	//	router->w.set_length(router->w.get_length()+6*router->MR_Dimension+router->WGD_width);
			//	//	router->res.PL[count]="Lc Lc Lp2";
			//		//The turning decision is already made
			//	//	router->res.path[count]=next;
			//	//	router->res.cur_port[count]=port_current;
			//	//	++count;
			//		prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//		prev_loc=in;
			//		find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->full_Lc2*prp_loss,prev_loc);
			//	}
			//}
			//else //If MR is OFF (Act like a crossing)
			//{
			//	if(port_current==1) //Current port is West
			//	{
			//		if(order>0)
			//		{
			//			//North port
			//			next=router->router.cse[in].get_north();
			//			next_port=2;
			//			prev_loc=in;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			//South port
			//			next=router->router.cse[in].get_south();
			//			next_port=4;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			
			//			//West port, Reflection
			//			next=router->router.cse[in].get_west();
			//			next_port=1;
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*pow(router->Lp1,2)*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*pow(router->Lp1,2)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);
			//			}

			//		}
			//	}
			//	else if(port_current==2) //Current port is North
			//	{
			//		if(order>0)
			//		{
			//			//West port
			//			next=router->router.cse[in].get_west();
			//			next_port=1;
			//			prev_loc=in;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			//East port
			//			next=router->router.cse[in].get_east();
			//			next_port=3;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			
			//			//North port, Reflection
			//			next=router->router.cse[in].get_north();
			//			next_port=2;
			//			if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*pow(router->Lp1,2)*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*pow(router->Lp1,2)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==3 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);
			//			}
			//		}
			//	}
			//	else if(port_current==3) //Current port is East
			//	{
			//		if(order>0)
			//		{
			//			//North port
			//			next=router->router.cse[in].get_north();
			//			next_port=2;
			//			prev_loc=in;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			//South port
			//			next=router->router.cse[in].get_south();
			//			next_port=4;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			
			//			//East port, Reflection
			//			next=router->router.cse[in].get_east();
			//			next_port=3;
			//			if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*pow(router->Lp1,2)*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*pow(router->Lp1,2)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);
			//			}
			//		}
			//	}
			//	else //Current port is South
			//	{
			//		if(order>0)
			//		{
			//			//West port
			//			next=router->router.cse[in].get_west();
			//			next_port=1;
			//			prev_loc=in;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			//East port
			//			next=router->router.cse[in].get_east();
			//			next_port=3;
			//			//router->res.path[count]=next;
			//			//router->res.cur_port[count]=port_current;
			//			//2*MR_D
			//			//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//			if(router->router.cse[in].get_CSE_MR_Loc()==2 || router->router.cse[in].get_CSE_MR_Loc()==3)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->K1*router->Lp1*prp_loss));
			//				//router->res.PL[count]="K1 Lp1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K1*router->Lp1*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//				router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss));
			//			//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K2+pow(router->Lp1,2)*router->K1)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1)
			//			{
			//				prp_loss=pow(router->Lp,(6*router->MR_Dimension+3*router->WGD_width)*0.0001);
			//				router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss));
			//				//	router->res.PL[count]="K1";
			//				//++count;
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*(router->K1+pow(router->Lc,2)*router->K2)*prp_loss,prev_loc);
			//			}
			//			
			//			//South port, Reflection
			//			next=router->router.cse[in].get_south();
			//			next_port=4;
			//			if(router->router.cse[in].get_CSE_MR_Loc()==3 || router->router.cse[in].get_CSE_MR_Loc()==4)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*pow(router->Lp1,2)*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*pow(router->Lp1,2)*prp_loss,prev_loc);
			//			}
			//			else if(router->router.cse[in].get_CSE_MR_Loc()==1 || router->router.cse[in].get_CSE_MR_Loc()==2)
			//			{
			//				prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);
			//				router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Kr*prp_loss));
			//				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->Kr*prp_loss,prev_loc);
			//			}
			//		}
			//	}

			//	if(port_current==1)
			//	{
			//		next=router->router.cse[in].get_east();
			//		next_port=3;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+power);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+(power*router->Lp1*prp_loss));
			//	}
			//	else if(port_current==2)
			//	{
			//		next=router->router.cse[in].get_south();
			//		next_port=4;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+power);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+(power*router->Lc1*prp_loss));
			//	}
			//	else if(port_current==3)
			//	{
			//		next=router->router.cse[in].get_west();
			//		next_port=1;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_e_pwr(router->router.cse[in].get_e_pwr()+power);
			//		router->router.cse[in].set_w_pwr(router->router.cse[in].get_w_pwr()+(power*router->Lc1*prp_loss));
			//	}
			//	else if(port_current==4)
			//	{
			//		next=router->router.cse[in].get_north();
			//		next_port=2;
			//		prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//		router->router.cse[in].set_s_pwr(router->router.cse[in].get_s_pwr()+power);
			//		router->router.cse[in].set_n_pwr(router->router.cse[in].get_n_pwr()+(power*router->Lc1*prp_loss));
			//	}
			//	//router->res.path[count]=next;
			//	//router->res.PL[count]="Lc1";
			//	//router->res.cur_port[count]=port_current;
			//	//++count;
			//	//Length of a waveguide crossing is equal to 2*MR_D
			//	//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);
			//	prp_loss=pow(router->Lp,(2*router->MR_Dimension+router->WGD_width)*0.0001);
			//	prev_loc=in;
			//	find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->Lc1*prp_loss,prev_loc);
			//}
		}
		else if(router->router.elm[in].type==6) // Parallel BOSE
		{//cout<<endl<<router->router.pse[in].get_PSE_MR()<<endl;
			if(router->router.pse[in].get_PSE_MR()==1) //If the PSE is in ON state make turn
			{
				if(port_current==1) //Input port
				{
					if(order>0)
					{
						//Through port
						next=router->router.pse[in].get_through();
						next_port=3; 
						prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
						router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+(power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss));
						//Turning length in PSE is 2*MR_D
						//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K3";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//	count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==2) //Drop port
				{
					if(order>0)
					{
						//Add port
						next=router->router.pse[in].get_add();
						next_port=4; 
						prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
						router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+(power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss));
						//Turning length in PSE is 2*MR_D
						//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K3";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==3) //Through port
				{
					if(order>0)
					{
						//Input port
						next=router->router.pse[in].get_input();
						next_port=1;
						prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
						router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+(power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss));
						//Turning length in PSE is 2*MR_D
						//router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K3";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==4) //Add port
				{
					if(order>0)
					{
						//Drop port
						next=router->router.pse[in].get_drop();
						next_port=2; 
						prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
						router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+(power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss));
						//Turning length in PSE is 2*MR_D
						//	router->w.set_length(router->w.get_length()+2*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K3";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*router->K3*pow(router->Lp1,wdm_no-1)*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				//Turning length in PSE is 4*MR_D
				//router->w.set_length(router->w.get_length()+4*router->MR_Dimension+router->WGD_width);

				//router->res.PL[count]="Lp2";

				//Turning decision is already made
				//router->res.path[count]=next;
				//router->res.cur_port[count]=port_current;
				//count++;
				if(port_current==1)
				{
					next=router->router.pse[in].get_drop();
					next_port=2; 
					prp_loss=pow(router->Lp,((4+2*(wvlth_no-1))*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+power);
					router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+(power*router->Lp2*prp_loss));
				}
				else if(port_current==2)
				{
					next=router->router.pse[in].get_input();
					next_port=1; 
					prp_loss=pow(router->Lp,((4+2*(wvlth_no-1))*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+power);
					router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+(power*router->Lp2*prp_loss));
				}
				else if(port_current==3)
				{
					next=router->router.pse[in].get_add();
					next_port=4;
					prp_loss=pow(router->Lp,((4+2*(wdm_no-wvlth_no))*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+power);
					router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+(power*router->Lp2*prp_loss));
				}
				else if(port_current==4)
				{
					next=router->router.pse[in].get_through();
					next_port=3;
					prp_loss=pow(router->Lp,((4+2*(wdm_no-wvlth_no))*router->MR_Dimension+router->WGD_width)*0.0001);
					router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+power);
					router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+(power*router->Lp2*prp_loss));
				}
				//prp_loss=pow(router->Lp,(4*router->MR_Dimension+router->WGD_width)*0.0001);
				prev_loc=in;
				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*router->Lp2*prp_loss,prev_loc,wvlth_no,wdm_no);
			}
			else //If MR is OFF
			{
				if(port_current==1) //a->c
				{
					if(order>0)
					{
						//Drop port
						next=router->router.pse[in].get_drop();
						next_port=2; 
						prp_loss=pow(router->Lp,((4+2*(wvlth_no-1))*router->MR_Dimension+router->WGD_width)*0.0001);

						double noise=1;
						noise*=router->K2*pow(router->Lp1,2*(wvlth_no-1));
						for(int j=1;j<=wdm_no;j++)
							if(j!=wvlth_no)
								noise+=pow(router->Lp1,2*(j-1))*MR_Drop_Noise_rtr((wvlth_no-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,(j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,((j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range)/(2*router->MR_Q));

						router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+(power*noise*prp_loss));
						//Turning length in PSE is 4*MR_D
						//router->w.set_length(router->w.get_length()+4*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K2";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*noise*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==2) //b->d
				{
					if(order>0)
					{
						//Input port
						next=router->router.pse[in].get_input();
						next_port=1; 
						prp_loss=pow(router->Lp,((4+2*(wvlth_no-1))*router->MR_Dimension+router->WGD_width)*0.0001);

						double noise=1;
						noise*=router->K2*pow(router->Lp1,2*(wvlth_no-1));
						for(int j=1;j<=wdm_no;j++)
							if(j!=wvlth_no)
								noise+=pow(router->Lp1,2*(j-1))*MR_Drop_Noise_rtr((wvlth_no-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,(j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,((j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range)/(2*router->MR_Q));



						router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+(power*noise*prp_loss));
						//Turning length in PSE is 4*MR_D
						//router->w.set_length(router->w.get_length()+4*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K2";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*noise*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==3) //c->1
				{
					if(order>0)
					{
						//Add port
						next=router->router.pse[in].get_add();
						next_port=4; 
						prp_loss=pow(router->Lp,((4+2*(wdm_no-wvlth_no))*router->MR_Dimension+router->WGD_width)*0.0001);

						double noise=1;
						noise*=router->K2*pow(router->Lp1,2*(wdm_no-wvlth_no));
						for(int j=1;j<=wdm_no;j++)
							if(j!=wvlth_no)
								noise+=pow(router->Lp1,2*(wdm_no-j))*MR_Drop_Noise_rtr((wvlth_no-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,(j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,((j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range)/(2*router->MR_Q));

						router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+(power*noise*prp_loss));
						//Turning length in PSE is 4*MR_D
						//router->w.set_length(router->w.get_length()+4*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K2";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*noise*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				else if(port_current==4) //d->b
				{
					if(order>0)
					{
						//Through port
						next=router->router.pse[in].get_through();
						next_port=3; 
						prp_loss=pow(router->Lp,((4+2*(wdm_no-wvlth_no))*router->MR_Dimension+router->WGD_width)*0.0001);

						double noise=1;
						noise*=router->K2*pow(router->Lp1,2*(wdm_no-wvlth_no));
						for(int j=1;j<=wdm_no;j++)
							if(j!=wvlth_no)
								noise+=pow(router->Lp1,2*(wdm_no-j))*MR_Drop_Noise_rtr((wvlth_no-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,(j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range,((j-1)*(router->FSR/wdm_no)+router->MR_wvlgth_range)/(2*router->MR_Q));


						router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+(power*noise*prp_loss));
						//Turning length in PSE is 4*MR_D
						//router->w.set_length(router->w.get_length()+4*router->MR_Dimension+router->WGD_width);

						//router->res.PL[count]="K2";
						//Turning decision is already made
						//router->res.path[count]=next;
						//router->res.cur_port[count]=port_current;
						//count++;
						prev_loc=in;
						find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order-1,power*noise*prp_loss,prev_loc,wvlth_no,wdm_no);
						//--order;
					}
				}
				if(port_current==1)
				{
					next=router->router.pse[in].get_through();
					next_port=3;
					prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
					router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+power);
					router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+(power*pow(router->Lp1,wdm_no)*prp_loss));
				}
				else if(port_current==2)
				{
					next=router->router.pse[in].get_add();
					next_port=4;
					prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
					router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+power);
					router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+(power*pow(router->Lp1,wdm_no)*prp_loss));
				}
				else if(port_current==3)
				{
					next=router->router.pse[in].get_input();
					next_port=1;
					prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
					router->router.pse[in].set_t_pwr(router->router.pse[in].get_t_pwr()+power);
					router->router.pse[in].set_in_pwr(router->router.pse[in].get_in_pwr()+(power*pow(router->Lp1,wdm_no)*prp_loss));
				}
				else if(port_current==4)
				{
					next=router->router.pse[in].get_drop();
					next_port=2;
					prp_loss=pow(router->Lp,((wdm_no+1)*router->MR_Dimension)*0.0001);
					router->router.pse[in].set_ad_pwr(router->router.pse[in].get_ad_pwr()+power);
					router->router.pse[in].set_d_pwr(router->router.pse[in].get_d_pwr()+(power*pow(router->Lp1,wdm_no)*prp_loss));
				}
				//Passing length of PSE is 2*MR_D
				//prp_loss=pow(router->Lp,(2*router->MR_Dimension)*0.0001);

				//router->res.PL[count]="Lp1";
				//Turning decision is already made
				//router->res.path[count]=next;
				//router->res.cur_port[count]=port_current;
				//count++;
				prev_loc=in;
				find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*pow(router->Lp1,wdm_no)*prp_loss,prev_loc,wvlth_no,wdm_no);
			}
		}
		else if(router->router.elm[in].type==7) //Extra waveguide
		{
			if(router->router.wgd[in].get_out()!=prev_loc)
			{
				next=router->router.wgd[in].get_out();
				next_port=2;
			}
			else
			{
				next=router->router.wgd[in].get_in();
				next_port=1;
			}
			router->router.wgd[in].set_in_pwr(router->router.wgd[in].get_in_pwr()+power);
			prp_loss=pow(router->Lp,(router->router.wgd[in].get_length())*0.0001);
			router->router.wgd[in].set_out_pwr(router->router.wgd[in].get_out_pwr()*(power*prp_loss));
			//router->res.path[count]=next;
			//router->res.PL[count]="Wgd";
			//router->res.cur_port[count]=port_current;
			//count++;

			//Length of the extra waveguide is specified by the user
			//router->w.set_length(router->w.get_length()+router->router.wgd[in].get_length());
			prev_loc=in;
			find_path_rtrfunc(next,router->router.elm[in].terminals[next_port],order,power*prp_loss,prev_loc,wvlth_no,wdm_no);
		}
	}
	//for(int i=0;i<router->router.number_of_ports;i++)
	//cout<<"\nPower at Prt"<<router->router.port_type[i]<<" = "<<output_pwrs[router->router.port_type[i]];	
}

void Router_func::show_output_pwrs(char ch,int wvlgth_no,ofstream& file_result)
{
	if(ch=='f')//Write the results to file
	{
		file_result<<"\nCLAP analyzed the following output powers when the crosstalk noise order is equal to "<<xtalk_order<<" for the wavelength ["<<wvlgth_no<<"]:\n";
		for(int i=0;i<router->router.number_of_ports;i++)
			file_result<<"\nOptical power at port with id "<<router->router.port_type[i]<<" = "<<output_pwrs[router->router.port_type[i]]<<" mw";
		file_result<<endl;
	}
	else if(ch=='s')//Show the results on the screen
	{
		cout<<"\nCLAP analyzed the following output powers when the crosstalk noise order is equal to "<<xtalk_order<<" for the wavelength ["<<wvlgth_no<<"] :\n";
		for(int i=0;i<router->router.number_of_ports;i++)
			cout<<"\nOptical power at port with id "<<router->router.port_type[i]<<" = "<<output_pwrs[router->router.port_type[i]]<<" mw";
		cout<<endl;
	}
}

void Router_func::optical_router_analyzer(char ch)
{
	ofstream file_result;
	
	if(ch=='f')
	{
		file_result.open("ORouter_Analyzer_Results.txt",ios::out);
		if(!file_result.is_open())
		{
			cout<<"\error: the orouter_analyzer_results file cannot be created!\n";
			exit(1);
		}
		file_result<<"\n########## Optical Router Analyzer Results ##########\n";
	}
		
	if(ch=='s')
		cout<<"\n########## Optical Router Analyzer Results ##########\n";

	for(int i=1;i<=wdm_num;i++)
	{
		Router_func analyzed_rtr;
		int inprt,outprt;
		analyzed_rtr.set_mrs();
		router_config.open("Router_Configuration.txt",ios::in);
		if(!router_config.is_open())
		{
			cout<<"\nERROR: Router Configuration file could not be opened!\n";
			exit(1);
		}
		string str,dummy;
		int loc;
		//Start reading input text file
		while(!router_config.eof())
		{
			getline(router_config,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str=="config_start")
				break;
		}

		while(!router_config.eof())
		{
			loc=0;
			getline(router_config,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str.compare(0,5,"config_end")==0)
				break;
			else if(str.compare(0,4,"from")==0)	
			{
				//Read the input port
				while(str[str.find("from")+4+loc]!='t')
					loc++;
				dummy.assign(str,str.find("from")+4,loc);
				inprt=atoi(dummy.c_str());
				loc=0;
				dummy.erase();
				//cout<<"\ninput="<<inprt<<endl;
				while(str[str.find("to")+2+loc]!=';')
					loc++;
				dummy.assign(str,str.find("to")+2,loc);
				outprt=atoi(dummy.c_str());
				//Start the router function
				//Find the input power
				double inpwr;
				inpwr=analyzed_rtr.router->router.port[inprt].get_pwr();
				analyzed_rtr.find_path_rtrfunc(inprt,outprt,xtalk_order,inpwr,-1,i,wdm_num);
			}
		}
		router_config.clear();
		router_config.close();
		analyzed_rtr.show_output_pwrs(ch,i,file_result);
	}
	file_result.close();
}
