/*********************************************************************************
 *
 * File name:       router_structure.h
 * Version:         5.0
 * Software:        CLAP
 * Author:          Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
 * Website:         http://www.ece.ust.hk/~eexu
 *
 * The copyright information of this program can be found in the file COPYRIGHT.
 *
 *********************************************************************************/
#include "basic_elements.h"
#include <fstream>
#include <iostream>
#include <string>
using namespace std;

#ifndef router_structure
#define router_structure

struct Element
{
	int id;
	int type;
	int terminals[5];
};

class Router_Struct
{
public:

	ifstream router_struct;

	MR* mr; //MRs
	Port* port; //ports
	Crossing* wc; //Crossings
	Bending* wb; //Bendings
	Terminator* ot; //Terminators
	OPN* coupler; //Optical Pins
	CSE* cse; //Crossing Basic Optical Switching Elements (Crossing BOSEs)
	PSE* pse; //Parallel Basic Optical Switching Elements (Parallel BOSEs)
	Waveguide* wgd; //Extra waveguide needs to construct the router structure
	
	int tech_profile_number; // Technology profile number
	int number_of_MRs; // Number of Microresonators
	int number_of_ports; // Number of ports
	int number_of_crossings; // Number of waveguide crossings
	int number_of_bendings; // Number of waveguide bendings
	int number_of_terminators; // Number of optical terminators
	int number_of_optical_pins; //Number of optical pins
	int number_of_PSEs; // Number of parallel BOSEs
	int number_of_CSEs; // Number of crossing BOSEs
	int number_of_WGDs; //Number of extra waveguides used to construct router structure
	int total_number; // Totall number of elements in the router structure (All excluding MRs)

	//input ports information, -1 for nothing, 1 =a, 2=b, 3=c,4=d
	Element* elm;
	//This helps to easily find each port type
	int* port_type; //Injection=0; Ejection=1; in_North=2; out_North=3; in_East=4; out_East=5; in_South=6; out_South=7; in_West=8; out_West=9;

	Router_Struct();
	~Router_Struct();

	int get_tech_profile(){return tech_profile_number;}
	int get_number_of_MRs(){return number_of_MRs;}
	int get_number_of_ports(){return number_of_ports;}
	int get_number_of_crossings(){return number_of_crossings;}
	int get_number_of_bendings(){return number_of_bendings;}
	int get_number_of_terminators(){return number_of_terminators;}
	int get_number_of_PSEs(){return number_of_PSEs;}
	int get_number_of__CSEs(){return number_of_CSEs;}
	int get_number_of__WGDs(){return number_of_WGDs;}
	int get_total_number(){return total_number;}
	int get_number_of_optical_pins(){return number_of_optical_pins;}


	//set up the structure
	void set_up_structure(int in,int out);
};

#endif
