/*********************************************************************************
*
* File name:        network_structure.cpp
* Version:          5.0
* Software:         CLAP
* Author:           Luan H.K. Duong (HKUST), Jiang Xu (HKUST)
* Website:          http://www.ece.ust.hk/~eexu
*
* The copyright information of this program can be found in the file COPYRIGHT.
**
*********************************************************************************/
#include "network_structure.h"
#include <iostream>
#include <cstdlib>
#include <string>

using namespace std;

Network::Network()
{
	string str,dummy;
	int loc;
	net_config.open("Network_Configuration.txt",ios::in);
	if(!net_config.is_open())
	{
		cout<<"\nThe Network_Configuration.txt file is not open!\n";
		exit(1);
	}
	else
	{
		while(!net_config.eof())
		{
			getline(net_config,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str=="com_pattern start")
				break;
			else if(str.compare(0,2,"M=")==0) //Network Dimension, M
			{
				loc=0;
				while(str[str.find("M=")+2+loc]!=';')
					loc++;
				dummy.assign(str,str.find("M=")+2,loc); // read the size M to a dummy string --> then store it to M
				size_M=atoi(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,2,"N=")==0) //Network Dimension, N
			{
				loc=0;
				while(str[str.find("N=")+2+loc]!=';')
					loc++;
				dummy.assign(str,str.find("N=")+2,loc); // read the size N to a dummy string --> then store it to N
				size_N=atoi(dummy.c_str());
				dummy.erase();
			}
			// M x N === M is the number of column, and N is the number of row!
			else if(str.compare(0,10,"chip_size=")==0) //Chip Size which is in cm2
			{
				loc=0;
				while(str[str.find("chip_size=")+10+loc]!=';')
					loc++;
				dummy.assign(str,str.find("chip_size=")+10,loc);
				chip_size=atoi(dummy.c_str());
				dummy.erase();
			}
		}
	}
	net_config.close();
	hop_length=sqrt((chip_size)/(size_M*size_N)); //Approximation of the hop_length
	hop_length=hop_length*10000; //Convert to micro-meter
	//cout<<size_M<<" "<<size_N<<" "<<chip_size<<" "<<hop_length;
}

Network::~Network()
{
	delete [] PL.Link_path;
	delete [] PL.Link_Power_loss;
	delete [] nodes_on_the_path;
}

//Calculate the Power Loss of the link
void Network::Power_loss_Mesh(int start_x,int start_y,int end_x,int end_y,int wvlgth_no,int wdm_no) //MR_D is the dimension of a microresonator and hop_length is the hop length
{
	src.x=start_x;
	src.y=start_y;
	des.x=end_x;
	des.y=end_y;
	routers_on_the_link = abs(des.x-src.x) + abs(des.y-src.y)+1; //Number of optical routers on the link
	nodes_on_the_path=new router_node[routers_on_the_link]; //Nodes on the link

	for(int i=0;i<routers_on_the_link;i++)
	{
		nodes_on_the_path[i].x=-1;
		nodes_on_the_path[i].y=-1;
		nodes_on_the_path[i].status=-1;
		nodes_on_the_path[i].net_router= new Router_Analyzer(-1,-1); //This will setup the structure
	}
	int total_num_element=nodes_on_the_path[0].net_router->router.total_number;
	PL.Link_Power_loss=new string[routers_on_the_link*total_num_element]; //Power loss of the link
	PL.Link_path=new int[routers_on_the_link*total_num_element]; //Elements on the path of the link

	for(int i=0;i<routers_on_the_link*total_num_element;i++)
	{
		PL.Link_Power_loss[i]="$";
		PL.Link_path[i]=-1;
	}

	int router_status; //Router status, there are totally 16 different statuses 

	int counter1,counter2;
	counter1=counter2=0;

	node current_node;
	current_node.x=src.x;
	current_node.y=src.y;

	//For WDM we need to reset the propagation loss
	wg_net.set_length(0);

	//Calculate the power loss based on the XY routing algortihm
	for(int i=0;i<routers_on_the_link;i++)
	{
		if(current_node.x<des.x || current_node.x>des.x) 
		{
			if(current_node.y<des.y) //Destination is on the right side
			{
				if(i==0) //The first optical router
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					//The waveguide length includes the waveguide inside the optical router 
					//and the waveguide connect the current optical router to the next one
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length); 
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				++current_node.y;
			}
			else if(current_node.y>des.y) //Destination is on the left side
			{
				if(i==0)
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				--current_node.y;
			}
			else //Same y
			{
				if(current_node.x<des.x)
				{
					if(i==0)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //Injection to South
						router_status=3;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if(current_node.y==des.y && src.y<des.y && src.x<des.x && current_node.x==src.x)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //West to South
						router_status=16;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if(current_node.y==des.y && src.y>des.y && src.x<des.x && current_node.x==src.x)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //East to South
						router_status=9;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //North to South
						router_status=6;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}


					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					++current_node.x;
				}
				else if(current_node.x>des.x)
				{
					if(i==0)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //Injetion to North
						router_status=1;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if(current_node.y==des.y && src.y<des.y && src.x>des.x && current_node.x==src.x)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //West to North
						router_status=14;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if(current_node.y==des.y && src.y>des.y && src.x>des.x && current_node.x==src.x)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //East to North
						router_status=8;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //South to North
						router_status=12;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}

					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					--current_node.x;
				}
			}
		}
		else if(current_node.x==des.x) //Same X
		{
			if(current_node.y<des.y)
			{
				if(i==0)
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				++current_node.y;
			}
			else if(current_node.y>des.y)
			{
				if(i==0)
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				--current_node.y;
			}
			else //Destination 
			{
				if(current_node.x>src.x)
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //North to Ejection
					router_status=5;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
				}
				else if(current_node.x<src.x)
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //South to Ejection
					router_status=11;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
				}
				else
				{
					if(current_node.y>src.y)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //West to Ejection
						router_status=13;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
					}
					else if(current_node.y<src.y)
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //East to Ejection
						router_status=7;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
					}
				}
				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;
			}
		}
		copy_res(nodes_on_the_path[i].net_router->get_res().PL,PL.Link_Power_loss,nodes_on_the_path[i].net_router->get_res().path,PL.Link_path,counter1,counter2);
	}
}
void copy_res(string ar1[], string ar2[],int ar3[],int ar4[],int& count1,int& count2) //Copyr array 1 to array 2
{
	int i=0;
	while(ar1[i]!="$")
		ar2[count1++]=ar1[i++];
	i=0;
	while(ar3[i]!=-1)
		ar4[count2++]=ar3[i++];
}
//The power loss function needs to be called before the crosstalk noise function
void Network::crosstalk_Mesh(long double &noise,long double &coh_xtalk,long double &pow_loss,int wvlgth_no,int wdm_no)
{
	string dummy;
	string* str;
	node sender,receiver;
	//Start reading the communication pattern file
	Network sub_network; //This network helps for crosstalk analyzing
	int total_num_elements=nodes_on_the_path[0].net_router->router.total_number;
	int counter=0;

	crosstalk_str= new string [size_M*size_N*total_num_elements];
	for(int i=0;i<size_M*size_N*total_num_elements;i++)
		crosstalk_str[i]="$";

	results_crosstalk.open("Crosstalk_Noise_Power_Equation.txt",ios::out);
	noise=0;
	long double crosstalk_noise_pow=1;
	pow_loss=1;
	string crosstalk_coef;
	int com_el;	
	str=new string[total_num_elements];

	double MR_D=nodes_on_the_path[0].net_router->MR_Dimension;
	double Prop_loss=nodes_on_the_path[0].net_router->Lp;
	double WG_w=nodes_on_the_path[0].net_router->WGD_width;
	//////////////////////////////////////////////////////////////////////////
	//Copy the power loss results to the output textfiles
	results_power_loss.open("Signal_Power_Equation.txt",ios::out);
	if(results_power_loss.is_open())
	{
		for(int i=0;i<routers_on_the_link;i++)
			for(int j=0;j<total_num_elements;j++)
			{
				string dummy=nodes_on_the_path[i].net_router->get_res().PL[j];
				if(dummy=="$")
					break;
				else if(dummy!="P0_1" && dummy!="P0_2" && dummy!="P0_3" && dummy!="P0_4" && dummy!="Wgd")
				{
					if(dummy=="P0_0")
					{
						//Add the power loss from the Modulator, please see the document for more information
						pow_loss*=nodes_on_the_path[0].net_router->input_power*pow(nodes_on_the_path[0].net_router->Lp1,wdm_no-wvlgth_no+1)*nodes_on_the_path[0].net_router->Lp2*pow(nodes_on_the_path[0].net_router->Lb,2);//Modulation loss, please see the document
						dummy="Lp2 Lb^2* P0_0 *Lp1^";
						dummy+=nodes_on_the_path[0].net_router->convertInt(wdm_no-wvlgth_no+1);
						dummy+="@ *";
						wg_net.set_length(wg_net.get_length()+8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width);//Approximate modulation propagation loss (see the document)
					}
					else if(dummy=="Lc")
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
					else if(dummy=="Lb")
						pow_loss*=nodes_on_the_path[0].net_router->Lb;
					else if(dummy=="(K1)" || dummy=="K1")
						pow_loss*=nodes_on_the_path[0].net_router->K1;
					else if(dummy=="Kr")
						pow_loss*=nodes_on_the_path[0].net_router->Kr;
					else if(dummy=="Kt")
						pow_loss*=nodes_on_the_path[0].net_router->Kt;
					else if(dummy=="Lp1")
						pow_loss*=nodes_on_the_path[0].net_router->Lp1;
					else if(dummy=="Lp2")
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
					else if(dummy=="(Lc Lp2 Lc)")
						pow_loss*=nodes_on_the_path[0].net_router->full_Lc2;
					else if(dummy=="(K2)" || dummy=="K2")
						pow_loss*=nodes_on_the_path[0].net_router->K2;
					else if(dummy=="(K3)" || dummy=="K3")
						pow_loss*=nodes_on_the_path[0].net_router->K3;
					else if(dummy=="Lc1")
						pow_loss*=nodes_on_the_path[0].net_router->Lc1;
					else if(dummy=="Lc2")
						pow_loss*=nodes_on_the_path[0].net_router->Lc2;
					else if(dummy.compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("(Lc Lp2 Lc) Lp1^")+16+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("(Lc Lp2 Lc) Lp1^")+16,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,8,"Lp2 Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp2 Lp1^")+8+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp2 Lp1^")+8,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,7,"Lc Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lc Lp1^")+7+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lc Lp1^")+7,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,4,"Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp1^")+4+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp1^")+4,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					//write to the file
					results_power_loss<<dummy<<" * ";
				}
			}
	}//We will add the power loss from the detectors and propagation loss later (To this point, this is the signal power before detection)
	///////////////////////////////////////////////////////////////////////////////////

	//Coherent noise
	coh_xtalk=0;
	for(int i=0;i<routers_on_the_link;i++)
		coh_xtalk+=nodes_on_the_path[i].net_router->coh_noise;
	//add the coherent crosstalk at modulation --- Matrix method replaced
	coh_xtalk+=pow(nodes_on_the_path[0].net_router->K2,2)/nodes_on_the_path[0].net_router->Lp1;
	for(int i=wvlgth_no+1;i<=wdm_no;i++)
		coh_xtalk+=(nodes_on_the_path[0].net_router->K2*
		MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1);
	//add the coherent crosstalk at detection --- Matrix method replaced
	for(int i=1;i<wvlgth_no;i++)
		coh_xtalk+=nodes_on_the_path[0].net_router->K2*nodes_on_the_path[0].net_router->Kd1[i];
        
//	for(int i=1;i<wvlgth_no;i++)
//		coh_xtalk+=nodes_on_the_path[0].net_router->K2*
//		MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
//		(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
//		((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1;
	///////////////////////////////////////////////////////////////////////////////////
	
	//Reading the communication pattern
	net_config.open("Network_Configuration.txt",ios::in);
	if(!net_config.is_open())
	{
		cout<<"\nThe Network_Configuration.txt file is not open!\n";
		exit(1);
	}
	else
	{
		getline(net_config,dummy);
		while(dummy.compare(0,17,"com_pattern start")!=0 && !net_config.eof())
			getline(net_config,dummy);

		double w_length;

		while(!net_config.eof())
		{
			getline(net_config,dummy);
			while((dummy.length()!=0 && dummy[0]=='/' && dummy[1]=='/') || dummy.length()==0 && !net_config.eof()) //Skip the commments and empty lines
				getline(net_config,dummy);
			if(net_config.eof() || dummy=="com_pattern end")
				break;
			find_sender_receiver(dummy,sender,receiver); //Find the sender and receiver defined by the user
			sub_network.wg_net.set_length(0);
			sub_network.Power_loss_Mesh(sender.y,sender.x,receiver.y,receiver.x,wvlgth_no,wdm_no);

			//Find common nodes
			for(int j=0;j<routers_on_the_link;j++) //j is the counter for the main link
				for(int k=0;k<sub_network.routers_on_the_link;k++) //k is the counter for the noise inroducing link
					if(nodes_on_the_path[j].x==sub_network.nodes_on_the_path[k].x && nodes_on_the_path[j].y==sub_network.nodes_on_the_path[k].y) //Common Node
					{
						//Find the common element in the router structure
						for(int m=0;m<total_num_elements;m++) 
							for(int n=0;n<total_num_elements;n++)
								if(nodes_on_the_path[j].net_router->get_res().path[m]==sub_network.nodes_on_the_path[k].net_router->get_res().path[n]) //The common element in the router structure
								{
									if(nodes_on_the_path[j].net_router->get_res().path[m]==-1 || sub_network.nodes_on_the_path[k].net_router->get_res().path[n]==-1)
										break;
									else
									{
										//Need to consider the common element type (CSE, PSE, or Crossing) and finally assign the crosstalk coefficient for it
										com_el=sub_network.nodes_on_the_path[k].net_router->get_res().path[n];

										for(int c=0;c<total_num_elements;c++)
											str[c]="$";

										for(int c=0;c<size_N*size_M*total_num_elements;c++)
											crosstalk_str[c]="$";

										w_length=0;
										counter=0;
										crosstalk_noise_pow=1; 
										if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==2) //The common element is crossing
										{
											crosstalk_coef="(K1)";
											w_length+=2*MR_D+WG_w;
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==5 && wdm_no==1) //The common element is CSE and Using a single wavelength
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) //2D
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";	//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";	
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
											}
											else // If the MR is ON in the CSE
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1 || 
													sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 1 or 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
													{
														crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
														//propagation loss
														w_length+=6*MR_D+2*WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) 
													{
														crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
														//propagation loss
														w_length+=6*MR_D+3*WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2 || 
													sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 2 or 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
													{
														crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
														//propagation loss
														w_length+=6*MR_D+2*WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) 
													{
														crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
														//propagation loss
														w_length+=6*MR_D+3*WG_w;
													}
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==5 && wdm_no>1) //The common element is CSE and Using WDM
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
														crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1 * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
														crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1 * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
														crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
														crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) //2D
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
														crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1 * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
														crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1 * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
														crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
														crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
												}
											}
											else // If the MR is ON in the CSE, consider first-order crosstalk noise only
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1 * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1 * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1 * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1 * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==6 && wdm_no==1) //The common element is PSE and using a Single wavelength
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_coef="(K2)";
													w_length+=4*MR_D+WG_w;
												}
											}
											else //If the MR is ON
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_coef="(K3)";
													w_length+=2*MR_D;
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==6 && wdm_no>1) //The common element is PSE and using WDM
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,7,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=7
													crosstalk_coef="Func_no=7";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
													w_length+=4*MR_D+2*(wdm_no-wvlgth_no)*MR_D+WG_w;
												}
												else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,8,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=8
													crosstalk_coef="Func_no=8";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
													w_length+=4*MR_D+2*(wvlgth_no-1)*MR_D+WG_w;
												}
												
											}
											else //If the MR is ON
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,9,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=9
													crosstalk_coef="Lp1 * K3";
													//propagation loss (estimation for WDM case in this case)
													w_length+=(wdm_no+1)*MR_D;
												}
											}
										}


										int p,q;
										for(p=0;p<n+1;p++)
											str[p]=sub_network.nodes_on_the_path[k].net_router->get_res().PL[p]; //The path before the incident from the link introduction link
										str[p++]=crosstalk_coef; //The crosstalk coefficient
										for(q=m+2;q<total_num_elements && nodes_on_the_path[j].net_router->get_res().PL[q]!="$";q++)
											str[p++]=nodes_on_the_path[j].net_router->get_res().PL[q]; //The path after the incident from the main link

										////Calculate the propagation length
										for(int y=j+1;y<routers_on_the_link;y++)
											w_length+=nodes_on_the_path[y].net_router->w.get_length(); //Consider the propagation loss in the optical routers located on the main link
										//Consider the propagation loss of the main link
										w_length+=(abs(nodes_on_the_path[j].x-nodes_on_the_path[routers_on_the_link-1].x)+abs(nodes_on_the_path[j].y-nodes_on_the_path[routers_on_the_link-1].y))*hop_length; 
										//Consider the propagation loss of the noise introduction link
										for(int i=0;i<k;i++)
											w_length+=sub_network.nodes_on_the_path[i].net_router->w.get_length(); 
										w_length+=(abs(nodes_on_the_path[j].x-sub_network.nodes_on_the_path[0].x)+abs(nodes_on_the_path[j].y-sub_network.nodes_on_the_path[0].y))*hop_length;

										//Add the propagation loss inside the optical router
										for(int z=0;z<total_num_elements;z++)
											if(str[z]=="$")
												break;
											else
											{
												if(str[z]=="Lc"|| str[z]=="Lc1" || str[z]=="Lc2")
													w_length+=2*MR_D+WG_w;
												else if(str[z]=="Lp1")
													w_length+=2*MR_D;
												else if(str[z]=="Lb")
													w_length+=MR_D+WG_w;
												else if(str[z]=="Lp2" || str[z].compare(0,8,"Lp2 Lp1")==0)
													w_length+=4*MR_D+WG_w;
												else if(str[z]=="(Lc Lp2 Lc)" || str[z].compare(0,16,"(Lc Lp2 Lc) Lp1")==0)
													w_length+=6*MR_D+3*WG_w;
												else if(str[z].compare(0,7,"Lc Lp1")==0)
													w_length+=(wdm_no+1)*MR_D+WG_w;
												else if(str[z].compare(0,4,"Lp1")==0)
													w_length+=(wdm_no+1)*MR_D;
											}
											for(int i=0;i<n;i++) //Noise link
												if(sub_network.nodes_on_the_path[k].net_router->get_res().PL[i]=="Wgd")
													w_length+=sub_network.nodes_on_the_path[k].net_router->router.wgd[sub_network.nodes_on_the_path[k].net_router->get_res().path[i-1]].get_length();
											for(int i=m+1;i<total_num_elements;i++) //Main link
												if(nodes_on_the_path[j].net_router->get_res().PL[i]=="$")
													break;
												else if(nodes_on_the_path[j].net_router->get_res().PL[i]=="Wgd")
													w_length+=nodes_on_the_path[j].net_router->router.wgd[nodes_on_the_path[j].net_router->get_res().path[i-1]].get_length();

											//Copy the result to the crosstalk_str
											for(int r=0;r<k;r++)
												for(int s=0;s<total_num_elements;s++)
													if(sub_network.nodes_on_the_path[r].net_router->get_res().PL[s]=="$")
														break;
													else
														crosstalk_str[counter++]=sub_network.nodes_on_the_path[r].net_router->get_res().PL[s]; //The Power Loss of the noise introduction link before the common node

											for(int t=0;t<total_num_elements;t++)
												if(str[t]=="$")
													break;
												else
													crosstalk_str[counter++]=str[t]; //The Power Loss inside the common node

											for(int u=j+1;u<routers_on_the_link;u++)
												for(int x=0;x<total_num_elements;x++)
													if(nodes_on_the_path[u].net_router->get_res().PL[x]=="$")
														break;
													else
														crosstalk_str[counter++]=nodes_on_the_path[u].net_router->get_res().PL[x];

											crosstalk_str[counter++]="+";

											//Wrting the result into the output textfile
											if(results_crosstalk.is_open())
											{
												int i=0;
												while(crosstalk_str[i]!="$")
												{
													if(crosstalk_str[i]!="+" && crosstalk_str[i]!="P0_1" && crosstalk_str[i]!="P0_2" && crosstalk_str[i]!="P0_3" && crosstalk_str[i]!="P0_4" && crosstalk_str[i]!="Wgd")
													{
														if(crosstalk_str[i]=="P0_0")
														{
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->input_power*nodes_on_the_path[0].net_router->Lp1*nodes_on_the_path[0].net_router->Lp2;
															crosstalk_str[i]="Lp1 Lp2 * P0_0";//for modulation
															w_length+=8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width;//modulation propagation loss
														}
														else if(crosstalk_str[i]=="Lc")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
														else if(crosstalk_str[i]=="Lb")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lb;
														else if(crosstalk_str[i]=="(K1)" || crosstalk_str[i]=="K1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K1;
														else if(crosstalk_str[i]=="Kr")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kr;
														else if(crosstalk_str[i]=="Kt")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kt;
														else if(crosstalk_str[i]=="Lp1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp1;
														else if(crosstalk_str[i]=="Lp2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
														else if(crosstalk_str[i]=="(K2)" || crosstalk_str[i]=="K2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K2;
														else if(crosstalk_str[i]=="(K3)" || crosstalk_str[i]=="K3")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K3;
														else if(crosstalk_str[i]=="Lc1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc1;
														else if(crosstalk_str[i]=="Lc2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc2;
														else if(crosstalk_str[i].compare(0,16,"(Lc Lp2 Lc) Lp1")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("(Lc Lp2 Lc) Lp1")+16+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("(Lc Lp2 Lc) Lp1")+16,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,8,"Lp2 Lp1")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lp2 Lp1")+8+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp2 Lp1")+8,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,7,"Lc Lp1")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lc Lp1")+7+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lc Lp1")+7,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,4,"Lp1")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lp1")+4+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp1")+4,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i]=="(Lp1 K1)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_1;
														else if(crosstalk_str[i]=="(K1 + Lc Lc K2)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_2;
														else if(crosstalk_str[i]=="(Lp1 Lp1 K1 + K2)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_3;
														else if(crosstalk_str[i]=="(K1 K3 + Lc Lp2 Kr K3)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_4;
														else if(crosstalk_str[i]=="(Lc K3 + Lc Lp2 K1 K3)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_5;
														else if(crosstalk_str[i]=="(Lc Lp2 Lc)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->full_Lc2;
														//Write to the file
														results_crosstalk<<crosstalk_str[i]<<" * ";
													}
													else if(crosstalk_str[i]=="+")
													{
														results_crosstalk<<"Lp";
														crosstalk_noise_pow*=pow(Prop_loss,w_length*0.0001);
														results_crosstalk<<" + ";
														noise+=crosstalk_noise_pow;
														crosstalk_noise_pow=1;
													}
													i++;
												}
											}
									}
								}
					}
		}
	}
	net_config.close();

	//power loss for noise power at detectors
        noise*=nodes_on_the_path[0].net_router->Lp1*nodes_on_the_path[0].net_router->Lp2; //Matrix method replaced
	//noise*=pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*nodes_on_the_path[0].net_router->Lp2;
	noise*=pow(Prop_loss,((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001); 
	results_crosstalk<<" * (Lp2 Ld1 * Lp^("<<((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001<<")";

	//Add the crosstalk noise at the detectors
//	if(wdm_no>1)//if using WDM
//	{
//            noise+=pow_loss*nodes_on_the_path[0].net_router->Kd1[wvlgth_no];
//		//noise+=pow_loss*pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*xtalk_coeff_cse(wvlgth_no,wdm_no,10,nodes_on_the_path[0].net_router->K1,nodes_on_the_path[0].net_router->K2,nodes_on_the_path[0].net_router->K3,nodes_on_the_path[0].net_router->Lp1,nodes_on_the_path[0].net_router->Lc,nodes_on_the_path[0].net_router->FSR,nodes_on_the_path[0].net_router->MR_Q,nodes_on_the_path[0].net_router->MR_wvlgth_range);//func_num=10
//	results_crosstalk<<" * Func_no=10 * ";
//	}


	//Add the detector at the end of the communication line
	pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*nodes_on_the_path[0].net_router->Lp2;
	results_power_loss<<"Lp2 Lp1 * ";
	wg_net.set_length(wg_net.get_length()+(3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width);
	results_power_loss<<"Lp \n**END**";
	pow_loss*=pow(Prop_loss,wg_net.get_length()*0.0001);
	results_power_loss.close();
	results_crosstalk<<"\n**END**";
	results_crosstalk.close();
	delete [] str;
	delete [] crosstalk_str;
}

void Network::SNR_Mesh(int src_x,int src_y,int rcv_x,int rcv_y,char ch,int wdm_no)
{
	long double max_xtalk_noise=-1000,max_coh_xtalk=-1000,min_signal_pow=1000,min_SNR=1000;
	int max_xtalk_no=-1, min_signal_pow_no=-1,min_SNR_no=-1,max_coh_xtalk_no=-1;
	bool show_wc_res=true;
	if(src_x>size_M || rcv_x>size_M || src_y>size_N || rcv_y>size_N || src_x<0 || src_y<0 || rcv_x<0 || rcv_y<0)// Adding the case where the user enters something less than 0
	{
		cout<<"\nERROR: The source and the destination are not within the defined network size!";
		exit(1);
	}
	if(ch=='s') // ch=s means printing the result to the screen
	{
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		cout<<"\nThe results for the optical link starts from the core ("<<src_x<<", "<<src_y<<") towards the core ("<<rcv_x<<", "<<rcv_y<<")\n in Mesh-based ONoC are as following:\n\n";
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_Mesh(src_x,src_y,rcv_x,rcv_y,i,1); //Calculate the power loss of the link
			crosstalk_Mesh(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files
			
			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				//Show the results to the user
				cout<<"Wavelength["<<i<<"] Singal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Incoherent Crosstalk Noise Power (at the destination)= "<<crosstalk_noise<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no coherent crosstalk)= "<<SNR_no_coh<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (coherent crosstalk + incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (singal power - coherent crosstalk)= "<<SNR_SP_coh<<" dB\n\n";

				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//cout<<"\nThe results for the optical link starts from the core ("<<src_y<<", "<<src_x<<") towards the core ("<<rcv_y<<", "<<rcv_x<<")\n in Mesh-based ONoC are as followings:\n\n";
				cout<<"Wavelength["<<i<<"] Signal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				cout<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wc_res=true;
			}
		}
		//Show the worst-case results for different wavelengths
		if(show_wc_res)
		{
			cout<<endl;
			cout<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			cout<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dBm\n";
			cout<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dBm\n";
			cout<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			cout<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR of (no coherent crosstalk considered) "<<min_SNR<<" dB\n\n";
		}
	}
	else if(ch=='f') // ch=f means printing the result to the file stored with the name "Network_Analyzer_Resuls.csv"
	{
		ofstream outfile("Network_Analyzer_Resuls.csv",ios::out);
		if(!outfile.is_open())
		{
			outfile<<"\nERROR: The network_analyzer_results file could not be created!\n";
			exit(1);
		}
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		outfile<<"\nThe results for the optical link starts from the core ("<<src_y<<" & "<<src_x<<") towards the core ("<<rcv_y<<" & "<<rcv_x<<") in Mesh-based ONoC are as following:"<<endl;
		outfile<<"Wavelength, Signal power (dBm), Incoherent crosstalk noise power (dBm), Coherent crosstalk (dBm), SNR (no coherent crosstalk)(dB), SNR (coherent crosstalk + incoherent crosstalk)(dB), SNR (signal power - coherent crosstalk)(dB)"<<endl;
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_Mesh(src_x,src_y,rcv_x,rcv_y,i,1); //Calculate the power loss of the link
			crosstalk_Mesh(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files

			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				//Show the results to the user
				//outfile<<"Wavelength["<<i<<"] Signal Power = "<<power_loss<<" dB\n";
				//outfile<<"Wavelength["<<i<<"] Crosstalk Noise Power (at the destination) = "<<crosstalk_noise<<" dB\n";
				//outfile<<"Wavelength["<<i<<"] SNR = "<<SNR<<" dB\n\n";
				outfile<<i<<","<<power_loss<<","<<crosstalk_noise<<","<<coh_xtalk<<","<<SNR_no_coh<<","<<SNR_ch_inc<<","<<SNR_SP_coh<<endl;

				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//outfile<<"\nThe results for the optical link starts from the core ("<<src_y<<", "<<src_x<<") towards the core ("<<rcv_y<<", "<<rcv_x<<")\n in Mesh-based ONoC are as followings:\n\n";
				outfile<<"Wavelength["<<i<<"] Signal power= "<<power_loss<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				outfile<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wc_res=true;
			}
		}
		//Show the worst-case results for different wavelengths
		if(show_wc_res)
		{
			outfile<<endl;
			outfile<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			outfile<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dBm\n";
			outfile<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dBm\n";
			outfile<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			outfile<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR (no coherent crosstalk) of "<<min_SNR<<" dB\n\n";
		}
		outfile.close();
	}
}

void Network::Power_loss_FTorus(int start_x,int start_y,int end_x,int end_y,int wvlgth_no,int wdm_no)
{
	src.x=start_x;
	src.y=start_y;
	des.x=end_x;
	des.y=end_y; 
	nodes_on_the_path=new router_node[size_M*size_N]; //Nodes on the link
	bool received=false;

	for(int i=0;i<size_M*size_N;i++)
	{
		nodes_on_the_path[i].x=-1;
		nodes_on_the_path[i].y=-1;
		nodes_on_the_path[i].status=-1;
		nodes_on_the_path[i].net_router= new Router_Analyzer(-1,-1); //This will setup the structure
	}
	int total_num_element=nodes_on_the_path[0].net_router->router.total_number;
	PL.Link_Power_loss=new string[size_M*size_N*total_num_element]; //Power loss of the link
	PL.Link_path=new int[size_M*size_N*total_num_element]; //Elements on the path of the link

	for(int i=0;i<size_M*size_N*total_num_element;i++)
	{
		PL.Link_Power_loss[i]="$";
		PL.Link_path[i]=-1;
	}

	int router_status; //Router status, there are totally 16 different statuses 

	int counter1,counter2;
	counter1=counter2=0;

	node current_node;
	current_node.x=src.x;
	current_node.y=src.y;

	//For WDM we need to reset the propagation loss
	wg_net.set_length(0);

	//Calculate the power loss based on the XY routing algortihm
	int i=0;
	while(!received)
	{
		if(current_node.x<des.x)
		{
			if((des.x-current_node.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Right
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if(((des.x-current_node.x)>(size_N/2)+1 || (des.x-current_node.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && current_node.x>size_N/2) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Right
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if(((des.x-current_node.x)>(size_N/2)+1 || (des.x-current_node.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Left
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
			}
			else if((des.x-current_node.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && current_node.x>size_N/2 ) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Left
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
			}

		}
		else if(current_node.x>des.x)
		{
			if((current_node.x-des.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Left
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}

			}
			else if(((current_node.x-des.x)>(size_N/2)+1 || (current_node.x-des.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && current_node.x>size_N/2) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Left
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //Injection to West
					router_status=4;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[9],0,wvlgth_no,wdm_no); //East to West
					router_status=10;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}

				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}

			}
			else if(((current_node.x-des.x)>(size_N/2)+1 || (current_node.x-des.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1))))//Go Right
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if((current_node.x-des.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && current_node.x>size_N/2 ) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1 )))//Go Right
			{
				if(i==0) //First Node
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //Injection to East
					router_status=2;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				else
				{
					nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5]);
					nodes_on_the_path[i].net_router->refresh();
					nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[5],0,wvlgth_no,wdm_no); //West to East
					router_status=15;
					wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
				}
				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;

				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
		}
		else //Same x
		{
			if(current_node.y<des.y)
			{
				if( ((des.y-current_node.y>(size_M/2)+1 || des.y-current_node.y==(size_M/2)+1) && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) ||
					(des.y-current_node.y<(size_M/2)+1 && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1)))) //Go Up
				{
					if(i==0) //First Node
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //Injetion to North
						router_status=1;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x<des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x>des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //West to North
						router_status=14;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x<des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x>des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						))  //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //West to North
						router_status=14;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x>des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x<des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && src.x-des.x<size_N/2+1)//Right
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //East to North
						router_status=8;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x>des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x<des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x-des.x<size_N/2+1)//Right
						)) //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //East to North
						router_status=8;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //South to North
						router_status=12;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					if((size_M*size_N)%2==0) //Even network size
					{
						if((current_node.y<size_M/2 || current_node.y==size_M/2) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
					else //Odd network size
					{
						if((current_node.y<size_M/2+1 || current_node.y==size_M/2+1) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2+1 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
				}
				else if((des.y-current_node.y<(size_M/2)+1 && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) ||
					((des.y-current_node.y>(size_M/2)+1 || des.y-current_node.y==(size_M/2)+1) && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1))))//Go Down
				{
					if(i==0) //First Node
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //Injection to South
						router_status=3;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x<des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x>des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //West to South
						router_status=16;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x<des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x>des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						))  //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //West to South
						router_status=16;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x>des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x<des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && src.x-des.x<size_N/2+1)//Right
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //East to South
						router_status=9;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x>des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x<des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x-des.x<size_N/2+1)//Right
						)) //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //East to South
						router_status=9;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //North to South
						router_status=6;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}

					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y<size_M/2 || current_node.y==size_M/2)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
					else //Odd network size
					{
						if(current_node.y<size_M/2+1 || current_node.y==size_M/2+1)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
				}
			}
			else if(current_node.y>des.y)
			{
				if(((current_node.y-des.y>(size_M/2)+1 || current_node.y-des.y==(size_M/2)+1) && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) || 
					(current_node.y-des.y<(size_M/2)+1 && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1))))//Go Down
				{
					if(i==0) //First Node
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //Injection to South
						router_status=3;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x<des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x>des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //West to South
						router_status=16;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x<des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x>des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						))  //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //West to South
						router_status=16;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x>des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x<des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && src.x-des.x<size_N/2+1)//Right
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //East to South
						router_status=9;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x>des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x<des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x-des.x<size_N/2+1)//Right
						)) //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //East to South
						router_status=9;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[7],0,wvlgth_no,wdm_no); //North to South
						router_status=6;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}

					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y<size_M/2 || current_node.y==size_M/2)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
					else //Odd network size
					{
						if(current_node.y<size_M/2+1 || current_node.y==size_M/2+1)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
				}
				else if((current_node.y-des.y<(size_M/2)+1 && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) || 
					((current_node.y-des.y>(size_M/2)+1 || current_node.y-des.y==(size_M/2)+1) && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1 ))))//Go Up
				{
					if(i==0) //First Node
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[0],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //Injetion to North
						router_status=1;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x<des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x>des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //West to North
						router_status=14;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x<des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x>des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && (des.x-src.x>size_N/2+1 || des.x-src.x==size_N/2+1)) ||//Left
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && (src.x-des.x>size_N/2+1 || src.x-des.x==size_N/2+1))//Left
						))  //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //West to North
						router_status=14;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2==0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2 || src.x==size_N/2) && (des.x<size_N/2 || des.x==size_N/2) && src.x>des.x) ||
						(src.x>size_N/2 && des.x>size_N/2 && src.x<des.x) ||
						((src.x<size_N/2 || src.x==size_N/2) && des.x>size_N/2 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2 && (des.x<size_N/2 || des.x==size_N/2) && src.x-des.x<size_N/2+1)//Right
						)) //Even size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //East to North
						router_status=8;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else if( size_M*size_N%2!=0 && src.y==current_node.y && des.x==current_node.x && 
						( ((src.x<size_N/2+1 || src.x==size_N/2+1) && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x>des.x) ||
						(src.x>size_N/2+1 && des.x>size_N/2+1 && src.x<des.x) ||
						((src.x<size_N/2+1 || src.x==size_N/2+1) && des.x>size_N/2+1 && des.x-src.x<size_N/2+1) ||//Right
						(src.x>size_N/2+1 && (des.x<size_N/2+1 || des.x==size_N/2+1) && src.x-des.x<size_N/2+1)//Right
						)) //Odd size
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //East to North
						router_status=8;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[3],0,wvlgth_no,wdm_no); //South to North
						router_status=12;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
					}

					nodes_on_the_path[i].x=current_node.x;
					nodes_on_the_path[i].y=current_node.y;
					nodes_on_the_path[i].status=router_status;

					if((size_M*size_N)%2==0) //Even network size
					{
						if((current_node.y<size_M/2 || current_node.y==size_M/2) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
					else //Odd network size
					{
						if((current_node.y<size_M/2+1 || current_node.y==size_M/2+1) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2+1 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
				}
			}
			else //Destination
			{
				int in_p;
				int j=0;
				while(nodes_on_the_path[i-1].net_router->get_res().path[j]!=-1)
					j++;
				in_p=nodes_on_the_path[i-1].net_router->get_res().path[j-1]; //This includes the input port id to the destination

				if(in_p==nodes_on_the_path[i].net_router->router.port_type[5]) //At output East
				{	
					if((size_M*size_N%2==0 && ((nodes_on_the_path[i-1].x==size_N/2  && current_node.x==size_N/2+1) || (nodes_on_the_path[i-1].x==size_N/2+1 && current_node.x==size_N/2))) ||
						(size_M*size_N%2!=0 && ((nodes_on_the_path[i-1].x==size_N/2+1  && current_node.x==size_N/2+2) || (nodes_on_the_path[i-1].x==size_N/2+2 && current_node.x==size_N/2+1))))
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //East to Ejection
						router_status=7;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;	
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //West to Ejection
						router_status=13;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
				}
				else if(in_p==nodes_on_the_path[i].net_router->router.port_type[9]) //At output West
				{
					if((nodes_on_the_path[i-1].x==1 && current_node.x==size_N) || (nodes_on_the_path[i-1].x==size_N && current_node.x==1 ))
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[8],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //West to Ejection
						router_status=13;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[4],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //East to Ejection
						router_status=7;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
				}
				else if(in_p==nodes_on_the_path[i].net_router->router.port_type[7]) //At South Output 
				{
					if((size_M*size_N%2==0 && ((nodes_on_the_path[i-1].y==size_M/2  && current_node.y==size_M/2+1) || (nodes_on_the_path[i-1].y==size_M/2+1 && current_node.y==size_M/2))) ||
						(size_M*size_N%2!=0 && ((nodes_on_the_path[i-1].y==size_M/2+1  && current_node.y==size_M/2+2) || (nodes_on_the_path[i-1].y==size_M/2+2 && current_node.y==size_M/2+1))))
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //South to Ejection
						router_status=11;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //North to Ejection
						router_status=5;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
				}
				else if(in_p==nodes_on_the_path[i].net_router->router.port_type[3])//At North Output
				{
					if((nodes_on_the_path[i-1].y==1 && current_node.y==size_M) || (nodes_on_the_path[i-1].y==size_M && current_node.y==1 ))
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[2],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //North to Ejection
						router_status=5;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
					else
					{
						nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1]);
						nodes_on_the_path[i].net_router->refresh();
						nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[6],nodes_on_the_path[i].net_router->router.port_type[1],0,wvlgth_no,wdm_no); //South to Ejection
						router_status=11;
						wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length());
						received=true;
					}
				}
				nodes_on_the_path[i].x=current_node.x;
				nodes_on_the_path[i].y=current_node.y;
				nodes_on_the_path[i].status=router_status;
			}
		}
		copy_res(nodes_on_the_path[i].net_router->get_res().PL,PL.Link_Power_loss,nodes_on_the_path[i].net_router->get_res().path,PL.Link_path,counter1,counter2);
		i++;
	}
	routers_on_the_link=i;
}
//
void Network::crosstalk_FTorus(long double &noise,long double &coh_xtalk,long double &pow_loss,int wvlgth_no,int wdm_no)
{
	string dummy;
	string* str;
	node sender,receiver;
	Network sub_network; //This network helps for crosstalk analyzing
	int total_num_elements=nodes_on_the_path[0].net_router->router.total_number;
	int counter=0;

	crosstalk_str= new string [size_M*size_N*total_num_elements];
	for(int i=0;i<size_M*size_N*total_num_elements;i++)
		crosstalk_str[i]="$";

	results_crosstalk.open("Crosstalk_Noise_Power_Equation.txt",ios::out);
	noise=0;
	long double crosstalk_noise_pow=1;
	pow_loss=1;
	string crosstalk_coef;
	int com_el;	
	str=new string[total_num_elements]; //This str is used to include the path inside the common router between the main and the noise link

	double MR_D=nodes_on_the_path[0].net_router->MR_Dimension;
	double Prop_loss=nodes_on_the_path[0].net_router->Lp;
	double WG_w=nodes_on_the_path[0].net_router->WGD_width;
	double L_c=nodes_on_the_path[0].net_router->Lc;
	double L_b=nodes_on_the_path[0].net_router->Lb;
	int sub_net_bending=0; //Bendings of the noise introduction link
	int sub_net_crossing=0; //Crossings of the noise introduction link
	int bendings=0,crossings=0; // bendings and crossings of the main link

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//Copy the power loss results to the output textfiles
	results_power_loss.open("Signal_Power_Equation.txt",ios::out);
	if(results_power_loss.is_open())
	{
		for(int i=0;i<routers_on_the_link;i++)
			for(int j=0;j<total_num_elements;j++)
			{
				string dummy=nodes_on_the_path[i].net_router->get_res().PL[j];
				if(dummy=="$")
					break;
				else if(dummy!="P0_1" && dummy!="P0_2" && dummy!="P0_3" && dummy!="P0_4" && dummy!="Wgd")
				{
					if(dummy=="P0_0")
					{
						//Add the power loss from the Modulator, please see the document for more information
						pow_loss*=nodes_on_the_path[0].net_router->input_power*pow(nodes_on_the_path[0].net_router->Lp1,wdm_no-wvlgth_no+1)*nodes_on_the_path[0].net_router->Lp2*pow(nodes_on_the_path[0].net_router->Lb,2);//Modulation loss, please see the document
						dummy="Lp2 Lb^2* P0_0 *Lp1^";
						dummy+=nodes_on_the_path[0].net_router->convertInt(wdm_no-wvlgth_no+1);
						dummy+="@ *";
						wg_net.set_length(wg_net.get_length()+8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width);//Approximate modulation propagation loss (see the document)
					}
					else if(dummy=="Lc")
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
					else if(dummy=="Lb")
						pow_loss*=nodes_on_the_path[0].net_router->Lb;
					else if(dummy=="(K1)" || dummy=="K1")
						pow_loss*=nodes_on_the_path[0].net_router->K1;
					else if(dummy=="Kr")
						pow_loss*=nodes_on_the_path[0].net_router->Kr;
					else if(dummy=="Kt")
						pow_loss*=nodes_on_the_path[0].net_router->Kt;
					else if(dummy=="Lp1")
						pow_loss*=nodes_on_the_path[0].net_router->Lp1;
					else if(dummy=="Lp2")
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
					else if(dummy=="(K2)" || dummy=="K2")
						pow_loss*=nodes_on_the_path[0].net_router->K2;
					else if(dummy=="(K3)" || dummy=="K3")
						pow_loss*=nodes_on_the_path[0].net_router->K3;
					else if(dummy=="Lc1")
						pow_loss*=nodes_on_the_path[0].net_router->Lc1;
					else if(dummy=="Lc2")
						pow_loss*=nodes_on_the_path[0].net_router->Lc2;
					else if(dummy=="(Lc Lp2 Lc)")
						pow_loss*=nodes_on_the_path[0].net_router->full_Lc2;
					else if(dummy.compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("(Lc Lp2 Lc) Lp1^")+16+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("(Lc Lp2 Lc) Lp1^")+16,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,8,"Lp2 Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp2 Lp1^")+8+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp2 Lp1^")+8,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,7,"Lc Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lc Lp1^")+7+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lc Lp1^")+7,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,4,"Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp1^")+4+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp1^")+4,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}

					//write the results to file
					results_power_loss<<dummy<<" * ";
				}
			}
			cal_ben_crs(nodes_on_the_path[0].x,nodes_on_the_path[0].y,nodes_on_the_path[routers_on_the_link-1].x,nodes_on_the_path[routers_on_the_link-1].y,bendings,crossings);
			results_power_loss<<" * Lb^("<<bendings<<")"<<" * Lc^("<<crossings<<")";
			pow_loss*=pow(L_b,bendings);
			pow_loss*=pow(L_c,crossings);
	}//Power loss before detection, the detection loss and the propagation loss will be considered after calculatiing the crosstalk noise power
	////////////////////////////////////////////////////////////////////////////////////////////////////////////

	//Coherent noise
	coh_xtalk=0;
	for(int i=0;i<routers_on_the_link;i++)
		coh_xtalk+=nodes_on_the_path[i].net_router->coh_noise;
	//add the coherent crosstalk at modulation 
	coh_xtalk+=pow(nodes_on_the_path[0].net_router->K2,2)/nodes_on_the_path[0].net_router->Lp1;
	for(int i=wvlgth_no+1;i<=wdm_no;i++)
		coh_xtalk+=(nodes_on_the_path[0].net_router->K2*
		MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1);
	//add the coherent crosstalk at detection
	for(int i=1;i<wvlgth_no;i++)
		coh_xtalk+=nodes_on_the_path[0].net_router->K2*nodes_on_the_path[0].net_router->Kd1[wvlgth_no-1]; // Matrix method replaced
		//coh_xtalk+=nodes_on_the_path[0].net_router->K2*
                //MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		//(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		//((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1;
	///////////////////////////////////////////////////////////////////////////////////

	//Reading the communication pattern
	net_config.open("Network_Configuration.txt",ios::in);
	if(!net_config.is_open())
	{
		cout<<"\nThe Network_Configuration.txt file is not open!\n";
		exit(1);
	}
	else
	{
		getline(net_config,dummy);
		while(dummy.compare(0,17,"com_pattern start")!=0 && !net_config.eof())
			getline(net_config,dummy);

		double w_length;

		while(!net_config.eof())
		{
			getline(net_config,dummy);
			while((dummy.length()!=0 && dummy[0]=='/' && dummy[1]=='/') || dummy.length()==0 && !net_config.eof()) //Skip the commments and empty lines
				getline(net_config,dummy);
			if(net_config.eof() || dummy=="com_pattern end")
				break;
			find_sender_receiver(dummy,sender,receiver); //Find the sender and receiver defined by the user
			sub_network.wg_net.set_length(0);
			sub_network.Power_loss_FTorus(sender.x,sender.y,receiver.x,receiver.y,wvlgth_no,wdm_no);

			//Find common nodes
			for(int j=0;j<routers_on_the_link;j++) //j is the counter for the main link
				for(int k=0;k<sub_network.routers_on_the_link;k++) //k is the counter for the noise inroducing link
					if(nodes_on_the_path[j].x==sub_network.nodes_on_the_path[k].x && nodes_on_the_path[j].y==sub_network.nodes_on_the_path[k].y) //Common Node
					{
						//Find the common element in the router structure
						for(int m=0;m<total_num_elements;m++) 
							for(int n=0;n<total_num_elements;n++)
								if(nodes_on_the_path[j].net_router->get_res().path[m]==sub_network.nodes_on_the_path[k].net_router->get_res().path[n]) //The common element in the router structure
								{
									if(nodes_on_the_path[j].net_router->get_res().path[m]==-1 || sub_network.nodes_on_the_path[k].net_router->get_res().path[n]==-1)
										break;
									else
									{
										//Need to consider the common element type (CSE, PSE, or Crossing) and finally assign the crosstalk coefficient for it
										com_el=sub_network.nodes_on_the_path[k].net_router->get_res().path[n];

										for(int c=0;c<total_num_elements;c++)
											str[c]="$";

										for(int c=0;c<size_N*size_M*total_num_elements;c++)
											crosstalk_str[c]="$";

										w_length=0;
										counter=0;
										crosstalk_noise_pow=1; 
										if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==2) //The common element is crossing
										{
											crosstalk_coef="(K1)";
											w_length+=2*MR_D+WG_w;
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==5 && wdm_no==1) //The common element is CSE and Using a single wavelength
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) //2D
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";	//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";	
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														crosstalk_coef="(K1 + Lc Lc K2)";
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
													//propagation loss
													w_length+=2*MR_D+WG_w;
												}
											}
											else // If the MR is ON in the CSE
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1 || 
													sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 1 or 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
													{
														crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
														//propagation loss
														w_length+=6*MR_D+2*WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) 
													{
														crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
														//propagation loss
														w_length+=6*MR_D+3*WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2 || 
													sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 2 or 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
													{
														crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
														//propagation loss
														w_length+=6*MR_D+2*WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) 
													{
														crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
														//propagation loss
														w_length+=6*MR_D+3*WG_w;
													}
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==5 && wdm_no>1) //The common element is CSE and Using WDM
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
														crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
														crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
														crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
														crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) //2D
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
														crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
														crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
														crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+2*MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
														crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*wdm_no*MR_D+MR_D+WG_w;
													}
													else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
														crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														//propagation loss (estimation for WDM case in this case)
														w_length+=2*MR_D+WG_w;
													}
												}
											}
											else // If the MR is ON in the CSE, consider first-order crosstalk noise only
											{
												if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
												else if(sub_network.nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4 
												{
													if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
														(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
														crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
														//propagation loss (estimation for WDM case in this case)
														w_length+=wdm_no*MR_D+MR_D+WG_w;
													}
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==6 && wdm_no==1) //The common element is PSE and using a Single wavelength
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_coef="(K2)";
													w_length+=4*MR_D+WG_w;
												}
											}
											else //If the MR is ON
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_coef="(K3)";
													w_length+=2*MR_D;
												}
											}
										}
										else if(sub_network.nodes_on_the_path[k].net_router->router.elm[com_el].type==6 && wdm_no>1) //The common element is PSE and using WDM
										{
											if(sub_network.nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,7,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=7
													crosstalk_coef="Func_no=7";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
													w_length+=4*MR_D+2*(wdm_no-wvlgth_no)*MR_D+WG_w;
												}
												else if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,8,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=8
													crosstalk_coef="Func_no=8";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
													w_length+=4*MR_D+2*(wvlgth_no-1)*MR_D+WG_w;
												}
												
											}
											else //If the MR is ON
											{
												if((nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==1 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==2 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==3 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
													(nodes_on_the_path[j].net_router->get_res().cur_port[m+1]==4 && sub_network.nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
												{
													crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,9,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=9
													crosstalk_coef="Lp1^(wdm_no-1) K3";
													//propagation loss (estimation for WDM case in this case)
													w_length+=(wdm_no+1)*MR_D;
												}
											}
										}

										int p,q;
										for(p=0;p<n+1;p++)
											str[p]=sub_network.nodes_on_the_path[k].net_router->get_res().PL[p]; //The path before the incident from the link introduction link
										str[p++]=crosstalk_coef; //The crosstalk coefficient
										for(q=m+2;q<total_num_elements && nodes_on_the_path[j].net_router->get_res().PL[q]!="$";q++)
											str[p++]=nodes_on_the_path[j].net_router->get_res().PL[q]; //The path after the incident from the main link
										////////////////////////////////////////////////
										//Calculate the propagation length
										for(int y=j+1;y<routers_on_the_link;y++)
											w_length+=nodes_on_the_path[y].net_router->w.get_length(); //Consider the propagation loss in the optical routers located on the main link
										//Consider the propagation loss of the main link 
										w_length+=(routers_on_the_link-j-1)*hop_length;
										//Consider the propagation loss of the noise introduction link
										for(int i=0;i<k;i++)
											w_length+=sub_network.nodes_on_the_path[i].net_router->w.get_length();  //Optical routers on the noise introduction link
										w_length+=k*hop_length; //The noise link itself

										//Add the propagation loss inside the optical router
										for(int z=0;z<total_num_elements;z++)
											if(str[z]=="$")
												break;
											else
											{
												if(str[z]=="Lc"|| str[z]=="Lc1" || str[z]=="Lc2")
													w_length+=2*MR_D+WG_w;
												else if(str[z]=="Lp1")
													w_length+=2*MR_D;
												else if(str[z]=="Lb")
													w_length+=MR_D+WG_w;
												else if(str[z]=="Lp2" || str[z].compare(0,8,"Lp2 Lp1^")==0)
													w_length+=4*MR_D+WG_w;
												else if(str[z]=="(Lc Lp2 Lc)" || str[z].compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
													w_length+=6*MR_D+3*WG_w;
												else if(str[z].compare(0,7,"Lc Lp1^")==0)
													w_length+=(wdm_no+1)*MR_D+WG_w;
												else if(str[z].compare(0,4,"Lp1^")==0)
													w_length+=(wdm_no+1)*MR_D;
											}

											for(int i=0;i<n;i++) //Noise link
												if(sub_network.nodes_on_the_path[k].net_router->get_res().PL[i]=="Wgd")
													w_length+=sub_network.nodes_on_the_path[k].net_router->router.wgd[sub_network.nodes_on_the_path[k].net_router->get_res().path[i-1]].get_length();
											for(int i=m+1;i<total_num_elements;i++) //Main link
												if(nodes_on_the_path[j].net_router->get_res().PL[i]=="$")
													break;
												else if(nodes_on_the_path[j].net_router->get_res().PL[i]=="Wgd")
													w_length+=nodes_on_the_path[j].net_router->router.wgd[nodes_on_the_path[j].net_router->get_res().path[i-1]].get_length();
											////////////////////////////////////////
											//Copy the result to the crosstalk_str
											for(int r=0;r<k;r++)
												for(int s=0;s<total_num_elements;s++)
													if(sub_network.nodes_on_the_path[r].net_router->get_res().PL[s]=="$")
														break;
													else
														crosstalk_str[counter++]=sub_network.nodes_on_the_path[r].net_router->get_res().PL[s]; //The Power Loss of the noise introduction link before the common node

											for(int t=0;t<total_num_elements;t++)
												if(str[t]=="$")
													break;
												else
													crosstalk_str[counter++]=str[t]; //The Power Loss inside the common node

											for(int u=j+1;u<routers_on_the_link;u++)
												for(int x=0;x<total_num_elements;x++)
													if(nodes_on_the_path[u].net_router->get_res().PL[x]=="$")
														break;
													else
														crosstalk_str[counter++]=nodes_on_the_path[u].net_router->get_res().PL[x];

											crosstalk_str[counter++]="+";

											//Wrting the result into the output textfile
											if(results_crosstalk.is_open())
											{
												int i=0;
												while(crosstalk_str[i]!="$")
												{
													if(crosstalk_str[i]!="+" && crosstalk_str[i]!="P0_1" && crosstalk_str[i]!="P0_2" && crosstalk_str[i]!="P0_3" && crosstalk_str[i]!="P0_4" && crosstalk_str[i]!="Wgd")
													{
														if(crosstalk_str[i]=="P0_0")
														{
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->input_power*nodes_on_the_path[0].net_router->Lp1*nodes_on_the_path[0].net_router->Lp2;
															crosstalk_str[i]="Lp1 Lp2 * P0_0";//for modulation
															w_length+=8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width;//modulation propagation loss
														}
														else if(crosstalk_str[i]=="Lc")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
														else if(crosstalk_str[i]=="Lb")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lb;
														else if(crosstalk_str[i]=="(K1)" || crosstalk_str[i]=="K1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K1;
														else if(crosstalk_str[i]=="Kr")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kr;
														else if(crosstalk_str[i]=="Kt")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kt;
														else if(crosstalk_str[i]=="Lp1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp1;
														else if(crosstalk_str[i]=="Lp2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
														else if(crosstalk_str[i]=="(K2)" || crosstalk_str[i]=="K2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K2;
														else if(crosstalk_str[i]=="(K3)" || crosstalk_str[i]=="K3")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K3;
														else if(crosstalk_str[i]=="Lc1")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc1;
														else if(crosstalk_str[i]=="Lc2")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc2;
														else if(crosstalk_str[i].compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("(Lc Lp2 Lc) Lp1^")+16+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("(Lc Lp2 Lc) Lp1^")+16,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,8,"Lp2 Lp1^")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lp2 Lp1^")+8+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp2 Lp1^")+8,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,7,"Lc Lp1^")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lc Lp1^")+7+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lc Lp1^")+7,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i].compare(0,4,"Lp1^")==0)
														{
															int lc_aux=0;
															string dum_aux;
															while(crosstalk_str[i][crosstalk_str[i].find("Lp1^")+4+lc_aux]!='@')
																lc_aux++;
															dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp1^")+4,lc_aux);
															int exponent=atoi(dum_aux.c_str());
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
														}
														else if(crosstalk_str[i]=="(Lp1 K1)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_1;
														else if(crosstalk_str[i]=="(K1 + Lc Lc K2)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_2;
														else if(crosstalk_str[i]=="(Lp1 Lp1 K1 + K2)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_3;
														else if(crosstalk_str[i]=="(K1 K3 + Lc Lp2 Kr K3)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_4;
														else if(crosstalk_str[i]=="(Lc K3 + Lc Lp2 K1 K3)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_5;
														else if(crosstalk_str[i]=="(Lc Lp2 Lc)")
															crosstalk_noise_pow*=nodes_on_the_path[0].net_router->full_Lc2;

														//write to the file
														results_crosstalk<<crosstalk_str[i]<<" * ";
													}
													else if(crosstalk_str[i]=="+")
													{
														results_crosstalk<<"Lp^("<<w_length*0.0001<<")";
														crosstalk_noise_pow*=pow(Prop_loss,w_length*0.0001);
														//Calculate number of bendings and crossings
														sub_network.cal_ben_crs(sender.x,sender.y,sub_network.nodes_on_the_path[k].x,sub_network.nodes_on_the_path[k].y,sub_net_bending,sub_net_crossing);//Noise link
														cal_ben_crs(nodes_on_the_path[j].x,nodes_on_the_path[j].y,nodes_on_the_path[routers_on_the_link-1].x,nodes_on_the_path[routers_on_the_link-1].y,bendings,crossings);//Main link
														results_crosstalk<<" * Lb^("<<sub_net_bending+bendings<<")"<<" * Lc^("<<sub_net_crossing+crossings<<")";
														results_crosstalk<<" + ";

														crosstalk_noise_pow*=pow(L_b,sub_net_bending+bendings);
														crosstalk_noise_pow*=pow(L_c,sub_net_crossing+crossings);

														noise+=crosstalk_noise_pow;
														crosstalk_noise_pow=1; 
													}
													i++;
												}
											}
									}
								}
								//else //Noise come from crossings at the network level
								//{
								//	int net_crs=0,net_ben=0;
								//	int dum_net_crs,dum_net_ben;


								//	if(j+1!=routers_on_the_link && nodes_on_the_path[j].y<size_N/2+1 && nodes_on_the_path[j+1].y<size_N/2+1 &&
								//		nodes_on_the_path[j].x<size_M/2+2 && nodes_on_the_path[j+1].x<size_M/2+2 && nodes_on_the_path[j].y==nodes_on_the_path[j+1].y &&
								//		nodes_on_the_path[j].x!=nodes_on_the_path[j+1].x) //Lower-Horizintal main link 
								//	{
								//		//Calculate the location of the crossing link
								//		int y_n=nodes_on_the_path[j].y+(size_N-(2*nodes_on_the_path[j].y-1));
								//		int x_n=nodes_on_the_path[j].x+(size_M-(2*nodes_on_the_path[j].x-1));
								//		int y_n_next=++y_n;
								//		int x_n_next=++x_n;
								//		if(x_n_next>size_M)
								//			x_n_next=1;
								//		if(y_n_next>size_N)
								//			y_n_next=1;

								//		if(nodes_on_the_path[j].x<nodes_on_the_path[j+1].x && //From Left to Right
								//			k+1!=sub_network.routers_on_the_link && sub_network.nodes_on_the_path[k].x==sub_network.nodes_on_the_path[k+1].x) //Vertical noise link
								//		{
								//			if(((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)) &&
								//				sub_network.nodes_on_the_path[k].x==nodes_on_the_path[j].x &&
								//				sub_network.nodes_on_the_path[k].x!=size_M/2+1)//First Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				cal_ben_crs(nodes_on_the_path[j].x,nodes_on_the_path[j].y,nodes_on_the_path[j+1].x,nodes_on_the_path[j+1].y,dum_net_ben,dum_net_crs); 
								//				net_crs+=dum_net_crs;
								//				net_ben+=dum_net_ben;
								//				//Exception case
								//				if(nodes_on_the_path[j].y==1 && sub_network.nodes_on_the_path[k].y==1)
								//					++net_ben;
								//				if((sub_network.nodes_on_the_path[k].y!=1 && sub_network.nodes_on_the_path[k+1].y!=size_N) &&
								//					(sub_network.nodes_on_the_path[k].y!=size_N && sub_network.nodes_on_the_path[k+1].y!=1) &&
								//					sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Top to Bottom
								//					net_crs+=2;
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n || sub_network.nodes_on_the_path[k+1].x==x_n) &&
								//				sub_network.nodes_on_the_path[k].x!=nodes_on_the_path[j].x &&
								//				(sub_network.nodes_on_the_path[k].y==nodes_on_the_path[j].y || sub_network.nodes_on_the_path[k+1].y==nodes_on_the_path[j].y))//Second Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				cal_ben_crs(nodes_on_the_path[j].x,nodes_on_the_path[j].y,nodes_on_the_path[j+1].x,nodes_on_the_path[j+1].y,dum_net_ben,dum_net_crs);
								//				dum_net_crs-=2;
								//				net_crs+=dum_net_crs;
								//				net_ben+=dum_net_ben;
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Bottom to Top
								//				{
								//					if(size_M*size_N%2==0 && sub_network.nodes_on_the_path[k].y==size_N/2+1)
								//						++net_ben;
								//					else if((sub_network.nodes_on_the_path[k].y!=size_N/2+1 && size_M*size_N%2==0) ||
								//						size_M*size_N%2!=0)
								//						net_crs+=2;
								//				}
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n || sub_network.nodes_on_the_path[k+1].x==x_n) &&
								//				((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)) &&
								//				sub_network.nodes_on_the_path[k].x!=size_M/2+1)//Last Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				if(sub_network.nodes_on_the_path[k].y==1)
								//				{
								//					++net_ben;
								//					net_crs+=4;
								//				}
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Top to Bottom
								//					net_crs+=6;
								//			}
								//		}
								//		else if(nodes_on_the_path[j].x>nodes_on_the_path[j+1].x && //From Right to Left
								//			k+1!=sub_network.routers_on_the_link && sub_network.nodes_on_the_path[k].x==sub_network.nodes_on_the_path[k+1].x) //Vertical noise link
								//		{
								//			if(((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)) &&
								//				sub_network.nodes_on_the_path[k].x==nodes_on_the_path[j+1].x &&
								//				sub_network.nodes_on_the_path[k].x!=size_M/2+1)//Last Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				if(sub_network.nodes_on_the_path[k].y==1 && nodes_on_the_path[j+1].y==sub_network.nodes_on_the_path[k].y)
								//					++net_ben;
								//				else
								//					net_crs+=4;
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y && sub_network.nodes_on_the_path[k].y!=1)//Top to Bottom
								//					net_crs-=2;
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n_next || sub_network.nodes_on_the_path[k+1].x==x_n_next) &&
								//				sub_network.nodes_on_the_path[k].x!=nodes_on_the_path[j+1].x &&
								//				(sub_network.nodes_on_the_path[k].y==nodes_on_the_path[j].y || sub_network.nodes_on_the_path[k+1].y==nodes_on_the_path[j].y))//Second Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				net_crs+=2;
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Bottom to Top
								//				{
								//					if(size_M*size_N%2==0 && sub_network.nodes_on_the_path[k].y==size_N/2+1)
								//					{
								//						++net_ben;
								//						net_crs+=4;
								//					}
								//					net_crs+=6;
								//				}
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n_next || sub_network.nodes_on_the_path[k+1].x==x_n_next) &&
								//				((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)) &&
								//				sub_network.nodes_on_the_path[k].x!=size_M/2+1)//First Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				net_crs+=6;
								//				if(sub_network.nodes_on_the_path[k].y==1)
								//					++net_ben;
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Top to Bottom
								//					net_crs+=2;
								//			}
								//		}
								//	}
								//	else if(j+1!=routers_on_the_link && nodes_on_the_path[j].y<size_N/2+1 && nodes_on_the_path[j+1].y<size_N/2+1 &&
								//		((nodes_on_the_path[j].x>size_M/2 && nodes_on_the_path[j+1].x>size_M/2) || ((nodes_on_the_path[j].x==size_M && nodes_on_the_path[j+1].x==1) || 
								//		(nodes_on_the_path[j].x==1 && nodes_on_the_path[j+1].x==size_M)))
								//		&& nodes_on_the_path[j].y==nodes_on_the_path[j+1].y &&
								//		nodes_on_the_path[j].x!=nodes_on_the_path[j+1].x) //Upper-Horizintal main link 
								//	{
								//		//Calculate the location of the crossing link
								//		int y_n=nodes_on_the_path[j].y+(size_N-(2*nodes_on_the_path[j].y-1));
								//		int x_n=nodes_on_the_path[j+1].x+(size_M-(2*nodes_on_the_path[j+1].x-1));
								//		int y_n_next=++y_n;
								//		int x_n_next=++x_n;
								//		if(y_n_next>size_N)
								//			y_n_next=1;

								//		if((nodes_on_the_path[j].x>nodes_on_the_path[j+1].x  || (nodes_on_the_path[j].x==1 && nodes_on_the_path[j+1].x==size_M))&& //From Left to Right
								//			k+1!=sub_network.routers_on_the_link && sub_network.nodes_on_the_path[k].x==sub_network.nodes_on_the_path[k+1].x) //Vertical noise link
								//		{
								//			if(((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)) &&
								//				sub_network.nodes_on_the_path[k].x==nodes_on_the_path[j].x &&
								//				nodes_on_the_path[j].x!=1)//First Crossing link
								//			{
								//				//Calculate the number of bendings and crossings after crossing
								//				if(sub_network.nodes_on_the_path[k].y==1)
								//					++net_ben;
								//				if((sub_network.nodes_on_the_path[k].y==1 && sub_network.nodes_on_the_path[k+1].y==size_N) ||
								//					(sub_network.nodes_on_the_path[k].y==size_N && sub_network.nodes_on_the_path[k+1].y==1))
								//					net_crs+=2;
								//				if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)//Top to Bottom
								//					net_crs+=8;
								//				else//Bottom to Top 
								//					net_crs+=6;
								//				if(nodes_on_the_path[j].x==size_N/2+2 && nodes_on_the_path[j+1].x==size_N/2+1 && size_M*size_N%2!=0)
								//				{
								//					++net_ben;
								//					if((sub_network.nodes_on_the_path[k].y!=1 && sub_network.nodes_on_the_path[k+1].y!=size_N) &&
								//						(sub_network.nodes_on_the_path[k].y!=size_N && sub_network.nodes_on_the_path[k+1].y!=1))
								//						net_crs-=2;
								//				}
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n || sub_network.nodes_on_the_path[k+1].x==x_n) &&
								//				nodes_on_the_path[j].y!=1 &&
								//				((sub_network.nodes_on_the_path[k+1].y==nodes_on_the_path[j].y && sub_network.nodes_on_the_path[k].y<sub_network.nodes_on_the_path[k+1].y)
								//				|| (sub_network.nodes_on_the_path[k].y==nodes_on_the_path[j].y && sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y)))//Second Crossing link
								//			{
								//				if(nodes_on_the_path[j].x==1)
								//				{
								//					if(sub_network.nodes_on_the_path[k].y<sub_network.nodes_on_the_path[k+1].y) //Top to Bottom
								//						net_crs+=4;
								//					else //Bottom to Top
								//						net_crs+=2;
								//				}
								//				else
								//				{
								//					if(sub_network.nodes_on_the_path[k].y<sub_network.nodes_on_the_path[k+1].y) //Top to Bottom
								//						net_crs+=8;
								//					else //Bottom to Top
								//						net_crs+=2;
								//				}
								//				if(size_M*size_N%2!=0 && sub_network.nodes_on_the_path[k].x==size_M/2+1)
								//				{
								//					++net_ben;
								//					net_crs-=2;
								//				}
								//			}
								//			else if((sub_network.nodes_on_the_path[k].x==x_n || sub_network.nodes_on_the_path[k+1].x==x_n) &&
								//				((sub_network.nodes_on_the_path[k].y==y_n && sub_network.nodes_on_the_path[k+1].y==y_n_next) ||
								//				(sub_network.nodes_on_the_path[k].y==y_n_next && sub_network.nodes_on_the_path[k+1].y==y_n)))
								//			{
								//				if(nodes_on_the_path[j].y!=1)
								//				{
								//					if(sub_network.nodes_on_the_path[k].y>sub_network.nodes_on_the_path[k+1].y) //Top to Bottom
								//						net_crs+=2;
								//					else //Bottom to Top
								//						net_crs+=4;
								//				}
								//				if(size_M*size_N%2!=0 && nodes_on_the_path[j].y==1 && nodes_on_the_path[j].x==size_M/2+2 && 
								//					nodes_on_the_path[j+1].x==size_M/2+1)
								//				{
								//					if(sub_network.nodes_on_the_path[k].y==size_N)
								//						net_crs+=4;
								//					++net_ben;
								//				}
								//			}
								//		}
								//	}
								//}
					}
		}
	}
	net_config.close();

	//power loss for noise power at detectors
        noise*=nodes_on_the_path[0].net_router->Ld1[wvlgth_no-1]*nodes_on_the_path[0].net_router->Lp2; //Matrix method replaced
	//noise*=pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*nodes_on_the_path[0].net_router->Lp2;
	noise*=pow(Prop_loss,((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001);
	results_crosstalk<<" * (Lp2 * Ld1 * Lp^("<<((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001<<")";
	
	//Add the crosstalk noise at the detectors
	if(wdm_no>1)//if using WDM
	{
		noise+=pow_loss*nodes_on_the_path[0].net_router->Ld1[wvlgth_no-1]*pow_loss*nodes_on_the_path[0].net_router->Kd1[wvlgth_no-1];//func_num=10
		results_crosstalk<<"* Func_no=10 * ";
	}

	//Add the detector at the end of the communication line
	pow_loss*=nodes_on_the_path[0].net_router->Ld1[wvlgth_no-1]*nodes_on_the_path[0].net_router->Lp2;
	results_power_loss<<"* Lp2 Ld1^"<<wvlgth_no-1<<"@ * ";
	wg_net.set_length(wg_net.get_length()+(3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width);
	results_power_loss<<"Lp^("<<wg_net.get_length()*0.0001<<")"<<"\n**END**";
	pow_loss*=pow(Prop_loss,wg_net.get_length()*0.0001);
	results_power_loss.close();
	results_crosstalk<<"\n**END**";
	results_crosstalk.close();
	delete [] str;
	delete [] crosstalk_str;
}

void Network::SNR_FTorus(int src_x,int src_y,int rcv_x,int rcv_y,char ch,int wdm_no)
{
	long double max_xtalk_noise=-1000,max_coh_xtalk=-1000,min_signal_pow=1000,min_SNR=1000;
	int max_xtalk_no=-1, min_signal_pow_no=-1,min_SNR_no=-1,max_coh_xtalk_no=-1;
	bool show_wrc_res=true;
	if(src_x>size_M || rcv_x>size_M || src_y>size_N || rcv_y>size_N)
	{
		cout<<"\nERROR: The source and the destination are not within the defined network size!";
		exit(1);
	}
	if(ch=='s')
	{
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		cout<<"\nThe results for the optical link starts from the core ("<<src_x<<", "<<src_y<<") towards the core ("<<rcv_x<<", "<<rcv_y<<")\n in Folded-Torus-based ONoC are as following:\n\n";
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_FTorus(src_x,src_y,rcv_x,rcv_y,i,1); //Calculate the power loss of the link
			crosstalk_FTorus(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files			
			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				//Show the results to the user
				cout<<"Wavelength["<<i<<"] Singal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Incoherent Crosstalk Noise Power (at the destination)= "<<crosstalk_noise<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no coherent crosstalk)= "<<SNR_no_coh<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (coherent crosstalk + incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (singal power - coherent crosstalk)= "<<SNR_SP_coh<<" dB\n\n";
				
				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//cout<<"\nThe results for the optical link starts from the core ("<<src_x<<", "<<src_y<<") towards the core ("<<rcv_x<<", "<<rcv_y<<")\n in folded-torus-based ONoC are as followings:\n\n";
				cout<<"Wavelength["<<i<<"] Signal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				cout<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wrc_res=true;
			}
		}
		//Show the worst-case results for different wavelengths
		if(show_wrc_res)
		{
			cout<<endl;
			cout<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			cout<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dBm\n";
			cout<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dBm\n";
			cout<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			cout<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR (no coherent crosstalk considered) of "<<min_SNR<<" dB\n\n";
		}
	}
	else if(ch=='f')
	{
		ofstream outfile("Network_Analyzer_Resuls.csv",ios::out);
		if(!outfile.is_open())
		{
			outfile<<"\nERROR: The network_analyzer_results file could not be created!\n";
			exit(1);
		}
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		outfile<<"\nThe results for the optical link starts from the core ("<<src_x<<"- "<<src_y<<") towards the core ("<<rcv_x<<"- "<<rcv_y<<") in Folded-Torus-based ONoC are as following:"<<endl;
		outfile<<"Wavelength, Signal power (dBm), Incoherent crosstalk noise power (dBm), Coherent crosstalk (dBm), SNR (no coherent crosstalk)(dB), SNR (coherent crosstalk + incoherent crosstalk)(dB), SNR (signal power - coherent crosstalk)(dB)"<<endl;
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_FTorus(src_x,src_y,rcv_x,rcv_y,i,1); //Calculate the power loss of the link
			crosstalk_FTorus(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files

			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				outfile<<i<<","<<power_loss<<","<<crosstalk_noise<<","<<coh_xtalk<<","<<SNR_no_coh<<","<<SNR_ch_inc<<","<<SNR_SP_coh<<endl;

				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
                                crosstalk_noise*=power_loss; //MR Matrix method added
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//outfile<<"\nThe results for the optical link starts from the core ("<<src_x<<", "<<src_y<<") towards the core ("<<rcv_x<<", "<<rcv_y<<")\n in folded-torus-based ONoC are as followings:\n\n";
				outfile<<"Wavelength["<<i<<"] Signal power= "<<power_loss<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				outfile<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wrc_res=true;
			}
		}
		//Show the worst-case results for different wavelengths
		if(show_wrc_res)
		{
			outfile<<endl;
			outfile<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			outfile<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dB\n";
			outfile<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dB\n";
			outfile<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			outfile<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR (no coherent crosstalk is considered) of "<<min_SNR<<" dB\n\n";
		}
		outfile.close();
	}
}
//
void Network::find_sender_receiver(string str,node &s, node &r)
{
	string dummy;
	int loc;
	if(str.compare(0,4,"from")==0) //Start of a communication pair
	{
		//x of the sender
		loc=0;
		while(str[4+loc]!=',')
			loc++;
		dummy.assign(str,4,loc);
		s.x=atoi(dummy.c_str());
		dummy.erase();

		//y of the sender
		loc=0;
		while(str[str.find(',')+1+loc]!='t')
			loc++;
		dummy.assign(str,str.find(',')+1,loc);
		s.y=atoi(dummy.c_str());
		dummy.erase();

		//x of the receiver
		loc=0;
		while(str[str.find('t')+2+loc]!=',')
			loc++;
		dummy.assign(str,str.find('t')+2,loc);
		r.x=atoi(dummy.c_str());
		dummy.erase();

		//y of the Receiver
		int loc2=0;
		while(str[str.find('t')+2+loc+1+loc2]!=';')
			loc2++;
		dummy.assign(str,str.find('t')+2+loc+1,loc2);
		r.y=atoi(dummy.c_str());
		dummy.erase();
	}
}
void Network::cal_ben_crs(int start_x,int start_y,int end_x,int end_y,int& num_bnd,int& num_crs)
{
	num_bnd=0,num_crs=0;
	src.x=start_x;
	src.y=start_y;
	des.x=end_x;
	des.y=end_y; 
	bool received=false;

	node current_node;
	current_node.x=src.x;
	current_node.y=src.y;
	//Calculate the power loss based on the XY routing algortihm
	int i=0;
	while(!received)
	{
		if(current_node.x<des.x)
		{
			if((des.x-current_node.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Right
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==size_N/2 || current_node.x==size_N/2+1) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=size_M/2+1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x==size_N/2+1 || current_node.x==size_N/2+2) //consider the bendings and crossings
					{
						++num_bnd;
						num_crs+=4;
					}
					else if((current_node.x>size_N/2+2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if(((des.x-current_node.x)>(size_N/2)+1 || (des.x-current_node.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && current_node.x>size_N/2) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Right
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==size_N/2 || current_node.x==size_N/2+1) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=size_M/2+1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x==size_N/2+1 || current_node.x==size_N/2+2) //consider the bendings and crossings
					{
						++num_bnd;
						num_crs+=4;
					}
					else if((current_node.x>size_N/2+2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if(((des.x-current_node.x)>(size_N/2)+1 || (des.x-current_node.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Left
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2+2 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
			}
			else if((des.x-current_node.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && current_node.x>size_N/2 ) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Left
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2+2 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
			}
		}
		else if(current_node.x>des.x)
		{
			if((current_node.x-des.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1)))) //Go Left
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2+2 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}

			}
			else if(((current_node.x-des.x)>(size_N/2)+1 || (current_node.x-des.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && current_node.x>size_N/2) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1))) //Go Left
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if((current_node.x<size_N/2 || current_node.x==size_N/2) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}
				else //Odd network size
				{
					if(current_node.x==1 || current_node.x==size_N) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=1)
							num_crs+=4;
					}
					else if((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2+2 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if((current_node.x<size_N/2+1 || current_node.x==size_N/2+1) && current_node.x!=1)
						--current_node.x;
					else if(current_node.x==1)
						current_node.x=size_N;
					else if(current_node.x>size_N/2+1 && current_node.x!=size_N)
						++current_node.x;
					else if(current_node.x==size_N)
						current_node.x=1;
				}

			}
			else if(((current_node.x-des.x)>(size_N/2)+1 || (current_node.x-des.x)==(size_N/2)+1) && ( (size_M*size_N%2==0 && (current_node.x<size_N/2 || current_node.x==size_N/2)) ||
				(size_M*size_N%2!=0 && (current_node.x<size_N/2+1 || current_node.x==size_N/2+1))))//Go Right
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==size_N/2 || current_node.x==size_N/2+1) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=size_M/2+1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x==size_N/2+1 || current_node.x==size_N/2+2) //consider the bendings and crossings
					{
						++num_bnd;
						num_crs+=4;
					}
					else if((current_node.x>size_N/2+2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
			else if((current_node.x-des.x)<(size_N/2)+1 && ( (size_M*size_N%2==0 && current_node.x>size_N/2 ) ||
				(size_M*size_N%2!=0 && current_node.x>size_N/2+1 )))//Go Right
			{
				if((size_M*size_N)%2==0) //Even network size
				{
					if(current_node.x==size_N/2 || current_node.x==size_N/2+1) //consider the bendings and crossings
					{
						++num_bnd;
						if(current_node.y!=size_M/2+1)
							num_crs+=4;
					}
					else if(((current_node.x>size_N/2+1 && current_node.y==1) || (current_node.x<size_N/2 && current_node.y==size_M/2+1)))
						num_crs+=2;
					else 
						num_crs+=6;
					if(current_node.x<size_N/2 || current_node.x==size_N/2)
						++current_node.x;
					else
						--current_node.x;
				}
				else //Odd network size
				{
					if(current_node.x==size_N/2+1 || current_node.x==size_N/2+2) //consider the bendings and crossings
					{
						++num_bnd;
						num_crs+=4;
					}
					else if((current_node.x>size_N/2+2 && current_node.y==1) || (current_node.x<size_N/2+1 && current_node.y==size_M/2+1))
						num_crs+=2;
					else
						num_crs+=6;
					if(current_node.x<size_N/2+1 || current_node.x==size_N/2+1)
						++current_node.x;
					else
						--current_node.x;
				}
			}
		}
		else //Same x
		{
			if(current_node.y<des.y)
			{
				if( ((des.y-current_node.y>(size_M/2)+1 || des.y-current_node.y==(size_M/2)+1) && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) ||
					(des.y-current_node.y<(size_M/2)+1 && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1)))) //Go Up
				{
					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y==1 || current_node.y==size_M) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=size_N/2+1)
								num_crs+=4;
						}
						else if(((current_node.y>size_M/2 && current_node.x==size_N+1) || (current_node.y<size_M/2+1 && current_node.x==1)))
							num_crs+=2;
						else 
							num_crs+=6;
						if((current_node.y<size_M/2 || current_node.y==size_M/2) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
					else //Odd network size
					{
						if(current_node.y==1 || current_node.y==size_M) //consider the bendings and crossings
						{
							++num_bnd;
							num_crs+=4;
						}
						else if((current_node.y>size_M/2+1 && current_node.x==size_N+1) || (current_node.y<size_M/2+1 && current_node.x==1))
							num_crs+=2;
						else
							num_crs+=6;
						if((current_node.y<size_M/2+1 || current_node.y==size_M/2+1) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2+1 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
				}
				else if((des.y-current_node.y<(size_M/2)+1 && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) ||
					((des.y-current_node.y>(size_M/2)+1 || des.y-current_node.y==(size_M/2)+1) && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1))))//Go Down
				{
					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y==size_M/2 || current_node.y==size_M/2+1) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=1)
								num_crs+=4;
						}
						else if(((current_node.y>size_M/2+1 && current_node.x==size_N+1) || (current_node.y<size_M/2 && current_node.x==1)))
							num_crs+=2;
						else 
							num_crs+=6;
						if(current_node.y<size_M/2 || current_node.y==size_M/2)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
					else //Odd network size
					{
						if(current_node.y==size_M/2+1 || current_node.y==size_M/2+2) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=size_N/2+1)
								num_crs+=4;
						}
						else if((current_node.y>size_M/2+2 && current_node.x==size_N/2+1) || (current_node.y<size_M/2+1 && current_node.x==1))
							num_crs+=2;
						else
							num_crs+=6;
						if(current_node.y<size_M/2+1 || current_node.y==size_M/2+1)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
				}
			}
			else if(current_node.y>des.y)
			{
				if(((current_node.y-des.y>(size_M/2)+1 || current_node.y-des.y==(size_M/2)+1) && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) || 
					(current_node.y-des.y<(size_M/2)+1 && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1))))//Go Down
				{
					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y==size_M/2 || current_node.y==size_M/2+1) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=1)
								num_crs+=4;
						}
						else if(((current_node.y>size_M/2+1 && current_node.x==size_N+1) || (current_node.y<size_M/2 && current_node.x==1)))
							num_crs+=2;
						else 
							num_crs+=6;
						if(current_node.y<size_M/2 || current_node.y==size_M/2)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
					else //Odd network size
					{
						if(current_node.y==size_M/2+1 || current_node.y==size_M/2+2) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=size_N/2+1)
								num_crs+=4;
						}
						else if((current_node.y>size_M/2+2 && current_node.x==size_N/2+1) || (current_node.y<size_M/2+1 && current_node.x==1))
							num_crs+=2;
						else
							num_crs+=6;
						if(current_node.y<size_M/2+1 || current_node.y==size_M/2+1)
							++current_node.y;
						else if(current_node.y>size_M/2)
							--current_node.y;
					}
				}
				else if((current_node.y-des.y<(size_M/2)+1 && ((size_M*size_N%2==0 && (current_node.y<size_M/2 || current_node.y==size_M/2)) ||
					(size_M*size_N%2!=0 && (current_node.y<size_M/2+1 || current_node.y==size_M/2+1)))) || 
					((current_node.y-des.y>(size_M/2)+1 || current_node.y-des.y==(size_M/2)+1) && ((size_M*size_N%2==0 && current_node.y>size_M/2) ||
					(size_M*size_N%2!=0 && current_node.y>size_M/2+1 ))))//Go Up
				{
					if((size_M*size_N)%2==0) //Even network size
					{
						if(current_node.y==1 || current_node.y==size_M) //consider the bendings and crossings
						{
							++num_bnd;
							if(current_node.x!=size_N/2+1)
								num_crs+=4;
						}
						else if(((current_node.y>size_M/2 && current_node.x==size_N+1) || (current_node.y<size_M/2+1 && current_node.x==1)))
							num_crs+=2;
						else 
							num_crs+=6;
						if((current_node.y<size_M/2 || current_node.y==size_M/2) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
					else //Odd network size
					{
						if(current_node.y==1 || current_node.y==size_M) //consider the bendings and crossings
						{
							++num_bnd;
							num_crs+=4;
						}
						else if((current_node.y>size_M/2+1 && current_node.x==size_N+1) || (current_node.y<size_M/2+1 && current_node.x==1))
							num_crs+=2;
						else
							num_crs+=6;
						if((current_node.y<size_M/2+1 || current_node.y==size_M/2+1) && current_node.y!=1)
							--current_node.y;
						else if(current_node.y==1)
							current_node.y=size_M;
						else if(current_node.y>size_M/2+1 && current_node.y!=size_M)
							++current_node.y;
						else if(current_node.y==size_M)
							current_node.y=1;
					}
				}
			}
			else //Destination
				received=true;
		}
		i++;
	}
}
//Fat-Tree Network
FTree_Network::FTree_Network()
{
	string str,dummy;
	int loc;
	net_config.open("Network_Configuration.txt",ios::in);
	if(!net_config.is_open())
	{
		cout<<"\nThe Network_Configuration.txt file is not open!\n";
		exit(1);
	}
	else
	{
		while(!net_config.eof())
		{
			getline(net_config,str);
			if((str.length()!=0 && str[0]=='/' && str[1]=='/') || str.length()==0) //Skip the commments and empty lines
				continue;
			else if(str=="com_pattern start")
				break;
			else if(str.compare(0,2,"k=")==0) //Number of Cores
			{
				loc=0;
				while(str[str.find("k=")+2+loc]!=';')
					loc++;
				dummy.assign(str,str.find("k=")+2,loc);
				num_cores=atoi(dummy.c_str());
				dummy.erase();
			}
			else if(str.compare(0,10,"chip_size=")==0) //Chip Size which is in cm2
			{
				loc=0;
				while(str[str.find("chip_size=")+10+loc]!=';')
					loc++;
				dummy.assign(str,str.find("chip_size=")+10,loc);
				chip_size=atoi(dummy.c_str());
				dummy.erase();
			}
		}
	}
	net_config.close();

	//Floorplan size
	//In fat-tree ONoC we would like to have a MxN square floorplan
	//if M!=N, then we pick M and N in the way to make the aspect ratio of the floorpan very close to 1
	fp_M=floor(sqrt(double(num_cores)));
	while((num_cores%(int(fp_M))))
		fp_M++;
	fp_N=num_cores/fp_M;

	//Number of levels
	num_level=int(log10(double(num_cores))/log10(double(2)))-1;

	//Approximation of the hop-length
	hop_length=sqrt((chip_size)/(fp_M*fp_N)); //Approximation of the hop_length
	hop_length=hop_length*10000; //Convert to micro-meter

	//Build the fat-tree structure in the memory
	build_FTnet();

	//The deafult direction is Upward
	direction=1;

	//cout<<"\nM= "<<fp_M<<"\nN= "<<fp_N<<endl;
	//cout<<"\nNumber of levels="<<num_level<<endl;
	//cout<<endl<<num_cores<<" "<<chip_size<<"Hop Length"<<hop_length<<endl;
}
FTree_Network::~FTree_Network()
{
	delete [] PL.Link_path;
	delete [] PL.Link_Power_loss;
//	for(int i=0;i<routers_on_the_link;i++)
	//	delete [] nodes_on_the_path[i].net_router;
	delete [] nodes_on_the_path;
	//Don't forget to delete routers
}
void FTree_Network::build_FTnet()
{
	rtr_1st_level=new ft_rtr_node[num_cores/2];
	//Up levels and routers
	up_rtr=new ft_rtr_node*[num_level-1];
	for(int i=0;i<num_level-1;i++)
		up_rtr[i]=new ft_rtr_node[num_cores/4];
	//Down levels and routers
	dwn_rtr=new ft_rtr_node*[num_level-1];
	for(int i=0;i<num_level-1;i++)
		dwn_rtr[i]=new ft_rtr_node[num_cores/4];

	//Initiate the first level and also connect the processors
	for(int i=0,j=0;i<num_cores/2;i++)
	{
		rtr_1st_level[i].rtr_num=i;
		rtr_1st_level[i].level=1;//Routers at the first level
		rtr_1st_level[i].loc=-1;//First level routers loc=-1
		rtr_1st_level[i].prt_taken[0][0]=rtr_1st_level[i].prt_taken[1][0]=rtr_1st_level[i].prt_taken[2][0]=rtr_1st_level[i].prt_taken[3][0]=false;
		rtr_1st_level[i].prt_taken[0][1]=rtr_1st_level[i].prt_taken[1][1]=rtr_1st_level[i].prt_taken[2][1]=rtr_1st_level[i].prt_taken[3][1]=false;
		if(i%2==0)
		{
			//assign the childs
			rtr_1st_level[i].child[0]=2*i;
			rtr_1st_level[i].child[1]=2*i+2;
			//connection to up and down routers
			rtr_1st_level[i].nxt_rtr[0]=NULL;
			rtr_1st_level[i].nxt_rtr[1]=&up_rtr[0][j];
			rtr_1st_level[i].nxt_rtr[2]=&dwn_rtr[0][j];
			rtr_1st_level[i].nxt_rtr[3]=NULL;

			up_rtr[0][j].nxt_rtr[3]=&rtr_1st_level[i];
			dwn_rtr[0][j].nxt_rtr[2]=&rtr_1st_level[i];

			//port connection
			rtr_1st_level[i].prt[0]=-1;
			rtr_1st_level[i].prt[1]=3;
			rtr_1st_level[i].prt[2]=2;
			rtr_1st_level[i].prt[3]=-1;

			up_rtr[0][j].prt[3]=1;
			dwn_rtr[0][j].prt[2]=2;
		}
		else
		{
			//assign the childs
			rtr_1st_level[i].child[0]=2*i-1;
			rtr_1st_level[i].child[1]=2*i+1;
			//connection to up and down routers
			rtr_1st_level[i].nxt_rtr[0]=&up_rtr[0][j];	
			rtr_1st_level[i].nxt_rtr[1]=NULL;
			rtr_1st_level[i].nxt_rtr[2]=NULL;
			rtr_1st_level[i].nxt_rtr[3]=&dwn_rtr[0][j];	

			up_rtr[0][j].nxt_rtr[2]=&rtr_1st_level[i];
			dwn_rtr[0][j].nxt_rtr[3]=&rtr_1st_level[i];

			//port connection
			rtr_1st_level[i].prt[0]=2;
			rtr_1st_level[i].prt[1]=-1;
			rtr_1st_level[i].prt[2]=-1;
			rtr_1st_level[i].prt[3]=3;

			up_rtr[0][j].prt[2]=0;
			dwn_rtr[0][j].prt[3]=3;
		}
		if(i%2!=0)
			j++;
	}

	//Construct up and down levels and routers
	int cntr,cntr2,port_num_up,port_num_dwn;//a counter to help for connecting routers
	for(int i=0;i<num_level-2;i++)
	{
		cntr=0;
		cntr2=0;
		port_num_up=3;
		port_num_dwn=2;
		for(int j=0;j<num_cores/4;j++)
		{
			up_rtr[i][j].rtr_num=j;
			up_rtr[i][j].level=i+2;
			up_rtr[i][j].loc=1;
			up_rtr[i][j].prt_taken[0][0]=up_rtr[i][j].prt_taken[1][0]=up_rtr[i][j].prt_taken[2][0]=up_rtr[i][j].prt_taken[3][0]=false;
			up_rtr[i][j].prt_taken[0][1]=up_rtr[i][j].prt_taken[1][1]=up_rtr[i][j].prt_taken[2][1]=up_rtr[i][j].prt_taken[3][1]=false;
			up_rtr[i][j].child[0]=up_rtr[i][j].child[1]=-1; //No processor core is directly connected to up or down level routers

			dwn_rtr[i][j].rtr_num=j;
			dwn_rtr[i][j].level=i+2;
			dwn_rtr[i][j].loc=0;
			dwn_rtr[i][j].prt_taken[0][0]=dwn_rtr[i][j].prt_taken[1][0]=dwn_rtr[i][j].prt_taken[2][0]=dwn_rtr[i][j].prt_taken[3][0]=false;
			dwn_rtr[i][j].prt_taken[0][1]=dwn_rtr[i][j].prt_taken[1][1]=dwn_rtr[i][j].prt_taken[2][1]=dwn_rtr[i][j].prt_taken[3][1]=false;
			dwn_rtr[i][j].child[0]=dwn_rtr[i][j].child[1]=-1;//No processor core is directly connected to up or down level routers

			//UP Levels
			//Connet routers at level i to routers at level i+1
			up_rtr[i][j].nxt_rtr[0]=&up_rtr[i+1][cntr];
			up_rtr[i][j].nxt_rtr[1]=&up_rtr[i+1][++cntr];

			//Down Levels
			////Connet routers at level i to routers at level i+1
			dwn_rtr[i][j].nxt_rtr[1]=&dwn_rtr[i+1][cntr2];
			dwn_rtr[i][j].nxt_rtr[0]=&dwn_rtr[i+1][++cntr2];

			//UP Levels
			//Connet ports at level i to ports at level i+1
			up_rtr[i][j].prt[0]=port_num_up;
			up_rtr[i][j].prt[1]=port_num_up;

			//Down Levels
			//Connet ports at level i to ports at level i+1
			dwn_rtr[i][j].prt[1]=port_num_dwn;
			dwn_rtr[i][j].prt[0]=port_num_dwn;

			//UP Levels
			//Connet routers at level i+1 to routers at level i
			up_rtr[i+1][--cntr].nxt_rtr[port_num_up]=&up_rtr[i][j];
			up_rtr[i+1][++cntr].nxt_rtr[port_num_up]=&up_rtr[i][j];

			//Down Levels
			//Connet routers at level i+1 to routers at level i
			dwn_rtr[i+1][--cntr2].nxt_rtr[port_num_dwn]=&dwn_rtr[i][j];
			dwn_rtr[i+1][++cntr2].nxt_rtr[port_num_dwn]=&dwn_rtr[i][j];

			//UP Levels
			//Connet ports at level i+1 to ports at level i
			up_rtr[i+1][--cntr].prt[port_num_up]=0;
			up_rtr[i+1][++cntr].prt[port_num_up]=1;

			//Down Levels
			//Connet ports at level i+1 to ports at level i
			dwn_rtr[i+1][--cntr2].prt[port_num_dwn]=1;
			dwn_rtr[i+1][++cntr2].prt[port_num_dwn]=0;

			cntr++;
			cntr2++;
			if((j+1)%int((pow(double(2),i)))==0)//Return
			{
				cntr=cntr2=j+1-pow(double(2),i);
				port_num_up=2;
				port_num_dwn=3;
			}
			if((j+1)%int(pow(double(2),i+1))==0)//Next cluster
			{
				cntr=cntr2=j+1;
				port_num_up=3;
				port_num_dwn=2;
			}
		}
	}

	//Connetion of the last level routers
	for(int j=0;j<num_cores/4;j++)
	{
		up_rtr[num_level-2][j].rtr_num=j;
		up_rtr[num_level-2][j].level=num_level;
		up_rtr[num_level-2][j].loc=1;
		up_rtr[num_level-2][j].prt_taken[0][0]=up_rtr[num_level-2][j].prt_taken[1][0]=up_rtr[num_level-2][j].prt_taken[2][0]=up_rtr[num_level-2][j].prt_taken[3][0]=false;
		up_rtr[num_level-2][j].prt_taken[0][1]=up_rtr[num_level-2][j].prt_taken[1][1]=up_rtr[num_level-2][j].prt_taken[2][1]=up_rtr[num_level-2][j].prt_taken[3][1]=false;
		up_rtr[num_level-2][j].child[0]=up_rtr[num_level-2][j].child[1]=-1; //No processor core is directly connected to up or down level routers

		dwn_rtr[num_level-2][j].rtr_num=j;
		dwn_rtr[num_level-2][j].level=num_level;
		dwn_rtr[num_level-2][j].loc=0;
		dwn_rtr[num_level-2][j].prt_taken[0][0]=dwn_rtr[num_level-2][j].prt_taken[1][0]=dwn_rtr[num_level-2][j].prt_taken[2][0]=dwn_rtr[num_level-2][j].prt_taken[3][0]=false;
		dwn_rtr[num_level-2][j].prt_taken[0][1]=dwn_rtr[num_level-2][j].prt_taken[1][1]=dwn_rtr[num_level-2][j].prt_taken[2][1]=dwn_rtr[num_level-2][j].prt_taken[3][1]=false;
		dwn_rtr[num_level-2][j].child[0]=dwn_rtr[num_level-2][j].child[1]=-1;//No processor core is directly connected to up or down level routers

		//Up Level
		//Connect the routers
		up_rtr[num_level-2][j].nxt_rtr[0]=&up_rtr[num_level-2][(num_cores/4)-1-j];
		up_rtr[num_level-2][j].nxt_rtr[1]=&up_rtr[num_level-2][(num_cores/4)-1-j];
		//Connect the ports
		up_rtr[num_level-2][j].prt[0]=1;
		up_rtr[num_level-2][j].prt[1]=0;

		//Down Level
		//Connect the routers
		dwn_rtr[num_level-2][j].nxt_rtr[1]=&dwn_rtr[num_level-2][(num_cores/4)-1-j];
		dwn_rtr[num_level-2][j].nxt_rtr[0]=&dwn_rtr[num_level-2][(num_cores/4)-1-j];
		//Connect the ports
		dwn_rtr[num_level-2][j].prt[1]=0;
		dwn_rtr[num_level-2][j].prt[0]=1;
	}
	//	for(int j=0;j<num_cores/4;j++)
	//		for(int i=0;i<4;i++)
	//		cout<<"\nRouter (4"<<","<<j<<") ="<<up_rtr[2][j].prt[i];
	//	cout<<endl;
}

//Calculate the power loss
void FTree_Network:: Power_loss_FTree(int src,int dest,int &crss_num, int &bend_num,bool &received,int wvlgth_no,int wdm_no)
{
	int src_loc;//Location of the source processor
	int dest_loc;//Location of the destination processor
	int src_child_loc,dest_child_loc;//To which port processors are connected
	int counter1=0,counter2=0,counter3=0;
	int dest_child_loc_direc=-1;
	int child_direction=-1;
	//For WDM we need to reset the propagation loss
	wg_net.set_length(0);

	//Find source and destination location
	for(src_loc=0;src_loc<num_cores/2;src_loc++)
	{
		if(rtr_1st_level[src_loc].child[0]==src)
		{
			if(src_loc%2==0)
				src_child_loc=0;
			else
				src_child_loc=1;
			break;
		}
		else if(rtr_1st_level[src_loc].child[1]==src)
		{
			if(src_loc%2==0)
				src_child_loc=3;
			else
				src_child_loc=2;
			break;
		}
	}
	for(dest_loc=0;dest_loc<num_cores/2;dest_loc++)
	{
		if(rtr_1st_level[dest_loc].child[0]==dest) 
		{
			if(dest_loc%2==0)
				dest_child_loc=0;
			else
				dest_child_loc=1;
			break;
		}
		else if(rtr_1st_level[dest_loc].child[1]==dest)
		{
			if(dest_loc%2==0)
				dest_child_loc=3;
			else
				dest_child_loc=2;
			break;
		}
	}
	crss_num=bend_num=0;
	received=false;
	bool other_side=false;
	int ancestor_level=find_ancestor_level(src,dest,other_side);
	--ancestor_level;
	routers_on_the_link=(ancestor_level*2)+1;
	//Src and Dest are connected to a same router
	if(src_loc==dest_loc)
		routers_on_the_link=1;

	//If on the other side
	if(other_side)
		++routers_on_the_link;
	//Build the nodes on the path
	nodes_on_the_path=new ft_router[routers_on_the_link];
	for(int i=0;i<routers_on_the_link;i++)
	{
		nodes_on_the_path[i].in=-1;
		nodes_on_the_path[i].level_num=-1;
		nodes_on_the_path[i].rtr_num=-1;
		nodes_on_the_path[i].out=-1;
		nodes_on_the_path[i].crs=-1;
		nodes_on_the_path[i].rtr_net_side=-1;
		nodes_on_the_path[i].net_router=new Router_Analyzer(-1,-1);
	}
	int total_num_element=nodes_on_the_path[0].net_router->router.total_number;//Totall number of element in 4x4 optical router
	//int total_num_element=60;

	//Decide the source child location direction
	if(src_child_loc==0 || src_child_loc==1)
		child_direction=0;
	else if(src_child_loc==2 || src_child_loc==3)
		child_direction=1;

	//Decide the destination child location direction
	if(dest_child_loc==0 || dest_child_loc==1)
		dest_child_loc_direc=1;
	else if(dest_child_loc==2 || dest_child_loc==3)
		dest_child_loc_direc=0;

	//Power Loss PL
	PL.Link_path=new int[routers_on_the_link*total_num_element];
	PL.Link_Power_loss=new string[routers_on_the_link*total_num_element];
	for(int i=0;i<routers_on_the_link*total_num_element;i++)
	{
		PL.Link_Power_loss[i]="$";
		PL.Link_path[i]=-1;
	}

	//Find the path
	path_node* nodes=new path_node[routers_on_the_link];
	bool fin_res=false;
	bool lower_net=false,upprt_net=false;
	
	if(src_loc!=dest_loc)//Src and Dest are not at the same router
	{
		if(src_child_loc==0 || src_child_loc==1)//should be lower network
		{
			received=find_path_lower(1,rtr_1st_level[src_loc].rtr_num,src_child_loc,src,dest,ancestor_level+1,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,routers_on_the_link,0,false,child_direction,dest_child_loc_direc,fin_res);
			if(received)
				lower_net=true;
		}
		else
		{
			fin_res=false;
			received=find_path_lower(1,rtr_1st_level[src_loc].rtr_num,src_child_loc,src,dest,ancestor_level+1,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,routers_on_the_link,0,false,child_direction,dest_child_loc_direc,fin_res);
			if(received)
				lower_net=true;
			else
			{
				fin_res=false;
				received=find_path_upper(1,rtr_1st_level[src_loc].rtr_num,src_child_loc,src,dest,ancestor_level+1,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,routers_on_the_link,0,false,child_direction,dest_child_loc_direc,fin_res);
				if(received)
					upprt_net=true;
			}
		}
	}

	//Start routing
	if(received && src_loc!=dest_loc)
	{
		if(lower_net)//if the packet has gone through the lower network
		{
			direction=0;
		}
		else if(upprt_net)//if the packet has gone through the upper network
		{
			direction=1;
		}
		for(int i=0;i<routers_on_the_link;i++)
		{
			//Routing
			nodes_on_the_path[i].in=nodes[i].in;
			nodes_on_the_path[i].out=nodes[i].out;
			nodes_on_the_path[i].level_num=nodes[i].level_num;
			nodes_on_the_path[i].rtr_num=nodes[i].rtr_num;
			
			//Router setup
		    //nodes_on_the_path[i].net_router=new Router_Analyzer(nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].in,0)],nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].out,1)]);
			nodes_on_the_path[i].net_router->router.set_up_structure(nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].in,0)],nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].out,1)]);
			nodes_on_the_path[i].net_router->refresh();
			nodes_on_the_path[i].net_router->find_path(nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].in,0)],nodes_on_the_path[i].net_router->router.port_type[match_prt_ftree(nodes[i].out,1)],0,wvlgth_no,wdm_no);
			wg_net.set_length(wg_net.get_length()+nodes_on_the_path[i].net_router->w.get_length()+hop_length);
			
			//Crossing and Port Occupation
			if(nodes[i].level_num==1)//level one: src or dest
			{
				//crss_num+=FT_num_crossings(rtr_1st_level[nodes[i].rtr_num],*rtr_1st_level[nodes[i].rtr_num].nxt_rtr[nodes[i].out],nodes[i].out);
				if(src_loc==nodes[i].rtr_num)//input
				{
					rtr_1st_level[src_loc].prt_taken[nodes[i].in][child_direction]=true;
					rtr_1st_level[src_loc].prt_taken[nodes[i].out][direction]=true;
				}
				else if(dest_loc==nodes[i].rtr_num)//output
				{
					rtr_1st_level[dest_loc].prt_taken[nodes[i].in][direction]=true;
					rtr_1st_level[dest_loc].prt_taken[nodes[i].out][dest_child_loc_direc]=true;
				}
			}
			else if(lower_net)
			{
				if(nodes[i-1].level_num-2>=0)
					crss_num+=FT_num_crossings(dwn_rtr[nodes[i-1].level_num-2][nodes[i-1].rtr_num],*dwn_rtr[nodes[i-1].level_num-2][nodes[i-1].rtr_num].nxt_rtr[nodes[i-1].out],nodes[i-1].out);
				dwn_rtr[nodes[i].level_num-2][nodes[i].rtr_num].prt_taken[nodes[i].in][direction]=true;//input
				//change the direction
				if((nodes[i].level_num==ancestor_level+1 && !other_side) || 
					(nodes[i].level_num==ancestor_level+1 && other_side && nodes[i].level_num==nodes[i-1].level_num))//change the direction
					if(direction==0)
						direction=1;
					else
						direction=0;
				dwn_rtr[nodes[i].level_num-2][nodes[i].rtr_num].prt_taken[nodes[i].out][direction]=true;//output
			}
			else if(upprt_net)
			{
				if(nodes[i-1].level_num-2>=0)
					crss_num+=FT_num_crossings(up_rtr[nodes[i-1].level_num-2][nodes[i-1].rtr_num],*up_rtr[nodes[i-1].level_num-2][nodes[i-1].rtr_num].nxt_rtr[nodes[i-1].out],nodes[i-1].out);
				up_rtr[nodes[i].level_num-2][nodes[i].rtr_num].prt_taken[nodes[i].in][direction]=true;//input
				//change the direction
				if((nodes[i].level_num==ancestor_level+1 && !other_side) || 
					(nodes[i].level_num==ancestor_level+1 && other_side && nodes[i].level_num==nodes[i-1].level_num))//change the direction
					if(direction==0)
						direction=1;
					else
						direction=0;
				up_rtr[nodes[i].level_num-2][nodes[i].rtr_num].prt_taken[nodes[i].out][direction]=true;//output
			}


			//copy the results
			copy_res(nodes_on_the_path[i].net_router->get_res().PL,PL.Link_Power_loss,nodes_on_the_path[i].net_router->get_res().path,PL.Link_path,counter1,counter2);
			nodes_on_the_path[i].crs=crss_num;
			if(lower_net)
				nodes_on_the_path[i].rtr_net_side=0;
			else if(upprt_net)
				nodes_on_the_path[i].rtr_net_side=1;
		}
		bend_num=routers_on_the_link*2;//Set the number of bendings
	}
	else if(src_loc==dest_loc)//Source and Dest are connected to the same optical router
	{
		//Routing
		nodes_on_the_path[0].in=src_child_loc;
		nodes_on_the_path[0].out=dest_child_loc;
		nodes_on_the_path[0].level_num=1;
		nodes_on_the_path[0].rtr_num=rtr_1st_level[src_loc].rtr_num;

		//Router setup
		nodes_on_the_path[0].net_router->router.set_up_structure(nodes_on_the_path[0].net_router->router.port_type[match_prt_ftree(src_child_loc,0)],nodes_on_the_path[0].net_router->router.port_type[match_prt_ftree(dest_child_loc,1)]);
		nodes_on_the_path[0].net_router->refresh();
		nodes_on_the_path[0].net_router->find_path(nodes_on_the_path[0].net_router->router.port_type[match_prt_ftree(src_child_loc,0)],nodes_on_the_path[0].net_router->router.port_type[match_prt_ftree(dest_child_loc,1)],0,wvlgth_no,wdm_no);
		wg_net.set_length(wg_net.get_length()+nodes_on_the_path[0].net_router->w.get_length()+2*hop_length);

		rtr_1st_level[dest_loc].prt_taken[src_child_loc][child_direction]=true;
		rtr_1st_level[dest_loc].prt_taken[dest_child_loc][dest_child_loc_direc]=true;
		
		//copy the results
		copy_res(nodes_on_the_path[0].net_router->get_res().PL,PL.Link_Power_loss,nodes_on_the_path[0].net_router->get_res().path,PL.Link_path,counter1,counter2);
		nodes_on_the_path[0].crs=0;
		nodes_on_the_path[0].rtr_net_side=0;//first level
		crss_num=0;
		bend_num=routers_on_the_link*2;//Set the number of bendings
		received=true;
	}
	else//the packer is drooped
	{
		cout<<"\nThe packet from "<<src<<" to "<<dest<<" has been dropped due to a collision!\n";
	}
	/*cout<<"\n for "<<src<<" to "<<dest<<" :\n";
	for(int i=0;i<routers_on_the_link;i++)
	{
		cout<<endl<<nodes[i].in<<endl<<nodes[i].out<<endl<<nodes[i].level_num<<endl<<nodes[i].rtr_num<<endl;
		if(lower_net)
			cout<<"\nLower\n";
		else if(upprt_net)
			cout<<"\nUpper\n";
	}*/
}
//Find path function for fat-tree
bool FTree_Network::find_path_lower(int lvl,int rtr_num,int in,int src,int dest,int anc_lvl,int src_loc,int dest_loc,int src_child_loc,int dest_child_loc,path_node* nodes,int arr_size,int count,bool turned,int src_child_dir,int dest_child_dir,bool& fin_res)
{
	if(lvl==1 && rtr_num%2==0 && rtr_num==src_loc)
	{
		if(rtr_1st_level[src_loc].prt_taken[2][0]==false && rtr_1st_level[src_loc].prt_taken[src_child_loc][src_child_dir]==false)
		{
			nodes[count].in=src_child_loc;
			nodes[count].level_num=1;
			nodes[count].out=2;
			nodes[count].rtr_num=src_loc;
			find_path_lower(2,rtr_1st_level[src_loc].nxt_rtr[2]->rtr_num,2,src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
		}
		else
			fin_res=false;
	}
	else if(lvl==1 && rtr_num%2!=0 && rtr_num==src_loc)
	{
		if(rtr_1st_level[src_loc].prt_taken[3][0]==false && rtr_1st_level[src_loc].prt_taken[src_child_loc][src_child_dir]==false)
		{
			nodes[count].in=src_child_loc;
			nodes[count].level_num=1;
			nodes[count].rtr_num=src_loc;
			nodes[count].out=3;
			find_path_lower(2,rtr_1st_level[src_loc].nxt_rtr[3]->rtr_num,3,src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
		}
		else
			fin_res=false;
	}
	else if(!turned)
	{
		if(lvl!=anc_lvl)
		{
			if(dwn_rtr[lvl-2][rtr_num].prt_taken[1][0]==false && fin_res==false)//go to port 1
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=1;
				nodes[count].rtr_num=rtr_num;
				find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[1]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[1]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[1],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
			}
			if(dwn_rtr[lvl-2][rtr_num].prt_taken[0][0]==false && fin_res==false)//go to port 0
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=0;
				nodes[count].rtr_num=rtr_num;
				find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[0]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[0]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[0],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
			
			}
		}
		else if(lvl==anc_lvl)
		{
			bool other_side;
			find_ancestor_level(src,dest,other_side);
			if(!other_side)
			{
				turned=true;
				if(find_child_path(dwn_rtr[anc_lvl-2][rtr_num],dest,2))
				{
					if(dwn_rtr[anc_lvl-2][rtr_num].prt_taken[2][1]==false)
					{
						nodes[count].in=in;
						nodes[count].level_num=anc_lvl;
						nodes[count].out=2;
						nodes[count].rtr_num=rtr_num;
						find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[2]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[2]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[2],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
						
					}
				}
				else if(find_child_path(dwn_rtr[anc_lvl-2][rtr_num],dest,3))
				{
					if(dwn_rtr[anc_lvl-2][rtr_num].prt_taken[3][1]==false)
					{
						nodes[count].in=in;
						nodes[count].level_num=anc_lvl;
						nodes[count].out=3;
						nodes[count].rtr_num=rtr_num;
						find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[3]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[3]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[3],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
						
					}
				}
			}
			else
			{
				turned=true;
				if(dwn_rtr[anc_lvl-2][rtr_num].prt_taken[1][0]==false && fin_res==false)//go to port 1
				{
					nodes[count].in=in;
					nodes[count].level_num=anc_lvl;
					nodes[count].out=1;
					nodes[count].rtr_num=rtr_num;
					find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[1]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[1]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[1],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
					
				}
				if(dwn_rtr[anc_lvl-2][rtr_num].prt_taken[0][0]==false && fin_res==false)//go to port 0
				{
					nodes[count].in=in;
					nodes[count].level_num=anc_lvl;
					nodes[count].out=0;
					nodes[count].rtr_num=rtr_num;
					find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[0]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[0]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[0],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
					
				}
			}
		}
	}
	else if(turned)
	{
		if(lvl!=1)
		{
			if(find_child_path(dwn_rtr[lvl-2][rtr_num],dest,2) && dwn_rtr[lvl-2][rtr_num].prt_taken[2][1]==false)//go port 2
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=2;
				nodes[count].rtr_num=rtr_num;
				find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[2]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[2]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[2],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
				
			}
			else if(find_child_path(dwn_rtr[lvl-2][rtr_num],dest,3) && dwn_rtr[lvl-2][rtr_num].prt_taken[3][1]==false)//go port 3
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=3;
				nodes[count].rtr_num=rtr_num;
				find_path_lower(dwn_rtr[lvl-2][rtr_num].nxt_rtr[3]->level,dwn_rtr[lvl-2][rtr_num].nxt_rtr[3]->rtr_num,dwn_rtr[lvl-2][rtr_num].prt[3],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
				
			}
		}
		else if(lvl==1 && rtr_num==dest_loc)//destination
		{
			if(rtr_1st_level[rtr_num].prt_taken[dest_child_loc][dest_child_dir]==false)
			{
				nodes[count].in=in;
				nodes[count].level_num=1;
				nodes[count].out=dest_child_loc;
				nodes[count].rtr_num=rtr_num;
				fin_res=true;
			}
		}
	}
	if(fin_res)
		return true;
	else
		return false;
}
bool FTree_Network::find_path_upper(int lvl,int rtr_num,int in,int src,int dest,int anc_lvl,int src_loc,int dest_loc,int src_child_loc,int dest_child_loc,path_node* nodes,int arr_size,int count,bool turned,int src_child_dir,int dest_child_dir,bool& fin_res)
{
	if(lvl==1 && rtr_num%2==0 && rtr_num==src_loc && src_child_loc==3)
	{
		if(rtr_1st_level[src_loc].prt_taken[1][1]==false && rtr_1st_level[src_loc].prt_taken[src_child_loc][src_child_dir]==false)//go up prt 1
		{
			nodes[count].in=src_child_loc;
			nodes[count].level_num=1;
			nodes[count].out=1;
			nodes[count].rtr_num=src_loc;
			find_path_upper(2,rtr_1st_level[src_loc].nxt_rtr[1]->rtr_num,3,src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
		}
		else
			fin_res=false;
	}
	else if(lvl==1 && rtr_num%2!=0 && rtr_num==src_loc && src_child_loc==2)//go up prt 0
	{
		if(rtr_1st_level[src_loc].prt_taken[0][1]==false && rtr_1st_level[src_loc].prt_taken[src_child_loc][src_child_dir]==false)
		{
			nodes[count].in=src_child_loc;
			nodes[count].level_num=1;
			nodes[count].rtr_num=src_loc;
			nodes[count].out=0;
			find_path_upper(2,rtr_1st_level[src_loc].nxt_rtr[0]->rtr_num,2,src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
		}
		else
			fin_res=false;
	}
	else if(!turned)
	{
		if(lvl!=anc_lvl)
		{
			if(up_rtr[lvl-2][rtr_num].prt_taken[1][1]==false && fin_res==false)//go to port 1
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=1;
				nodes[count].rtr_num=rtr_num;
				find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[1]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[1]->rtr_num,up_rtr[lvl-2][rtr_num].prt[1],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
			}
			if(up_rtr[lvl-2][rtr_num].prt_taken[0][1]==false && fin_res==false)//go to port 0
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=0;
				nodes[count].rtr_num=rtr_num;
				find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[0]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[0]->rtr_num,up_rtr[lvl-2][rtr_num].prt[0],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,false,src_child_dir,dest_child_dir,fin_res);
			}
		}
		else if(lvl==anc_lvl)
		{
			bool other_side;
			find_ancestor_level(src,dest,other_side);
			if(!other_side)
			{
				turned=true;
				if(find_child_path(up_rtr[anc_lvl-2][rtr_num],dest,2))
				{
					if(up_rtr[anc_lvl-2][rtr_num].prt_taken[2][0]==false)//go prt 2
					{
						nodes[count].in=in;
						nodes[count].level_num=anc_lvl;
						nodes[count].out=2;
						nodes[count].rtr_num=rtr_num;
						find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[2]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[2]->rtr_num,up_rtr[lvl-2][rtr_num].prt[2],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
					}
				}
				else if(find_child_path(up_rtr[anc_lvl-2][rtr_num],dest,3))//go prt 3
				{
					if(up_rtr[anc_lvl-2][rtr_num].prt_taken[3][0]==false)
					{
						nodes[count].in=in;
						nodes[count].level_num=anc_lvl;
						nodes[count].out=3;
						nodes[count].rtr_num=rtr_num;
						find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[3]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[3]->rtr_num,up_rtr[lvl-2][rtr_num].prt[3],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
					}
				}
			}
			else
			{
				turned=true;
				if(up_rtr[anc_lvl-2][rtr_num].prt_taken[1][1]==false && fin_res==false)//go to port 1
				{
					nodes[count].in=in;
					nodes[count].level_num=anc_lvl;
					nodes[count].out=1;
					nodes[count].rtr_num=rtr_num;
					find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[1]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[1]->rtr_num,up_rtr[lvl-2][rtr_num].prt[1],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
				}
				if(up_rtr[anc_lvl-2][rtr_num].prt_taken[0][1]==false && fin_res==false)//go to port 0
				{
					nodes[count].in=in;
					nodes[count].level_num=anc_lvl;
					nodes[count].out=0;
					nodes[count].rtr_num=rtr_num;
					find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[0]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[0]->rtr_num,up_rtr[lvl-2][rtr_num].prt[0],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
				}
			}
		}
	}
	else if(turned)
	{
		if(lvl!=1)
		{
			if(find_child_path(up_rtr[lvl-2][rtr_num],dest,2) && up_rtr[lvl-2][rtr_num].prt_taken[2][0]==false)//go port 2
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=2;
				nodes[count].rtr_num=rtr_num;
				find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[2]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[2]->rtr_num,up_rtr[lvl-2][rtr_num].prt[2],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
			}
			else if(find_child_path(up_rtr[lvl-2][rtr_num],dest,3) && up_rtr[lvl-2][rtr_num].prt_taken[3][0]==false)//go port 3
			{
				nodes[count].in=in;
				nodes[count].level_num=lvl;
				nodes[count].out=3;
				nodes[count].rtr_num=rtr_num;
				find_path_upper(up_rtr[lvl-2][rtr_num].nxt_rtr[3]->level,up_rtr[lvl-2][rtr_num].nxt_rtr[3]->rtr_num,up_rtr[lvl-2][rtr_num].prt[3],src,dest,anc_lvl,src_loc,dest_loc,src_child_loc,dest_child_loc,nodes,arr_size,++count,true,src_child_dir,dest_child_dir,fin_res);
			}
		}
		else if(lvl==1 && rtr_num==dest_loc && (dest_child_loc==2 || dest_child_loc==3))//destination
		{
			if(rtr_1st_level[rtr_num].prt_taken[dest_child_loc][dest_child_dir]==false)
			{
				nodes[count].in=in;
				nodes[count].level_num=1;
				nodes[count].out=dest_child_loc;
				nodes[count].rtr_num=rtr_num;
				fin_res=true;
			}
		}
	}
	if(fin_res)
		return true;
	else
		return false;
}

void FTree_Network::crosstalk_FTree(long double &noise,long double &coh_xtalk,long double &pow_loss,int wvlgth_no,int wdm_no)
{
	string dummy;
	string* str;
	int total_num_elements=nodes_on_the_path[0].net_router->router.get_total_number();
	int counter=0;
	int src,des,crossing_num,bending_num;

	crosstalk_str= new string [num_cores*total_num_elements];
	for(int i=0;i<num_cores*total_num_elements;i++)
		crosstalk_str[i]="$";

	results_crosstalk.open("Crosstalk_Noise_Power_Equation.txt",ios::out);
	noise=0;
	long double crosstalk_noise_pow=1;
	pow_loss=1;
	string crosstalk_coef;
	int com_el;	
	bool received=false;
	str=new string[total_num_elements];

	double MR_D=nodes_on_the_path[0].net_router->MR_Dimension;
	double Prop_loss=nodes_on_the_path[0].net_router->Lp;
	double WG_w=nodes_on_the_path[0].net_router->WGD_width;

	//Save the main path information
	ft_router *saved_path=new ft_router[routers_on_the_link];
	for(int i=0;i<routers_on_the_link;i++)
		saved_path[i].net_router=new Router_Analyzer(-1,-1);
	for(int i=0,j=0;i<routers_on_the_link;i++)
	{
		saved_path[i].in=nodes_on_the_path[i].in;
		saved_path[i].out=nodes_on_the_path[i].out;
		saved_path[i].level_num=nodes_on_the_path[i].level_num;
		saved_path[i].rtr_num=nodes_on_the_path[i].rtr_num;
		saved_path[i].crs=nodes_on_the_path[i].crs;
		saved_path[i].rtr_net_side=nodes_on_the_path[i].rtr_net_side;
		while(nodes_on_the_path[i].net_router->res.path[j]!=-1)
		{
			saved_path[i].net_router->res.path[j]=nodes_on_the_path[i].net_router->res.path[j];
			++j;
		}
		j=0;
		while(nodes_on_the_path[i].net_router->res.PL[j]!="$")
		{
			saved_path[i].net_router->res.PL[j]=nodes_on_the_path[i].net_router->res.PL[j];
			++j;
		}
		j=1;
		while(nodes_on_the_path[i].net_router->res.cur_port[j]!=-1)
		{
			saved_path[i].net_router->res.cur_port[j]=nodes_on_the_path[i].net_router->res.cur_port[j];
			j++;
		}
		j=0;
	}
	result saved_PL;
	saved_PL.Link_path=new int[routers_on_the_link*total_num_elements];
	saved_PL.Link_Power_loss=new string[routers_on_the_link*total_num_elements];
	for(int i=0;i<routers_on_the_link*total_num_elements;i++)
	{
		saved_PL.Link_Power_loss[i]=PL.Link_Power_loss[i];
		saved_PL.Link_path[i]=PL.Link_path[i];
	}	
	double saved_wg_net=wg_net.get_length();
	int saved_num_crs=nodes_on_the_path[routers_on_the_link-1].crs;
	int saved_num_ben=routers_on_the_link*2;
	int saved_routers_on_the_link=routers_on_the_link;
	////////////////////////////////////////////////////////////////////////////////////////////////////
	//Copy the power loss results to the output textfiles
	results_power_loss.open("Signal_Power_Equation.txt",ios::out);
	if(results_power_loss.is_open())
	{
		//Add the power loss from the Modulator, please see the document for more information
		pow_loss*=nodes_on_the_path[0].net_router->input_power*pow(nodes_on_the_path[0].net_router->Lp1,wdm_no-wvlgth_no+1)*nodes_on_the_path[0].net_router->Lp2*pow(nodes_on_the_path[0].net_router->Lb,2);//Modulation loss, please see the document
		results_power_loss<<"Lp2 * Lb^2 * Pin * Lp1^"<<wdm_no-wvlgth_no+1<<"@ *";
		saved_wg_net+=8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width;//Approximate modulation propagation loss (see the document)
		
		for(int i=0;i<saved_routers_on_the_link;i++)
			for(int j=0;j<total_num_elements;j++)
			{
				string dummy=saved_path[i].net_router->get_res().PL[j];
				if(dummy=="$")
					break;
				else if(dummy!="P0_1" && dummy!="P0_2" && dummy!="P0_3" && dummy!="P0_4" && dummy!="Wgd" && dummy!="P0_0")
				{
					results_power_loss<<dummy<<" * ";
					//if(dummy=="P0_0")
						//pow_loss*=nodes_on_the_path[0].net_router->input_power;
					if(dummy=="Lc")
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
					else if(dummy=="Lb")
						pow_loss*=nodes_on_the_path[0].net_router->Lb;
					else if(dummy=="(K1)" || dummy=="K1")
						pow_loss*=nodes_on_the_path[0].net_router->K1;
					else if(dummy=="Kr")
						pow_loss*=nodes_on_the_path[0].net_router->Kr;
					else if(dummy=="Kt")
						pow_loss*=nodes_on_the_path[0].net_router->Kt;
					else if(dummy=="Lp1")
						pow_loss*=nodes_on_the_path[0].net_router->Lp1;
					else if(dummy=="Lp2")
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
					else if(dummy=="(Lc Lp2 Lc)")
						pow_loss*=nodes_on_the_path[0].net_router->full_Lc2;
					else if(dummy=="(K2)" || dummy=="K2")
						pow_loss*=nodes_on_the_path[0].net_router->K2;
					else if(dummy=="(K3)" || dummy=="K3")
						pow_loss*=nodes_on_the_path[0].net_router->K3;
					else if(dummy=="Lc1")
						pow_loss*=nodes_on_the_path[0].net_router->Lc1;
					else if(dummy=="Lc2")
						pow_loss*=nodes_on_the_path[0].net_router->Lc2;
					else if(dummy.compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("(Lc Lp2 Lc) Lp1^")+16+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("(Lc Lp2 Lc) Lp1^")+16,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,8,"Lp2 Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp2 Lp1^")+8+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp2 Lp1^")+8,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lp2;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,7,"Lc Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lc Lp1^")+7+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lc Lp1^")+7,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=nodes_on_the_path[0].net_router->Lc;
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
					else if(dummy.compare(0,4,"Lp1^")==0)
					{
						int lc_aux=0;
						string dum_aux;
						while(dummy[dummy.find("Lp1^")+4+lc_aux]!='@')
							lc_aux++;
						dum_aux.assign(dummy,dummy.find("Lp1^")+4,lc_aux);
						int exponent=atoi(dum_aux.c_str());
						pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
					}
				}
			}
			//bendings
			pow_loss*=pow(nodes_on_the_path[0].net_router->Lb,2*saved_routers_on_the_link);
			results_power_loss<<"Lb^("<<2*saved_routers_on_the_link<<") * ";
			//Crossing
			pow_loss*=pow(nodes_on_the_path[0].net_router->Lc,saved_path[saved_routers_on_the_link-1].crs);
			results_power_loss<<"Lc^("<<saved_path[saved_routers_on_the_link-1].crs<<") * ";
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////

	//Coherent noise
	coh_xtalk=0;
	for(int i=0;i<routers_on_the_link;i++)
		coh_xtalk+=nodes_on_the_path[i].net_router->coh_noise;
	//add the coherent crosstalk at modulation 
	coh_xtalk+=pow(nodes_on_the_path[0].net_router->K2,2)/nodes_on_the_path[0].net_router->Lp1;
	for(int i=wvlgth_no+1;i<=wdm_no;i++)
		coh_xtalk+=(nodes_on_the_path[0].net_router->K2*
		MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1);
	//add the coherent crosstalk at detection
	for(int i=1;i<wvlgth_no;i++)
		coh_xtalk+=nodes_on_the_path[0].net_router->K2*
		MR_Drop_Noise((wvlgth_no-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		(i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range,
		((i-1)*nodes_on_the_path[0].net_router->FSR/wdm_no+nodes_on_the_path[0].net_router->MR_wvlgth_range)/(2*nodes_on_the_path[0].net_router->MR_Q))/nodes_on_the_path[0].net_router->Lp1;
	///////////////////////////////////////////////////////////////////////////////////

	//Reading the communication pattern
	net_config.open("Network_Configuration.txt",ios::in);
	if(!net_config.is_open())
	{
		cout<<"\nThe Network_Configuration.txt file is not open!\n";
		exit(1);
	}
	else
	{
		getline(net_config,dummy);
		while(dummy.compare(0,17,"com_pattern start")!=0 && !net_config.eof())
			getline(net_config,dummy);

		double w_length;

		while(!net_config.eof())
		{
			getline(net_config,dummy);
			while((dummy.length()!=0 && dummy[0]=='/' && dummy[1]=='/') || dummy.length()==0 && !net_config.eof()) //Skip the commments and empty lines
				getline(net_config,dummy);
			if(net_config.eof() || dummy=="com_pattern end")
				break;
			find_src_dest_ftree(src,des,dummy); //Find the sender and receiver defined by the user
			received=false;
			wg_net.set_length(0);
			crossing_num=bending_num=0;
			Power_loss_FTree(src,des,crossing_num,bending_num,received,wvlgth_no,wdm_no);
			//if not dropped??
			//Find common nodes
			if(received)
			{
				for(int j=0;j<saved_routers_on_the_link;j++) //j is the counter for the main link
					for(int k=0;k<routers_on_the_link;k++) //k is the counter for the noise inroducing link
						if(saved_path[j].level_num==nodes_on_the_path[k].level_num && saved_path[j].rtr_num==nodes_on_the_path[k].rtr_num && 
							(saved_path[j].rtr_net_side==nodes_on_the_path[k].rtr_net_side || (saved_path[j].level_num==1 && nodes_on_the_path[k].level_num==1))) //Common Node
						{
							//Find the common element in the router structure
							for(int m=0;m<total_num_elements;m++)//main link
								for(int n=0;n<total_num_elements;n++)//noise introducing link
									if(saved_path[j].net_router->get_res().path[m]==nodes_on_the_path[k].net_router->get_res().path[n]) //The common element in the router structure
									{
										if(saved_path[j].net_router->get_res().path[m]==-1 || nodes_on_the_path[k].net_router->get_res().path[n]==-1)
											break;
										else
										{
											//Need to consider the common element type (CSE, PSE, or Crossing) and finally assign the crosstalk coefficient for it
											com_el=nodes_on_the_path[k].net_router->get_res().path[n];

											for(int c=0;c<total_num_elements;c++)
												str[c]="$";

											for(int c=0;c<num_cores*total_num_elements;c++)
												crosstalk_str[c]="$";

											w_length=0;
											counter=0;
											crosstalk_noise_pow=1; 
											if(nodes_on_the_path[k].net_router->router.elm[com_el].type==2) //The common element is crossing
											{
												crosstalk_coef="(K1)";
												w_length+=2*MR_D+WG_w;
											}
											else if(nodes_on_the_path[k].net_router->router.elm[com_el].type==5) //The common element is CSE and using a Single Wavelength
											{
												if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
												{
													if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
															crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
															crosstalk_coef="(K1 + Lc Lc K2)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) //2D
															crosstalk_coef="(Lp1 Lp1 K1 + K2)";	//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";	
														//propagation loss
														w_length+=2*MR_D+WG_w;
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
															crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
															crosstalk_coef="(K1 + Lc Lc K2)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
															crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
														//propagation loss
														w_length+=2*MR_D+WG_w;
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
															crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
															crosstalk_coef="(K1 + Lc Lc K2)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
															crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
														//propagation loss
														w_length+=2*MR_D+WG_w;
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
															crosstalk_coef="(Lp1 K1)";//crosstalk_coef="(Lp1 K1 + Lp1 K1 K2 K1)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
															crosstalk_coef="(K1 + Lc Lc K2)";
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
															crosstalk_coef="(Lp1 Lp1 K1 + K2)";//crosstalk_coef="(Lp1 Lp1 K1 + Lp1 K1 K2 K1 Lp1 + K2)";
														//propagation loss
														w_length+=2*MR_D+WG_w;
														//cout<<"\nHere:"<<nodes_on_the_path[k].net_router->get_res().cur_port[n+1]<<endl;
													}
												}
												else // If the MR is ON in the CSE
												{
													if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1 || 
														nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 1 or 4
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
														{
															crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
															//propagation loss
															w_length+=6*MR_D+2*WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1)) 
														{
															crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
															//propagation loss
															w_length+=6*MR_D+3*WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2 || 
														nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 2 or 3
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) 
														{
															crosstalk_coef="(K1 K3 + Lc Lp2 Kr K3)";
															//propagation loss
															w_length+=6*MR_D+2*WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || 
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) 
														{
															crosstalk_coef="(Lc K3 + Lc Lp2 K1 K3)";
															//propagation loss
															w_length+=6*MR_D+3*WG_w;
														}
													}
												}
											}
											else if(nodes_on_the_path[k].net_router->router.elm[com_el].type==5 && wdm_no>1) //The common element is CSE and Using WDM
											{
												if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR()!=1) //If the MR is OFF
												{
													if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3)) //2D
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
															crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
															crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
															crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*wdm_no*MR_D+MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
															crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
															crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*wdm_no*MR_D+MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
															crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2)) //2D
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,1,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=1
															crosstalk_coef="Func_no=1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
															crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,3,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=3
															crosstalk_coef="Func_no=3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*wdm_no*MR_D+MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,2,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=2
															crosstalk_coef="Lp1^(wdm_no) * K1";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+2*MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,4,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=4
															crosstalk_coef="Func_no=4";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*wdm_no*MR_D+MR_D+WG_w;
														}
														else if((saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,5,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=5
															crosstalk_coef="Func_no=5";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
															//propagation loss (estimation for WDM case in this case)
															w_length+=2*MR_D+WG_w;
														}
													}
												}
												else // If the MR is ON in the CSE, consider first-order crosstalk noise only
												{
													if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==1) //if the MR is in the region 1 
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
															crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==2) //if the MR is in the region 2 
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
															crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==3) //if the MR is in the region 3 
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
															crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+MR_D+WG_w;
														}
													}
													else if(nodes_on_the_path[k].net_router->router.cse[com_el].get_CSE_MR_Loc()==4) //if the MR is in the region 4 
													{
														if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
															(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) || //2D
															(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
														{
															crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,6,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=6
															crosstalk_coef="Lp1^(wdm_no-1) * Lc * K3";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case
															//propagation loss (estimation for WDM case in this case)
															w_length+=wdm_no*MR_D+MR_D+WG_w;
														}
													}
												}
											}
											else if(nodes_on_the_path[k].net_router->router.elm[com_el].type==6) //The common element is PSE and using a Single Wavelength
											{
												if(nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
												{
													if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_coef="(K2)";
														w_length+=4*MR_D+WG_w;
													}
												}
												else //If the MR is ON
												{
													if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_coef="(K3)";
														w_length+=2*MR_D;
													}
												}
											}
											else if(nodes_on_the_path[k].net_router->router.elm[com_el].type==6 && wdm_no>1) //The common element is PSE and using WDM
											{
												if(nodes_on_the_path[k].net_router->router.pse[com_el].get_PSE_MR()!=1) //If the MR is OFF
												{
													if((saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,7,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=7
														crosstalk_coef="Func_no=7";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														w_length+=4*MR_D+2*(wdm_no-wvlgth_no)*MR_D+WG_w;
													}
													else if((saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,8,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=8
														crosstalk_coef="Func_no=8";//This is just to help the user to understand the general formula used for calculating the crosstalk coefficient in this case, please refer to the manual for more information
														w_length+=4*MR_D+2*(wvlgth_no-1)*MR_D+WG_w;
													}

												}
												else //If the MR is ON
												{
													if((saved_path[j].net_router->get_res().cur_port[m+1]==1 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==4) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==2 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==3) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==3 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==2) ||
														(saved_path[j].net_router->get_res().cur_port[m+1]==4 && nodes_on_the_path[k].net_router->get_res().cur_port[n+1]==1))
													{
														crosstalk_noise_pow*=xtalk_coeff_cse(wvlgth_no,wdm_no,9,nodes_on_the_path[j].net_router->K1,nodes_on_the_path[j].net_router->K2,nodes_on_the_path[j].net_router->K3,nodes_on_the_path[j].net_router->Lp1,nodes_on_the_path[j].net_router->Lc,nodes_on_the_path[j].net_router->FSR,nodes_on_the_path[j].net_router->MR_Q,nodes_on_the_path[j].net_router->MR_wvlgth_range);//func_num=9
														crosstalk_coef="Lp1^(wdm_no-1) K3";
														//propagation loss (estimation for WDM case in this case)
														w_length+=(wdm_no+1)*MR_D;
													}
												}
											}

											int p,q;
											for(p=0;p<n+1;p++)
												str[p]=nodes_on_the_path[k].net_router->get_res().PL[p]; //The path before the incident from the link introduction link
											str[p++]=crosstalk_coef; //The crosstalk coefficient
											for(q=m+2;q<total_num_elements && saved_path[j].net_router->get_res().PL[q]!="$";q++)
												str[p++]=saved_path[j].net_router->get_res().PL[q]; //The path after the incident from the main link

											////Calculate the propagation length
											for(int y=j+1;y<saved_routers_on_the_link;y++)
												w_length+=saved_path[y].net_router->w.get_length(); //Consider the propagation loss in the optical routers located on the main link
											//Consider the propagation loss of the main link
											w_length+=(saved_routers_on_the_link+1)*hop_length; 
											//Consider the propagation loss of the noise introduction link
											for(int i=0;i<k;i++)
												w_length+=nodes_on_the_path[i].net_router->w.get_length(); 
											w_length+=(routers_on_the_link+1)*hop_length;

											//Add the propagation loss inside the optical router
											for(int z=0;z<total_num_elements;z++)
												if(str[z]=="$")
													break;
												else
												{
													if(str[z]=="Lc"|| str[z]=="Lc1" || str[z]=="Lc2")
														w_length+=2*MR_D+WG_w;
													else if(str[z]=="Lp1")
														w_length+=2*MR_D;
													else if(str[z]=="Lb")
														w_length+=MR_D+WG_w;
													else if(str[z]=="Lp2" || str[z].compare(0,8,"Lp2 Lp1^")==0)
														w_length+=4*MR_D+WG_w;
													else if(str[z]=="(Lc Lp2 Lc)" || str[z].compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
														w_length+=6*MR_D+3*WG_w;
													else if(str[z].compare(0,7,"Lc Lp1^")==0)
														w_length+=(wdm_no+1)*MR_D+WG_w;
													else if(str[z].compare(0,4,"Lp1^")==0)
														w_length+=(wdm_no+1)*MR_D;
												}
												for(int i=0;i<n;i++) //Noise link
													if(nodes_on_the_path[k].net_router->get_res().PL[i]=="Wgd")
														w_length+=nodes_on_the_path[k].net_router->router.wgd[nodes_on_the_path[k].net_router->get_res().path[i-1]].get_length();
												for(int i=m+1;i<total_num_elements;i++) //Main link
													if(saved_path[j].net_router->get_res().PL[i]=="$")
														break;
													else if(saved_path[j].net_router->get_res().PL[i]=="Wgd")
														w_length+=saved_path[j].net_router->router.wgd[saved_path[j].net_router->get_res().path[i-1]].get_length();

												//Copy the result to the crosstalk_str
												for(int r=0;r<k;r++)
													for(int s=0;s<total_num_elements;s++)
														if(nodes_on_the_path[r].net_router->get_res().PL[s]=="$")
															break;
														else
															crosstalk_str[counter++]=nodes_on_the_path[r].net_router->get_res().PL[s]; //The Power Loss of the noise introduction link before the common node

												for(int t=0;t<total_num_elements;t++)
													if(str[t]=="$")
														break;
													else
														crosstalk_str[counter++]=str[t]; //The Power Loss inside the common node

												for(int u=j+1;u<saved_routers_on_the_link;u++)
													for(int x=0;x<total_num_elements;x++)
														if(saved_path[u].net_router->get_res().PL[x]=="$")
															break;
														else
															crosstalk_str[counter++]=saved_path[u].net_router->get_res().PL[x];

												crosstalk_str[counter++]="+";

												//Wrting the result into the output textfile
												if(results_crosstalk.is_open())
												{
													crosstalk_noise_pow*=nodes_on_the_path[0].net_router->input_power*nodes_on_the_path[0].net_router->Lp1*nodes_on_the_path[0].net_router->Lp2;
													results_crosstalk<<"Lp1 Lp2 * Pin * ";//for modulation
													w_length+=8*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width;//modulation propagation loss
													
													int i=0;
													while(crosstalk_str[i]!="$")
													{
														if(crosstalk_str[i]!="+" && crosstalk_str[i]!="P0_1" && crosstalk_str[i]!="P0_2" && crosstalk_str[i]!="P0_3" && crosstalk_str[i]!="P0_4" && crosstalk_str[i]!="Wgd" && crosstalk_str[i]!="P0_0")
														{
															results_crosstalk<<crosstalk_str[i]<<" * ";
															//if(crosstalk_str[i]=="P0_0")
																//crosstalk_noise_pow*=nodes_on_the_path[0].net_router->input_power;
															if(crosstalk_str[i]=="Lc")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
															else if(crosstalk_str[i]=="Lb")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lb;
															else if(crosstalk_str[i]=="(K1)" || crosstalk_str[i]=="K1")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K1;
															else if(crosstalk_str[i]=="Kr")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kr;
															else if(crosstalk_str[i]=="Kt")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Kt;
															else if(crosstalk_str[i]=="Lp1")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp1;
															else if(crosstalk_str[i]=="Lp2")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
															else if(crosstalk_str[i]=="(K2)" || crosstalk_str[i]=="K2")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K2;
															else if(crosstalk_str[i]=="(K3)" || crosstalk_str[i]=="K3")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->K3;
															else if(crosstalk_str[i]=="Lc1")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc1;
															else if(crosstalk_str[i]=="Lc2")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc2;
															else if(crosstalk_str[i].compare(0,16,"(Lc Lp2 Lc) Lp1^")==0)
															{
																int lc_aux=0;
																string dum_aux;
																while(crosstalk_str[i][crosstalk_str[i].find("(Lc Lp2 Lc) Lp1^")+16+lc_aux]!='@')
																	lc_aux++;
																dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("(Lc Lp2 Lc) Lp1^")+16,lc_aux);
																int exponent=atoi(dum_aux.c_str());
																crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lc,2)*nodes_on_the_path[0].net_router->Lp2;
																crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
															}
															else if(crosstalk_str[i].compare(0,8,"Lp2 Lp1^")==0)
															{
																int lc_aux=0;
																string dum_aux;
																while(crosstalk_str[i][crosstalk_str[i].find("Lp2 Lp1^")+8+lc_aux]!='@')
																	lc_aux++;
																dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp2 Lp1^")+8,lc_aux);
																int exponent=atoi(dum_aux.c_str());
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lp2;
																crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
															}
															else if(crosstalk_str[i].compare(0,7,"Lc Lp1^")==0)
															{
																int lc_aux=0;
																string dum_aux;
																while(crosstalk_str[i][crosstalk_str[i].find("Lc Lp1^")+7+lc_aux]!='@')
																	lc_aux++;
																dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lc Lp1^")+7,lc_aux);
																int exponent=atoi(dum_aux.c_str());
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->Lc;
																crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
															}
															else if(crosstalk_str[i].compare(0,4,"Lp1^")==0)
															{
																int lc_aux=0;
																string dum_aux;
																while(crosstalk_str[i][crosstalk_str[i].find("Lp1^")+4+lc_aux]!='@')
																	lc_aux++;
																dum_aux.assign(crosstalk_str[i],crosstalk_str[i].find("Lp1^")+4,lc_aux);
																int exponent=atoi(dum_aux.c_str());
																crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lp1,exponent);
															}
															else if(crosstalk_str[i]=="(Lp1 K1)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_1;
															else if(crosstalk_str[i]=="(K1 + Lc Lc K2)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_2;
															else if(crosstalk_str[i]=="(Lp1 Lp1 K1 + K2)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_3;
															else if(crosstalk_str[i]=="(K1 K3 + Lc Lp2 Kr K3)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_4;
															else if(crosstalk_str[i]=="(Lc K3 + Lc Lp2 K1 K3)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->cr_coef_5;
															else if(crosstalk_str[i]=="(Lc Lp2 Lc)")
																crosstalk_noise_pow*=nodes_on_the_path[0].net_router->full_Lc2;
														}
														else if(crosstalk_str[i]=="+")
														{
															//Bending
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lb,((2*k+1)+(2*(saved_routers_on_the_link-j)-1)));
															results_crosstalk<<"Lb^("<<((2*k+1)+(2*(saved_routers_on_the_link-j)-1))<<") * ";
															//Crossing
															crosstalk_noise_pow*=pow(nodes_on_the_path[0].net_router->Lc,(nodes_on_the_path[k-1].crs+(saved_path[saved_routers_on_the_link-1].crs-saved_path[j-1].crs)));
															results_crosstalk<<"Lc^("<<(nodes_on_the_path[k].crs+(saved_path[saved_routers_on_the_link-1].crs-saved_path[j].crs))<<") * ";
															
															//cout<<"\nLc numbers "<<(nodes_on_the_path[k].crs+(saved_path[saved_routers_on_the_link-1].crs-saved_path[j].crs))<<endl;

															//Propagation Loss
															results_crosstalk<<"Lp^("<<w_length*0.0001<<")";
															crosstalk_noise_pow*=pow(Prop_loss,w_length*0.0001);
															
															results_crosstalk<<" + ";
															noise+=crosstalk_noise_pow;
															crosstalk_noise_pow=1;
														}
														i++;
													}
												}
										}
									}
						}
			}
		}
	}
	net_config.close();
	
	//power loss for noise power at detectors
        noise*=nodes_on_the_path[0].net_router->Ld1[wvlgth_no-1]*nodes_on_the_path[0].net_router->Lp2; //Matrix method replaced
	//noise*=pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*nodes_on_the_path[0].net_router->Lp2;
	noise*=pow(Prop_loss,((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001);
	results_crosstalk<<" * (Lp2 Lp1^"<<wvlgth_no-1<<"@) * Lp^("<<((3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width)*0.0001<<")";
	
	//Add the crosstalk noise at the detectors
	if(wdm_no>1)//if using WDM
	{
		noise+=pow_loss*nodes_on_the_path[0].net_router->Ld1[wvlgth_no-1]*xtalk_coeff_cse(wvlgth_no,wdm_no,10,nodes_on_the_path[0].net_router->K1,nodes_on_the_path[0].net_router->K2,nodes_on_the_path[0].net_router->K3,nodes_on_the_path[0].net_router->Lp1,nodes_on_the_path[0].net_router->Lc,nodes_on_the_path[0].net_router->FSR,nodes_on_the_path[0].net_router->MR_Q,nodes_on_the_path[0].net_router->MR_wvlgth_range);//func_num=10
		results_crosstalk<<"* Func_no=10 * ";
	}

	//Add the detector at the end of the communication line
	pow_loss*=pow(nodes_on_the_path[0].net_router->Lp1,wvlgth_no-1)*nodes_on_the_path[0].net_router->Lp2;
	results_power_loss<<"* Lp2 Lp1^"<<wvlgth_no-1<<"@ * ";
	saved_wg_net+=(3+wvlgth_no)*nodes_on_the_path[0].net_router->MR_Dimension+nodes_on_the_path[0].net_router->WGD_width;
	results_power_loss<<"Lp^("<<saved_wg_net*0.0001<<")"<<"\n**END**";
	pow_loss*=pow(Prop_loss,saved_wg_net*0.0001);
	results_power_loss.close();
	results_crosstalk<<"\n**END**";
	results_crosstalk.close();
	delete [] str;
	delete [] crosstalk_str;
}

//Number of Crossings in Fat-Tree
int FTree_Network::FT_num_crossings(ft_rtr_node rtr1,ft_rtr_node rtr2,int prt)
{
	if((rtr1.level==1 && rtr2.level==2) || (rtr1.level==2 && rtr2.level==1))//No crossing between level 1 and 2
		return 0;
	if(rtr1.level==num_level && rtr2.level==num_level && rtr1.level==rtr2.level && (rtr1.loc==rtr2.loc))//No crossing among the last level rouerts
		return 0;
	if(abs(rtr1.level-rtr2.level)>1)//We would like to calculate the number of crossings between two sequential levels
		return -1;
	if(rtr1.level<1 || rtr2.level<1)//The level number is wrong
		return -1;
	//The path is always from rtr1 to rtr2. We need to consider any contradiction between this path and direction.
	if(rtr1.level<rtr2.level && direction==0 && rtr1.loc==1 && rtr2.loc==1)//Direction should be 1 (Upward)
		return -1;
	if(rtr1.level>rtr2.level && direction==1 && rtr1.loc==1 && rtr2.loc==1)//Direction should be 1 (Upward)
		return -1;
	if(rtr1.level<rtr2.level && direction==1 && rtr1.loc==0 && rtr2.loc==0)//Direction should be 1 (Upward)
		return -1;
	if(rtr1.level>rtr2.level && direction==0 && rtr1.loc==0 && rtr2.loc==0)//Direction should be 1 (Upward)
		return -1;
	//OTHERWISE
	if(rtr1.loc==1 && rtr2.loc==1)//Upper Network
	{
		if(direction==1)//Upward direction
		{
			if(prt==0)//The number of crossings at port 0 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num>rtr1.rtr_num)//Go right
					return 4*(rtr2.rtr_num-rtr1.rtr_num);
				else//Go left
					return 4*(rtr1.rtr_num-rtr2.rtr_num)-2;
			}
			else if(prt==1)//The number of crossings at port 1 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num>rtr1.rtr_num)//Go right
					return 4*(rtr2.rtr_num-rtr1.rtr_num)-2;
				else//Go left
					return 4*(rtr1.rtr_num-rtr2.rtr_num);
			}
		}
		else if(direction==0)//Downward direction
		{
			if(prt==2)//The number of crossings at port 2 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr1.rtr_num<rtr2.rtr_num)//Go right
				{
					if(rtr1.rtr_num%2==0)
						return 4*(rtr2.rtr_num-rtr1.rtr_num)-2;
					else
						return 4*(rtr2.rtr_num-rtr1.rtr_num);
				}
			}
			else if(prt==3)//The number of crossings at port 3 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num<rtr1.rtr_num)//Go left
				{
					if(rtr1.rtr_num%2==0)
						return 4*(rtr1.rtr_num-rtr2.rtr_num);
					else
						return 4*(rtr1.rtr_num-rtr2.rtr_num)-2;
				}
			}
		}
	}
	else if(rtr1.loc==0 && rtr2.loc==0)//Down Network
	{
		if(direction==1)//Upward direction
		{
			if(prt==2)//The number of crossings at port 2 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num<rtr1.rtr_num)//Go left
				{
					if(rtr1.rtr_num%2==0)
						return 4*(rtr1.rtr_num-rtr2.rtr_num);
					else
						return 4*(rtr1.rtr_num-rtr2.rtr_num)-2;
				}
			}
			else if(prt==3)//The number of crossings at port 3 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr1.rtr_num<rtr2.rtr_num)//Go right
				{
					if(rtr1.rtr_num%2==0)
						return 4*(rtr2.rtr_num-rtr1.rtr_num)-2;
					else
						return 4*(rtr2.rtr_num-rtr1.rtr_num);
				}
			}
		}
		else if(direction==0)//Downward direction
		{
			if(prt==1)//The number of crossings at port 1 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num>rtr1.rtr_num)//Go right
					return 4*(rtr2.rtr_num-rtr1.rtr_num);
				else//Go left
					return 4*(rtr1.rtr_num-rtr2.rtr_num)-2;
			}
			else if(prt==0)//The number of crossings at port 0 of rtr1
			{
				if(rtr1.rtr_num==rtr2.rtr_num)//Same place
					return 0;
				else if(rtr2.rtr_num>rtr1.rtr_num)//Go right
					return 4*(rtr2.rtr_num-rtr1.rtr_num)-2;
				else//Go left
					return 4*(rtr1.rtr_num-rtr2.rtr_num);
			}
		}
	}
	return -1;
}
void FTree_Network::copy_two_rtrs(ft_rtr_node &rtr1,ft_rtr_node rtr2)
{
	rtr1.child[0]=rtr2.child[0];
	rtr1.child[1]=rtr2.child[1];
	rtr1.level=rtr2.level;
	rtr1.loc=rtr2.loc;
	for(int i=0;i<4;i++)
	{
		rtr1.nxt_rtr[i]=rtr2.nxt_rtr[i];
		rtr1.prt[i]=rtr2.prt[i];
		rtr1.prt_taken[i][0]=rtr2.prt_taken[i][0];
		rtr1.prt_taken[i][1]=rtr2.prt_taken[i][1];
	}
	rtr1.rtr_num=rtr2.rtr_num;
}
bool FTree_Network::find_child_path(ft_rtr_node cur_rtr,int dest,int prt_num)//Return -1, if the router is not the common ancestor, otherwise, return the proper port number
{
	if(cur_rtr.level!=2)
	{
		int child_level=cur_rtr.nxt_rtr[prt_num]->level;
		int dummy=cur_rtr.nxt_rtr[prt_num]->rtr_num/int(pow(double(2),child_level-2));
		int strt=dummy*int(pow(double(2),child_level-2))*4;
		if(dest<(strt+int(pow(double(2),child_level))) && (dest>strt || dest==strt))
			return true;
		return false;
	}
	else if(cur_rtr.level==2)
	{
		if(cur_rtr.nxt_rtr[prt_num]->child[0]==dest || cur_rtr.nxt_rtr[prt_num]->child[1]==dest)
			return true;
		return false;
	}
}
int FTree_Network::find_ancestor_level(int src,int dest,bool &other_side)
{
	int half=num_cores/2;
	if(abs((src/half)-(dest/half))>0)//Source and Destination are on the two sides of the network
	{
		other_side=true;
		return num_level;
	}
	else
	{
		other_side=false;
		half/=2;
		while((src/half)-(dest/half)==0)
			half/=2;
		if(half==1)//Neighbouring routers
			return 2;
		return int(log10(double(half))/log10(double(2)))+1;;
	}
}
int FTree_Network::match_prt_ftree(int prt,int direc)//Input port (direct=0), Output port (direct=1)
{
	if(prt==0 && direc==1)
		return 1;
	else if(prt==0 && direc==0)
		return 0;
	else if(prt==1 && direc==1)
		return 7;
	else if(prt==1 && direc==0)
		return 6;
	else if(prt==2 && direc==1)
		return 5;
	else if(prt==2 && direc==0)
		return 4;
	else if(prt==3 && direc==1)
		return 3;
	else if(prt==3 && direc==0)
		return 2;
	//Otherwise
	return -1;
}
void FTree_Network::find_src_dest_ftree(int &src, int &des, string str)
{
	string dummy;
	int loc;
	des=src=-1;
	if(str.compare(0,4,"from")==0) //Start of a communication pair
	{
		//find the source
		loc=0;
		while(str[4+loc]!='t')
			loc++;
		dummy.assign(str,4,loc);
		src=atoi(dummy.c_str());
		dummy.erase();

		//find the destination
		loc=0;
		while(str[str.find('t')+2+loc]!=';')
			loc++;
		dummy.assign(str,str.find('t')+2,loc);
		des=atoi(dummy.c_str());
		dummy.erase();
	}
	if(src>num_cores || des>num_cores || src<0 || des<0)
	{
		cout<<"ERROR: from "<<src<<" to "<<des<<" is not valid!\n";
		exit(1);
	}
}
void FTree_Network::SNR_FTree(int src,int dest,char ch,int wdm_no)
{
	int crs,bend;
	bool received=false;
	long double max_xtalk_noise=-1000,max_coh_xtalk=-1000,min_signal_pow=1000,min_SNR=1000;
	int max_xtalk_no=-1, min_signal_pow_no=-1,min_SNR_no=-1,max_coh_xtalk_no=-1;
	bool show_wc_res=true;
	if(src>num_cores || dest>num_cores || src<0 || dest<0)
	{
		cout<<"\nERROR: The source and the destination are not within the defined network size!";
		exit(1);
	}
	if(ch=='s')
	{
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		cout<<"\nThe results for the optical link starts from the core "<<src<<" towards the core "<<dest<<"\n in Fat-tree-based ONoC are as following:\n\n";
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_FTree(src,dest,crs,bend,received,i,1); //Calculate the power loss of the link
			crosstalk_FTree(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files

			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				//Show the results to the user
				cout<<"Wavelength["<<i<<"] Singal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Incoherent Crosstalk Noise Power (at the destination)= "<<crosstalk_noise<<" dBm\n";
				cout<<"Wavelength["<<i<<"] Coherent crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no coherent crosstalk)= "<<SNR_no_coh<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (coherent crosstalk + incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n";
				cout<<"Wavelength["<<i<<"] SNR (singal power - coherent crosstalk)= "<<SNR_SP_coh<<" dB\n\n";
				
				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//cout<<"\nThe results for the optical link starts from the core "<<src<<" towards the core "<<dest<<"\n in Fat-tree-based ONoC are as following:\n\n";
				cout<<"Wavelength["<<i<<"] Signal Power= "<<power_loss<<" dBm\n";
				cout<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				cout<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				cout<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wc_res=true;
			}
			this->build_FTnet();//refresh the network
		}
		//Show the worst-case results for different wavelengths
		if(show_wc_res)
		{
			cout<<endl;
			cout<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			cout<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dBm\n";
			cout<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dBm\n";
			cout<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			cout<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR of "<<min_SNR<<" dB\n\n";
		}
	}
	else if(ch=='f')
	{
		ofstream outfile("Network_Analyzer_Resuls.csv",ios::out);
		if(!outfile.is_open())
		{
			outfile<<"\nERROR: The network_analyzer_results file could not be created!\n";
			exit(1);
		}
		long double crosstalk_noise,power_loss,SNR_no_coh,SNR_ch_inc,SNR_SP_coh,coh_xtalk;
		outfile<<"\nThe results for the optical link starts from the core "<<src<<" towards the core "<<dest<<" in Fat-tree-based ONoC are as following:"<<endl;
		outfile<<"Wavelength, Signal power (dBm), Incoherent crosstalk noise power (dBm), Coherent crosstalk (dBm), SNR (no coherent crosstalk)(dB), SNR (coherent crosstalk + incoherent crosstalk)(dB), SNR (signal power - coherent crosstalk)(dB)"<<endl;
		for(int i=1;i<=wdm_no;i++)
		{
			Power_loss_FTree(src,dest,crs,bend,received,i,1); //Calculate the power loss of the link
			crosstalk_FTree(crosstalk_noise,coh_xtalk,power_loss,i,1); //Calculate the crosstalk noise, write the results to the output files
			if(crosstalk_noise!=0)
			{
				coh_xtalk*=power_loss;
				SNR_no_coh=power_loss/crosstalk_noise; //Calculate the SNR without conisdering the coherent crosstalk 
				SNR_ch_inc=power_loss/(crosstalk_noise+coh_xtalk);//Calculate the SNR with adding the coherent crosstalk and incoherent crosstalk together
				SNR_SP_coh=(power_loss-coh_xtalk)/crosstalk_noise;//Calculate the SNR with substracting the coherent crosstalk from signal power
				power_loss=10*log10(power_loss); //Convert power loss to dB
				SNR_no_coh=10*log10(SNR_no_coh); //Convert SNR to dB
				SNR_ch_inc=10*log10(SNR_ch_inc); //Convert SNR to dB
				SNR_SP_coh=10*log10(SNR_SP_coh); //Convert SNR to dB
				crosstalk_noise=10*log10(crosstalk_noise); //Convert crosstalk noise to dB
				coh_xtalk=10*log10(coh_xtalk);//
				outfile<<i<<","<<power_loss<<","<<crosstalk_noise<<","<<coh_xtalk<<","<<SNR_no_coh<<","<<SNR_ch_inc<<","<<SNR_SP_coh<<endl;

				//Find the worst-case results among different wavelengths
				if(crosstalk_noise>max_xtalk_noise)
				{
					max_xtalk_noise=crosstalk_noise;
					max_xtalk_no=i;
				}
				if(power_loss<min_signal_pow)
				{
					min_signal_pow=power_loss;
					min_signal_pow_no=i;
				}
				if(coh_xtalk>max_coh_xtalk)
				{
					max_coh_xtalk=coh_xtalk;
					max_coh_xtalk_no=i;
				}
				if(SNR_no_coh<min_SNR)
				{
					min_SNR=SNR_no_coh;
					min_SNR_no=i;
				}
			}
			else
			{
				coh_xtalk*=power_loss;
				SNR_ch_inc=power_loss/coh_xtalk;
				SNR_ch_inc=10*log10(SNR_ch_inc);
				power_loss=10*log10(power_loss); //Convert power loss to dB
				coh_xtalk=10*log10(coh_xtalk);
				//Show the results to the user
				//outfile<<"\nThe results for the optical link starts from the core ("<<src<<") towards the core ("<<dest<<")\n in Fat-tree-based ONoC are as followings:\n\n";
				outfile<<"Wavelength["<<i<<"] Signal power= "<<power_loss<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] No Incoherent Crosstalk Noise Power Received at the Destination!\n";
				outfile<<"Wavelength["<<i<<"] Coherent Crosstalk Power (at the destination)= "<<coh_xtalk<<" dBm\n";
				outfile<<"Wavelength["<<i<<"] SNR (no incoherent crosstalk)= "<<SNR_ch_inc<<" dB\n\n";
				show_wc_res=true;
			}
			this->build_FTnet();//refresh the network
		}
		//Show the worst-case results for different wavelengths
		if(show_wc_res)
		{
			outfile<<endl;
			outfile<<"The worst-case results among "<<wdm_no<<" different wavelengths are as the following:\n";
			outfile<<"Wavelength["<<min_signal_pow_no<<"] has the worst-case Singal Power of "<<min_signal_pow<<" dBm\n";
			outfile<<"Wavelength["<<max_xtalk_no<<"] has the worst-case Incoherent Crosstalk Noise Power of "<<max_xtalk_noise<<" dBm\n";
			outfile<<"Wavelength["<<max_coh_xtalk_no<<"] has the worst-case Coherent Crosstalk Power of "<<max_coh_xtalk<<" dBm\n";
			outfile<<"Wavelength["<<min_SNR_no<<"] has the worst-case SNR (no coherent crosstalk is considered) of "<<min_SNR<<" dB\n\n";
		}
		outfile.close();
	}
}



double xtalk_coeff_cse(int wvlgth_no,int wdm_no,int func_no,double K1,double K2,double K3,double Lp1,double Lc,double MR_FSR,double MR_Q,double MR_range)//the func_no determines which sub-function to be used
{
	MR_FSR=MR_FSR/(wdm_no*1.0);//we assume equal spacing among different wavelengths
	double xtalk_coeff=1;
	double shift_to_off=0;//this shift in the resonant wavelength is to turn OFF the MR
	if(func_no==1)//This subfunction is used when the CSE is in OFF state, CSE_MR_Loc=1, Signal_prt=1, Noise_prt=4 (a1)
	{
            xtalk_coeff*=(K1+K2*pow(Lc,2)*Lp1); //Matrix method here
		//xtalk_coeff*=(K1+K2*pow(Lc,2)*pow(Lp1,2*(wdm_no-wvlgth_no)));
		//for(int j=1;j<=wdm_no;j++)
		//	if(j!=wvlgth_no)
		//		xtalk_coeff+=pow(Lc,2)*pow(Lp1,2*(wdm_no-j))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==2)//This subfunction is used when the CSE is in OFF state, CSE_MR_Loc=1, Signal_prt=1, Noise_prt=2 (b1)
	{
            xtalk_coeff*=Lp1*K1; //Matrix method replaced
            //xtalk_coeff*=pow(Lp1,wdm_no)*K1;
	}
	else if(func_no==3)//This subfunction is used when the CSE is in OFF state, CSE_MR_Loc=1, Signal_prt=3, Noise_prt=2 (d1)
	{
            xtalk_coeff*=Lp1*K1+K2*Lp1; //Matrix method replaced
	//	xtalk_coeff*=pow(Lp1,2*wdm_no)*K1+K2*pow(Lp1,2*(wvlgth_no-1));
	//	for(int j=1;j<=wdm_no;j++)
	//		if(j!=wvlgth_no)
	//			xtalk_coeff+=pow(Lp1,2*(j-1))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==4)//This subfunction is used when the CSE is in OFF state, CSE_MR_Loc=2, Signal_prt=4, Noise_prt=3 (b2)
	{
            xtalk_coeff*=pow(Lp1,1)*K1+K2*pow(Lp1,1);  //Matrix method replaced
	//	xtalk_coeff*=pow(Lp1,2*wdm_no)*K1+K2*pow(Lp1,2*(wdm_no-wvlgth_no));
	//	for(int j=1;j<=wdm_no;j++)
	//		if(j!=wvlgth_no)
	//			xtalk_coeff+=pow(Lp1,2*(wdm_no-j))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==5)//This subfunction is used when the CSE is in OFF state, CSE_MR_Loc=2, Signal_prt=2, Noise_prt=1 (c2)
	{
            xtalk_coeff*=K1+K2*pow(Lc,2)*Lp1; //Matrix method replaced
	//	xtalk_coeff*=K1+K2*pow(Lc,2)*pow(Lp1,2*(wvlgth_no-1));
	//	for(int j=1;j<=wdm_no;j++)
	//		if(j!=wvlgth_no)
	//			xtalk_coeff+=pow(Lc,2)*pow(Lp1,2*(j-1))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==6)//This subfunction is used when the CSE is in ON state
	{
            xtalk_coeff*=pow(Lp1,1)*Lc*K3; //Matrix method replaced
	//	xtalk_coeff*=pow(Lp1,wdm_no-1)*Lc*K3;
	}
	else if(func_no==7)//This sub-function is used when the PSE is in OFF state, Signal_prt=2, Noise_prt=3 (Pa)
	{
            xtalk_coeff*=K2*pow(Lp1,1); //Matrix method replaced
	//	xtalk_coeff*=K2*pow(Lp1,2*(wdm_no-wvlgth_no));
	//	for(int j=1;j<=wdm_no;j++)
	//		if(j!=wvlgth_no)
	//			xtalk_coeff+=pow(Lp1,2*(wdm_no-j))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==8)//This subfunction is used when the PSE is in OFF state, Signal_prt=4, Noise_prt=2 (Pb)
	{
            xtalk_coeff*=K2*pow(Lp1,1); //Matrix method replaced
	//	xtalk_coeff*=K2*pow(Lp1,2*(wvlgth_no-1));
	//	for(int j=1;j<=wdm_no;j++)
	//		if(j!=wvlgth_no)
	//			xtalk_coeff+=pow(Lp1,2*(j-1))*MR_Drop_Noise((wvlgth_no-1)*MR_FSR+MR_range,(j-1)*MR_FSR+MR_range+shift_to_off,((j-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	else if(func_no==9)//This subfunction is used when the PSE is in ON state
	{
            xtalk_coeff*=pow(Lp1,1)*K3;
	//	xtalk_coeff*=pow(Lp1,wdm_no-1)*K3;
	}
	else if(func_no==10)//This subfunction is used for the noise at the detectors
	{
            xtalk_coeff=0;
	//	xtalk_coeff=0;
	//	for(int j=wvlgth_no+1;j<=wdm_no;j++)
	//		xtalk_coeff+=MR_Drop_Noise((j-1)*MR_FSR+MR_range,(wvlgth_no-1)*MR_FSR+MR_range,((wvlgth_no-1)*MR_FSR+MR_range)/(2*MR_Q));
	}
	return xtalk_coeff;
}
double MR_Drop_Noise(double wvlngth,double MR_wvlngth,double MR_3dB_BW)
{
	return pow(MR_3dB_BW,2)/(pow(wvlngth-MR_wvlngth,2)+pow(MR_3dB_BW,2));
}
